(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    668951,      17609]
NotebookOptionsPosition[    654813,      17186]
NotebookOutlinePosition[    655191,      17203]
CellTagsIndexPosition[    655148,      17200]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["New Integrals", "Section",
 CellChangeTimes->{{3.661537054551502*^9, 3.661537057062385*^9}}],

Cell[CellGroupData[{

Cell["Initialize", "Subsection",
 CellChangeTimes->{{3.6128003517078953`*^9, 3.6128003598898907`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimpPowerExpand", "[", "x_", "]"}], ":=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"PowerExpand", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.574023493293314*^9, 3.574023502996666*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Combinatorica`\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5790123628033*^9, 3.579012367575576*^9}, 
   3.584065470630303*^9, {3.661536825259873*^9, 3.6615368263979387`*^9}, {
   3.661536946662216*^9, 3.661536948349629*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "compat"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Combinatorica Graph and Permutations functionality has \
been superseded by preloaded functionality. The package now being loaded may \
conflict with this. Please see the Compatibility Guide for details.\"\>"}]], \
"Message", "MSG",
 CellChangeTimes->{3.674840049887134*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ZeroList", "[", "arglist_", "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"arglist", "/.", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"#", "\[Rule]", "0"}], ")"}], "&"}], ",", 
     RowBox[{"Subsets", "[", "arglist", "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579010832240737*^9, 3.579010839546554*^9}, {
  3.57901092222068*^9, 3.579011013531438*^9}, {3.5790110591063747`*^9, 
  3.5790110840218887`*^9}, {3.579011266832509*^9, 3.579011485515212*^9}, {
  3.579011854148758*^9, 3.579011867482092*^9}, {3.57901193206878*^9, 
  3.5790119883315287`*^9}, {3.579012658529066*^9, 3.579012682209354*^9}, {
  3.5790127837545023`*^9, 3.579012784160153*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"EqualList", "[", "arglist_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"Flatten", "[", "#", "]"}], "&"}], "/@", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "\[Rule]", 
         RowBox[{"#", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", 
          RowBox[{"Length", "[", "#", "]"}]}], "}"}]}], "]"}], "&"}], ",", 
     RowBox[{"SetPartitions", "[", "arglist", "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579012661754188*^9, 3.579012688640642*^9}, 
   3.5790127862655163`*^9, {3.5790137990672817`*^9, 3.579013838511228*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "The", " ", "PaVe", " ", "integrals", " ", "are", " ", "singular", " ", 
    "when", " ", "any", " ", "argument", " ", "is", " ", "zero", " ", "or", 
    " ", "any", " ", "two", " ", "arguments", " ", "are", " ", 
    RowBox[{"equal", ".", " ", "This"}], " ", "function", " ", "generates", 
    " ", "a", " ", "list", " ", "of", " ", "all", " ", "possible", " ", 
    "ways", " ", "that", " ", "an", " ", "argument", " ", "list", " ", "can", 
    " ", "be", " ", 
    RowBox[{"singular", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"SingularityList", "[", "arglist_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"zerolist", "=", 
        RowBox[{"ZeroList", "[", "arglist", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"equallist", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"EqualList", "[", 
           RowBox[{"Select", "[", 
            RowBox[{"#", ",", 
             RowBox[{
              RowBox[{"!", 
               RowBox[{"TrueQ", "[", 
                RowBox[{"#", "\[Equal]", "0"}], "]"}]}], "&"}]}], "]"}], 
           "]"}], "&"}], "/@", "zerolist"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"#", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "/.", 
              RowBox[{"#", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
           RowBox[{"Partition", "[", 
            RowBox[{
             RowBox[{"Riffle", "[", 
              RowBox[{"zerolist", ",", "equallist"}], "]"}], ",", "2"}], 
            "]"}]}], ")"}], ",", "2"}], "]"}]}]}], "\[IndentingNewLine]", 
     "\[IndentingNewLine]", "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.579012702655048*^9, 3.5790128214478083`*^9}, {
   3.579012885296301*^9, 3.57901293891328*^9}, {3.579013000509573*^9, 
   3.579013077439859*^9}, {3.579013131996681*^9, 3.579013201587511*^9}, {
   3.579013231662423*^9, 3.57901332886128*^9}, {3.579013472849806*^9, 
   3.579013526103385*^9}, {3.579013614920434*^9, 3.579013660152219*^9}, 
   3.5790143458638153`*^9, {3.5790144353458138`*^9, 3.579014455253799*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DefineIntegralFunction2", "[", 
   RowBox[{"funcname_", ",", "funcexpr_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"singlist", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"SingularityList", "[", 
        RowBox[{"{", 
         RowBox[{"m1temp", ",", "m2temp"}], "}"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"funcsinglist", "=", 
      RowBox[{"Collect", "[", 
       RowBox[{
        RowBox[{"Simplify", "[", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"Simplify", "[", 
             RowBox[{"Series", "[", 
              RowBox[{"funcexpr", ",", 
               RowBox[{"{", 
                RowBox[{"m1", ",", 
                 RowBox[{"singlist", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", "0"}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"m2", ",", 
                 RowBox[{"singlist", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "2"}], "]"}], "]"}], ",", "0"}], "}"}]}],
               "]"}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "singlist", "]"}]}], "}"}]}], "]"}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"psq", ",", 
          RowBox[{"Log", "[", "x_", "]"}]}], "}"}], ",", "FullSimplify"}], 
       "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"singlist2", "=", 
      RowBox[{
       RowBox[{"singlist", "/.", 
        RowBox[{"m1temp", "\[Rule]", "m1temp_"}]}], "/.", 
       RowBox[{"m2temp", "\[Rule]", "m2temp_"}]}]}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"funcname", "[", 
         RowBox[{"psqsub_", ",", 
          RowBox[{"singlist2", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"singlist2", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "]"}], ":=", 
        RowBox[{"Evaluate", "[", 
         RowBox[{
          RowBox[{"funcsinglist", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "/.", 
          RowBox[{"psq", "\[Rule]", "psqsub"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "singlist", "]"}]}], "}"}]}], "]"}], ";"}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQ/WydVX3uqleOTCz7poFoXfv6TSDaxvcMmC6dx3QG
RHuxzT0LouM+n7HMA9Iz/i51AtFH9Le6gehJP6/7gujk7RdCQbRMWVMEiK64
YhAHoj/KrYsH0Yycu/JA9MqzLvUg+umja+0g+m5OaheIllul3Q+id8bfnAyi
LXZ7LAXbd//SKhCdMtVwF4iOO+YGplN5ZfeBaIlJIWDa5fKO8h9Ael52RwWI
bo3b0QKiY3rft4NomcX1fSB6zqT9YPqPZO80EL3v7k0wvUlX74DZaqA/uC4c
AtHzjLceA9FP7q4+DqKnLJuqs3L9K8el0Z/1QfSyqwtquTcC3bvyDpgGABnQ
va8=
  "]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"c0", "=", 
     RowBox[{
      RowBox[{"-", "16"}], 
      SuperscriptBox["\[Pi]", "2"]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"dd", "=", 
     RowBox[{"16", 
      SuperscriptBox["\[Pi]", "2"]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ww", "=", 
     RowBox[{"-", "1"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"subtry", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"b0", "\[Rule]", 
       RowBox[{"ww", " ", "c0"}]}], ",", 
      RowBox[{"b1", "\[Rule]", 
       RowBox[{"-", "c0"}]}], ",", 
      RowBox[{"c1", "\[Rule]", "c0"}], ",", 
      RowBox[{"c00", "\[Rule]", 
       RowBox[{"-", "c0"}]}], ",", 
      RowBox[{"c11", "\[Rule]", "c0"}], ",", 
      RowBox[{"c12", "\[Rule]", "c0"}], ",", 
      RowBox[{"c22", "\[Rule]", "c0"}], ",", 
      RowBox[{"d1", "\[Rule]", "dd"}], ",", 
      RowBox[{"d2", "\[Rule]", "dd"}], ",", 
      RowBox[{"d3", "\[Rule]", "dd"}]}], "}"}]}]}]}]], "Input",
 CellChangeTimes->{{3.575042954867443*^9, 3.5750429854586678`*^9}, {
   3.575490849009437*^9, 3.57549084997838*^9}, {3.5754910209177322`*^9, 
   3.575491021163351*^9}, {3.579005015456066*^9, 3.579005031369314*^9}, {
   3.579005887926588*^9, 3.579005891361506*^9}, {3.579006051576683*^9, 
   3.579006051778133*^9}, {3.5790064448038588`*^9, 3.579006445024472*^9}, {
   3.5790072207109756`*^9, 3.579007229439061*^9}, {3.5793038816604958`*^9, 
   3.5793039016902857`*^9}, 3.579456292710253*^9, 3.579456339374145*^9, {
   3.5798239987784986`*^9, 3.579824004964258*^9}, 3.6733676013807383`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"b0", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"b1", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"c1", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "16"}], " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"c00", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"c11", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "16"}], " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"c12", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "16"}], " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"c22", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "16"}], " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"d1", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"d2", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}], ",", 
   RowBox[{"d3", "\[Rule]", 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.575043013869433*^9, 3.575043073655641*^9, 3.575071447921322*^9, 
   3.575121679858591*^9, 3.575199762167226*^9, 3.575394513588811*^9, 
   3.575410763762823*^9, 3.5754139967079487`*^9, 3.5754140702052794`*^9, 
   3.575431612462995*^9, 3.5754773374465427`*^9, 3.575478030286368*^9, 
   3.575480185856987*^9, {3.575485369424922*^9, 3.575485389918748*^9}, 
   3.575490850977228*^9, 3.575491021522698*^9, 3.5754911886749687`*^9, 
   3.575503795428752*^9, 3.575503961484762*^9, 3.575741183105297*^9, 
   3.5757428916671553`*^9, 3.575818535991591*^9, 3.5759271437889023`*^9, 
   3.575928273017831*^9, 3.5759305656761923`*^9, 3.575931310528709*^9, 
   3.575932802779191*^9, 3.575933088104995*^9, 3.575933675104514*^9, 
   3.575934955362894*^9, 3.575935239906148*^9, 3.575935875555892*^9, 
   3.57593835744022*^9, 3.5759386212125483`*^9, 3.5759394133113956`*^9, 
   3.575940269890809*^9, 3.575940605718133*^9, 3.575943006933403*^9, 
   3.5759445173487453`*^9, 3.575945937190374*^9, 3.57600770536974*^9, 
   3.576008619362673*^9, 3.576015064394923*^9, 3.5760973103045177`*^9, 
   3.576099970816325*^9, 3.5761924359087553`*^9, 3.576194998911701*^9, 
   3.576198776887838*^9, 3.576245992068396*^9, {3.5762519228143187`*^9, 
   3.576251938096982*^9}, 3.576253347619233*^9, 3.576359014786941*^9, 
   3.576363199951838*^9, 3.5763640960016737`*^9, 3.576364222025247*^9, 
   3.576364303118949*^9, 3.576364441335236*^9, 3.5763725433491*^9, 
   3.576429131389999*^9, 3.576536707108782*^9, 3.576539474404297*^9, 
   3.576539810151184*^9, 3.576690429626*^9, 3.5766928964569387`*^9, 
   3.576712134950122*^9, 3.57679573961194*^9, 3.576983736157333*^9, 
   3.576983816134763*^9, 3.5772154577109947`*^9, 3.577278670205554*^9, 
   3.577283263880157*^9, 3.577296658000499*^9, 3.5773831225389433`*^9, 
   3.577401292769883*^9, 3.577662996102186*^9, 3.577675339104742*^9, 
   3.5776763105773983`*^9, 3.577676944934498*^9, 3.577816807099882*^9, 
   3.5778344712265463`*^9, 3.577915846484864*^9, 3.577928330524172*^9, {
   3.577930787717387*^9, 3.577930793311027*^9}, 3.5779310532815523`*^9, 
   3.577934502094308*^9, 3.578330288585644*^9, 3.578342608279067*^9, 
   3.578344520409423*^9, 3.5784941980194483`*^9, 3.5787568165700293`*^9, 
   3.5788499542025747`*^9, 3.578850467574522*^9, 3.578867481129546*^9, 
   3.5789536021744556`*^9, 3.5790050376628447`*^9, {3.579005891877615*^9, 
   3.579005894786524*^9}, {3.579006052743475*^9, 3.579006056337336*^9}, {
   3.579006445615044*^9, 3.579006450744247*^9}, {3.57900723156706*^9, 
   3.579007233723658*^9}, 3.579023930003867*^9, 3.5791951436440077`*^9, 
   3.579216905806884*^9, 3.57928251272476*^9, 3.579283963278603*^9, {
   3.5793038833196297`*^9, 3.579303905744555*^9}, 3.579452384551778*^9, 
   3.579456293208825*^9, {3.579456339880159*^9, 3.579456349621071*^9}, 
   3.5798208789293423`*^9, 3.579824006747295*^9, 3.580008808176806*^9, 
   3.5804391643150253`*^9, 3.5805960519499903`*^9, 3.580642964841667*^9, 
   3.580671586364586*^9, 3.5806991896142597`*^9, 3.580845931808278*^9, 
   3.58117207528395*^9, 3.5811903574594107`*^9, 3.581195613071712*^9, 
   3.5812689697065487`*^9, 3.581270924314546*^9, 3.581276300736116*^9, 
   3.5812918055404797`*^9, 3.5812955085564203`*^9, {3.5812976194617577`*^9, 
   3.581297646507716*^9}, 3.581298245362618*^9, 3.581298844929579*^9, 
   3.581346628053157*^9, 3.5813635143482027`*^9, 3.581369182114607*^9, 
   3.5813695722705927`*^9, 3.581441999866383*^9, 3.5814507973301*^9, 
   3.5817579117250032`*^9, 3.581915387945673*^9, 3.582021737089329*^9, 
   3.58248345526013*^9, 3.584065494446225*^9, 3.584109822216515*^9, 
   3.584110054418703*^9, 3.5842086784711027`*^9, 3.5842088873323517`*^9, 
   3.5842110865926867`*^9, 3.5842112633698463`*^9, 3.5842136057592163`*^9, 
   3.584214152736388*^9, 3.584221412069625*^9, 3.58422408566437*^9, 
   3.584279759833972*^9, 3.584317071592132*^9, 3.584362233178879*^9, 
   3.584383349846652*^9, 3.584448803259794*^9, 3.5844532981658*^9, 
   3.5844533806445713`*^9, 3.5844535874814796`*^9, 3.5844538269952383`*^9, 
   3.584479563361059*^9, 3.584485173930067*^9, 3.5847084878627357`*^9, 
   3.5847203737156057`*^9, 3.5847255086131144`*^9, 3.584874692958721*^9, 
   3.584874747575588*^9, 3.584875999948765*^9, 3.5848795437531023`*^9, 
   3.591549611236767*^9, 3.603755703697228*^9, 3.603757445198309*^9, 
   3.603849868847774*^9, 3.6038501955242977`*^9, 3.603853945090892*^9, 
   3.603925571749226*^9, 3.6041575927138033`*^9, 3.604236386430853*^9, 
   3.60505044157209*^9, 3.6054522256683083`*^9, 3.605453002876965*^9, {
   3.6127948048206577`*^9, 3.6127948200980177`*^9}, 3.6127949524444733`*^9, 
   3.6127985979048433`*^9, 3.612798787235694*^9, 3.612800457060669*^9, {
   3.61331412390136*^9, 3.613314144948628*^9}, 3.613320288769598*^9, 
   3.613578349453062*^9, 3.6148130433714857`*^9, 3.6615368297368813`*^9, {
   3.661536951420266*^9, 3.661536962740807*^9}, 3.661537712899165*^9, 
   3.661611768627763*^9, 3.661615865866035*^9, 3.661637958983931*^9, 
   3.6617045664708223`*^9, 3.661810153755095*^9, 3.674840050803158*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"2", 
        SuperscriptBox["\[Pi]", 
         RowBox[{"d", "/", "2"}]]}], ")"}], "/", 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"1", "/", "2"}], "d"}], "]"}]}], " ", 
     RowBox[{"1", "/", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"2", "\[Pi]"}], ")"}], "d"]}], 
     SuperscriptBox["k", 
      RowBox[{"d", "-", "1", "+", "\[Alpha]"}]], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "mm"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "0", ",", "Infinity"}], "}"}], ",", 
    RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5776600473816767`*^9, 3.577660080024358*^9}, {
  3.577660935572315*^9, 3.5776609704676437`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"-", "d"}]], " ", 
   SuperscriptBox["mm", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"d", "+", "\[Alpha]", "-", 
       RowBox[{"2", " ", "\[Beta]"}]}], ")"}]}]], " ", 
   SuperscriptBox["\[Pi]", 
    RowBox[{
     RowBox[{"-", "d"}], "/", "2"}]], " ", 
   RowBox[{"Gamma", "[", 
    FractionBox[
     RowBox[{"d", "+", "\[Alpha]"}], "2"], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["d", "2"]}], "-", 
     FractionBox["\[Alpha]", "2"], "+", "\[Beta]"}], "]"}]}], 
  RowBox[{
   RowBox[{"Gamma", "[", 
    FractionBox["d", "2"], "]"}], " ", 
   RowBox[{"Gamma", "[", "\[Beta]", "]"}]}]]], "Output",
 CellChangeTimes->{{3.577660048527835*^9, 3.577660080426022*^9}, 
   3.5776609715526648`*^9, 3.5776629969663363`*^9, 3.577675339972603*^9, 
   3.577676311441531*^9, 3.577676945426168*^9, 3.577816807894815*^9, 
   3.577834472138543*^9, 3.577915846987459*^9, 3.5779283309577837`*^9, {
   3.577930789011544*^9, 3.5779307934958353`*^9}, 3.57793105355891*^9, 
   3.577934512467959*^9, 3.578330291442461*^9, 3.578342608767334*^9, 
   3.578344520760738*^9, 3.578494198603936*^9, 3.5787568169570847`*^9, 
   3.578849954671998*^9, 3.5788504679263487`*^9, 3.57886748190263*^9, 
   3.578953603129985*^9, 3.5790050380004263`*^9, 3.5790058950743523`*^9, 
   3.579006056651231*^9, 3.579006451023212*^9, 3.579007233916791*^9, 
   3.579023930696919*^9, 3.579195144244574*^9, 3.579216906259956*^9, 
   3.5792825130894747`*^9, 3.579283963679009*^9, {3.579303887407576*^9, 
   3.579303905928524*^9}, 3.5794523848955517`*^9, 3.579456294865025*^9, 
   3.579456349779402*^9, 3.57982088053382*^9, 3.5798240072719316`*^9, 
   3.580008808898921*^9, 3.580439165259691*^9, 3.580596052719626*^9, 
   3.5806429655281773`*^9, 3.580671587357574*^9, 3.58069918998799*^9, 
   3.580845933045457*^9, 3.581172075617673*^9, 3.581190358270054*^9, 
   3.581195613497218*^9, 3.5812689706693687`*^9, 3.581270925098887*^9, 
   3.581276301562886*^9, 3.581291806168305*^9, 3.5812955103366137`*^9, {
   3.581297619884066*^9, 3.581297646781621*^9}, 3.5812982456597977`*^9, 
   3.58129884523463*^9, 3.581346629000722*^9, 3.581363514883255*^9, 
   3.581369182607357*^9, 3.5813695727603703`*^9, 3.581442000621718*^9, 
   3.581450797984355*^9, 3.581757912799136*^9, 3.5819153888488894`*^9, 
   3.5820217376034184`*^9, 3.582483456758857*^9, 3.5840654951843224`*^9, 
   3.584109823438748*^9, 3.58411005504243*^9, 3.58420867894324*^9, 
   3.5842088878072433`*^9, 3.584211087042275*^9, 3.584211263782467*^9, 
   3.584213606190509*^9, 3.584214153152915*^9, 3.584221412532528*^9, 
   3.584224086125415*^9, 3.584279760493761*^9, 3.58431707224433*^9, 
   3.5843622338428993`*^9, 3.584383350521386*^9, 3.584448803924879*^9, 
   3.584453298868166*^9, 3.584453381048203*^9, 3.584453587903215*^9, 
   3.58445382741108*^9, 3.58447956401999*^9, 3.5844851743450823`*^9, 
   3.58470848852697*^9, 3.584720374139813*^9, 3.584725509037991*^9, 
   3.584874693630862*^9, 3.5848747479882927`*^9, 3.584876000441311*^9, 
   3.58487954425815*^9, 3.591549611883752*^9, 3.603755705923913*^9, 
   3.603757446313044*^9, 3.603849869988105*^9, 3.6038501960197163`*^9, 
   3.603853945936942*^9, 3.603925573099287*^9, 3.6041575932345943`*^9, 
   3.604236387710288*^9, 3.605050442133197*^9, 3.605452226214574*^9, 
   3.605453003403495*^9, {3.612794813018045*^9, 3.612794820385364*^9}, 
   3.612794952779544*^9, 3.612798603253881*^9, 3.612798787622415*^9, 
   3.612800457615397*^9, {3.613314124773509*^9, 3.613314145343203*^9}, 
   3.613320289427966*^9, 3.6135783503922367`*^9, 3.61481304489581*^9, 
   3.661536830310236*^9, {3.661536951818811*^9, 3.661536963262538*^9}, 
   3.661537713449461*^9, 3.661611769205964*^9, 3.661615866429778*^9, 
   3.6616379595724087`*^9, 3.661704567103942*^9, 3.661810154399089*^9, 
   3.674840051424423*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"This", " ", "is", " ", 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "d"], 
      RowBox[{"k", " ", "/", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", "\[Pi]"}], ")"}], 
        RowBox[{"d", " "}]]}], 
      RowBox[{
       SuperscriptBox["k", "\[Alpha]"], "/", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "mm"}], ")"}], "\[Beta]"]}]}]}]}], 
   " ", "*)"}], 
  RowBox[{
   RowBox[{
    RowBox[{"Intfunc1", "[", 
     RowBox[{"\[Alpha]_", ",", "\[Beta]_", ",", "mm_"}], "]"}], ":=", 
    FractionBox[
     RowBox[{
      SuperscriptBox["2", 
       RowBox[{"-", "d"}]], " ", 
      SuperscriptBox["mm", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"d", "+", "\[Alpha]", "-", 
          RowBox[{"2", " ", "\[Beta]"}]}], ")"}]}]], " ", 
      SuperscriptBox["\[Pi]", 
       RowBox[{
        RowBox[{"-", "d"}], "/", "2"}]], " ", 
      RowBox[{"Gamma", "[", 
       FractionBox[
        RowBox[{"d", "+", "\[Alpha]"}], "2"], "]"}], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["d", "2"]}], "-", 
        FractionBox["\[Alpha]", "2"], "+", "\[Beta]"}], "]"}]}], 
     RowBox[{
      RowBox[{"Gamma", "[", 
       FractionBox["d", "2"], "]"}], " ", 
      RowBox[{"Gamma", "[", "\[Beta]", "]"}]}]]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.575483860879274*^9, 3.5754838949164057`*^9}, {
  3.575484045540969*^9, 3.5754840970366707`*^9}, {3.5754843873860703`*^9, 
  3.575484400969928*^9}, {3.5776599594727707`*^9, 3.577659959844493*^9}, {
  3.577660927022317*^9, 3.5776609300441093`*^9}, {3.577660984423538*^9, 
  3.577660997688994*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Intfuncgen", "[", "integrand_", "]"}], ":=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{"Normal", "[", 
      RowBox[{"PowerExpand", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"PowerExpand", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Expand", "[", 
                  RowBox[{"Apart", "[", 
                   RowBox[{"integrand", ",", "k"}], "]"}], "]"}], "/.", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["k", "\[Alpha]\[Alpha]_"], 
                   SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "x_"}], ")"}], 
                    "\[Beta]\[Beta]_"]}], "\[Rule]", 
                  RowBox[{"Inttemp", "[", 
                   RowBox[{"\[Alpha]\[Alpha]", ",", 
                    RowBox[{"-", "\[Beta]\[Beta]"}], ",", "x"}], "]"}]}]}], "/.", 
                RowBox[{
                 RowBox[{"k", 
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "x_"}], ")"}], 
                   "\[Beta]\[Beta]_"]}], "\[Rule]", 
                 RowBox[{"Inttemp", "[", 
                  RowBox[{"1", ",", 
                   RowBox[{"-", "\[Beta]\[Beta]"}], ",", "x"}], "]"}]}]}], "/.", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox["k", "2"], "+", "x_"}], ")"}], 
                 "\[Beta]\[Beta]_"], "\[Rule]", 
                RowBox[{"Inttemp", "[", 
                 RowBox[{"0", ",", 
                  RowBox[{"-", "\[Beta]\[Beta]"}], ",", "x"}], "]"}]}]}], "/.", 
              RowBox[{"Inttemp", "\[Rule]", "Intfunc1"}]}], "]"}], "/.", 
            RowBox[{"d", "\[Rule]", 
             RowBox[{"4", "-", "\[Delta]"}]}]}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Log", "[", "x_", "]"}], ",", "qsq", ",", 
             "\[Eta]\[Mu]\[Nu]", ",", "p1\[Mu]", ",", "p2\[Mu]", ",", 
             "p1\[Nu]", ",", "p2\[Nu]"}], "}"}], ",", "Simplify"}], "]"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]", ",", "0", ",", "0"}], "}"}]}], "]"}], "]"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Log", "[", "x_", "]"}], ",", "qsq", ",", "\[Eta]\[Mu]\[Nu]", 
       ",", "p1\[Mu]", ",", "p2\[Mu]", ",", "p1\[Nu]", ",", "p2\[Nu]"}], 
      "}"}], ",", "Simplify"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.575483929629788*^9, 3.575483973212853*^9}, {
  3.5754840289325237`*^9, 3.575484029679343*^9}, {3.5754841276675253`*^9, 
  3.575484152700367*^9}, {3.5754844722541647`*^9, 3.57548451871596*^9}, {
  3.5754846423030376`*^9, 3.5754847350562983`*^9}, {3.575484901778983*^9, 
  3.575484918355789*^9}, {3.577661217455694*^9, 3.577661228557322*^9}, {
  3.5779310270457983`*^9, 3.577931029206278*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cut\[Delta]", "=", 
   RowBox[{"{", 
    RowBox[{"\[Delta]", "\[Rule]", "\[Infinity]"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6615368872769823`*^9, 3.661536902878824*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Need: 2-pt integrals B0 and B1 for general m1,m2 and expanded to first order \
around p^2=0\
\>", "Subsection",
 CellChangeTimes->{{3.5740226090101967`*^9, 3.574022667481187*^9}, 
   3.5740235572529907`*^9, {3.57502543741217*^9, 3.575025448218295*^9}, {
   3.612794809783206*^9, 3.612794810981319*^9}}],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection",
 CellChangeTimes->{{3.6615463808173513`*^9, 3.661546385228072*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "B0", " ", "expanded", " ", "to", " ", "first", " ", "order", " ", 
     "around", " ", 
     RowBox[{"p", "^", "2"}]}], "=", "0"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"integrandB0temp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Expand", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"Simplify", "[", 
           RowBox[{"Series", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"1", "/", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["k", "2"], "+", 
                RowBox[{"2", "\[Epsilon]", " ", "kdotp"}], "+", 
                RowBox[{
                 SuperscriptBox["\[Epsilon]", "2"], "ww", " ", "psq"}], "+", 
                "m2"}], ")"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"\[Epsilon]", ",", "0", ",", "2"}], "}"}]}], "]"}], 
           "]"}], "]"}], "/.", 
         RowBox[{
          SuperscriptBox["\[Epsilon]", "2"], "\[Rule]", "1"}]}], "/.", 
        RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "/.", 
      RowBox[{
       SuperscriptBox["kdotp", "2"], "\[Rule]", 
       RowBox[{"ww", " ", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{"4", "-", "\[Delta]"}], ")"}]}], 
        SuperscriptBox["k", "2"], "psq"}]}]}], ",", "psq", ",", 
     "FullSimplify"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.574897145698737*^9, 3.574897187536634*^9}, {
   3.5748973919474897`*^9, 3.574897394916353*^9}, {3.57489873915858*^9, 
   3.574898755726925*^9}, {3.5750253028314753`*^9, 3.57502530739159*^9}, 
   3.575025385969935*^9, {3.5750254195706177`*^9, 3.575025475271454*^9}, {
   3.5754706366444883`*^9, 3.575470636946663*^9}, {3.575472807561222*^9, 
   3.575472810594472*^9}, {3.575472881748713*^9, 3.575472882013166*^9}, {
   3.575476938210127*^9, 3.575476944629327*^9}, {3.575477964872813*^9, 
   3.575477967522271*^9}, {3.5776692478457003`*^9, 3.577669248913002*^9}, {
   3.661537559334858*^9, 3.6615376257930393`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m2"}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{"psq", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"m2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", "\[Delta]"}]}], ")"}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]]}]], "Output",
 CellChangeTimes->{
  3.5748971480726013`*^9, 3.574897188067897*^9, 3.5748973952454357`*^9, 
   3.574903924283656*^9, 3.575024417169873*^9, {3.5750254688715363`*^9, 
   3.575025475564116*^9}, 3.575037147637155*^9, 3.5750393165566797`*^9, 
   3.5750430739543324`*^9, 3.575071448380361*^9, 3.575121680344964*^9, 
   3.575199763377063*^9, 3.575394514018334*^9, 3.575410763854545*^9, 
   3.575413996858362*^9, 3.5754140702989807`*^9, 3.575431612599882*^9, 
   3.575470637560069*^9, 3.575472812441102*^9, 3.5754728825295486`*^9, 
   3.575477245208291*^9, 3.5754773376181726`*^9, 3.5754779682439423`*^9, 
   3.575478033333494*^9, 3.575480870676506*^9, 3.5754847513976507`*^9, {
   3.575485369614955*^9, 3.575485390171187*^9}, 3.5754911888371487`*^9, 
   3.575503795576527*^9, 3.575503961752125*^9, 3.575741183540249*^9, 
   3.575742891945629*^9, 3.575818536420475*^9, 3.57592714433144*^9, 
   3.575928273322296*^9, 3.575930566033886*^9, 3.575931310701819*^9, 
   3.5759328031188917`*^9, 3.575933088470902*^9, 3.575933675562271*^9, 
   3.5759349555196342`*^9, 3.575935240194129*^9, 3.5759358760218687`*^9, 
   3.575938357698848*^9, 3.575938621764667*^9, 3.5759394138212013`*^9, 
   3.575940270172717*^9, 3.575940606045836*^9, 3.575943007367503*^9, 
   3.575944517590722*^9, 3.575945937489574*^9, 3.576007705658545*^9, 
   3.576008619785453*^9, 3.5760150647185173`*^9, 3.576097310687413*^9, 
   3.57609997115548*^9, 3.5761924363405647`*^9, 3.576194999266423*^9, 
   3.576198777211071*^9, 3.5762459926495657`*^9, {3.576251922960137*^9, 
   3.5762519383220043`*^9}, 3.5762533478544064`*^9, 3.576359015341858*^9, 
   3.576363200161202*^9, 3.576364096213388*^9, 3.576364222249543*^9, 
   3.57636430332203*^9, 3.5763644417757683`*^9, 3.57637254371066*^9, 
   3.576429131781621*^9, 3.576536707671377*^9, 3.5765394748086557`*^9, 
   3.576539810474167*^9, 3.576690430240218*^9, 3.5766928968843403`*^9, 
   3.5767121353771353`*^9, 3.576795740885474*^9, 3.5769837367358313`*^9, 
   3.576983816236206*^9, 3.577215457900812*^9, 3.577278670649599*^9, 
   3.57728326421833*^9, 3.577296658250574*^9, 3.5773831229637012`*^9, 
   3.5774012933095284`*^9, 3.577661013637348*^9, 3.577661236968512*^9, 
   3.577662997170597*^9, 3.577669944578038*^9, 3.577675340139019*^9, 
   3.5776763208346033`*^9, 3.577676945651194*^9, 3.577816808350091*^9, 
   3.577834472325987*^9, 3.577915847120616*^9, 3.577928331106043*^9, 
   3.577930793715952*^9, 3.577931053719604*^9, 3.57793451319487*^9, 
   3.5783302931251*^9, 3.5783426090140944`*^9, 3.578344520924871*^9, 
   3.5784941988033123`*^9, 3.5787568172080507`*^9, 3.578849954969954*^9, 
   3.578850468046525*^9, 3.578867482263431*^9, 3.578953603870117*^9, 
   3.5790050380955763`*^9, 3.5790058951892023`*^9, 3.579006056742367*^9, 
   3.579006451127404*^9, 3.5790072339644213`*^9, 3.5790239308208027`*^9, 
   3.57919514443148*^9, 3.579216906450232*^9, 3.579282513242794*^9, 
   3.579283963792451*^9, 3.579298895408025*^9, 3.579299000914176*^9, {
   3.579303887649921*^9, 3.579303906166692*^9}, 3.57945238512932*^9, 
   3.5794563020419073`*^9, 3.57945634986423*^9, 3.579820880827182*^9, 
   3.5798240075336657`*^9, 3.58000880980907*^9, 3.580439165652833*^9, 
   3.580596052881098*^9, 3.580642965672771*^9, 3.5806715876342688`*^9, 
   3.580699190355055*^9, 3.580845933454543*^9, 3.581172075806842*^9, 
   3.58119035861716*^9, 3.5811956136607103`*^9, 3.581268971063052*^9, 
   3.581270925305366*^9, 3.5812763017488813`*^9, 3.5812918063394938`*^9, 
   3.581295510550188*^9, {3.581297620066132*^9, 3.581297646995181*^9}, 
   3.5812982457910213`*^9, 3.581298845411449*^9, 3.581346629509644*^9, 
   3.581363515122386*^9, 3.581369182793726*^9, 3.5813695729446173`*^9, 
   3.581442000865353*^9, 3.581450798393248*^9, 3.58175781064107*^9, 
   3.581757913356634*^9, 3.581915389283721*^9, 3.582021737995367*^9, 
   3.582483457293336*^9, 3.584065495484778*^9, 3.584109823850079*^9, 
   3.584110055446577*^9, 3.58420867935999*^9, 3.584208888207376*^9, 
   3.584211087449856*^9, 3.584211264318192*^9, 3.584213606591154*^9, 
   3.58421415356994*^9, 3.584221412945539*^9, 3.584224086509255*^9, 
   3.584279760731715*^9, 3.5843170726381493`*^9, 3.58436223421378*^9, 
   3.5843833509438868`*^9, 3.584448804231526*^9, 3.584453299107111*^9, 
   3.5844533815592213`*^9, 3.5844535883366003`*^9, 3.584453827780903*^9, 
   3.5844795644148293`*^9, 3.5844851747173147`*^9, 3.5847084887213583`*^9, 
   3.58472037452293*^9, 3.584725509340436*^9, 3.5848746940768013`*^9, 
   3.584874748294691*^9, 3.584876000821691*^9, 3.5848795447088842`*^9, 
   3.591549612281967*^9, 3.6037557063824244`*^9, 3.603757446708041*^9, 
   3.603849870361224*^9, 3.603850196225211*^9, 3.60385394633781*^9, 
   3.603925573442759*^9, 3.604157593541947*^9, 3.604236388280788*^9, 
   3.6050504424098377`*^9, 3.605452226598083*^9, 3.60545300370308*^9, {
   3.612794815200815*^9, 3.6127948206611633`*^9}, 3.612794953101596*^9, 
   3.612798647920527*^9, 3.612798787795219*^9, 3.612800457971463*^9, 
   3.6133202897844048`*^9, 3.613578351585102*^9, 3.613578436449181*^9, 
   3.661536849440217*^9, {3.661536908532546*^9, 3.661536921378791*^9}, 
   3.661536967278789*^9, 3.6615375006817713`*^9, {3.661537531079461*^9, 
   3.661537539371904*^9}, 3.6615375699385767`*^9, {3.661537617001821*^9, 
   3.661537626750683*^9}, 3.661537714321247*^9, 3.661611780092044*^9, 
   3.6616158682295218`*^9, 3.661637960870775*^9, 3.6617045680259657`*^9, 
   3.661810155536441*^9, 3.674840068863874*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0temp", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"b0", " ", 
       RowBox[{"Intfuncgen", "[", "integrandB0temp", "]"}]}], "/.", 
      "subtry"}], ",", 
     RowBox[{"\[Delta]", "\[Rule]", "\[Infinity]"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5754808634290257`*^9, 3.575480867725335*^9}, {
   3.575481968745048*^9, 3.575481970120699*^9}, {3.575484534913494*^9, 
   3.575484538673607*^9}, {3.575490855385932*^9, 3.575490856464773*^9}, 
   3.577296603663827*^9, 3.577675298851139*^9, {3.661537644734112*^9, 
   3.66153765080684*^9}, {3.661537786843823*^9, 3.661537792240189*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], "3"]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "m1", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["m1", "2"], "-", 
        RowBox[{"2", " ", "m1", " ", "m2"}], "+", 
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{"m2", "+", "psq"}], ")"}]}]}], ")"}], " ", 
      RowBox[{"Log", "[", "m1", "]"}]}], "-", 
     RowBox[{"2", " ", "m2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["m1", "2"], "+", 
        SuperscriptBox["m2", "2"], "+", 
        RowBox[{"m1", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", "m2"}], "+", "psq"}], ")"}]}]}], ")"}], 
      " ", 
      RowBox[{"Log", "[", "m2", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"m1", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "psq"}], "+", 
           RowBox[{"m2", " ", 
            RowBox[{"(", 
             RowBox[{"4", "-", 
              RowBox[{"4", " ", "EulerGamma"}], "+", 
              RowBox[{"Log", "[", "256", "]"}], "+", 
              RowBox[{"4", " ", 
               RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}]}], ")"}]}], 
        "+", 
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "psq"}], "+", 
           RowBox[{"2", " ", "m2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "EulerGamma", "-", 
              RowBox[{"Log", "[", 
               RowBox[{"4", " ", "\[Pi]"}], "]"}]}], ")"}]}]}], ")"}]}], "+", 
        
        RowBox[{"2", " ", 
         SuperscriptBox["m1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "EulerGamma", "-", 
           RowBox[{"Log", "[", 
            RowBox[{"4", " ", "\[Pi]"}], "]"}]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{{3.661537636962386*^9, 3.661537651253599*^9}, 
   3.66153771749723*^9, 3.661537794029977*^9, 3.661611781586225*^9, 
   3.661615869681116*^9, 3.661637962088296*^9, 3.661704569217778*^9, 
   3.661810156741922*^9, 3.674840070566436*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "B1", " ", "expanded", " ", "to", " ", "first", " ", "order", " ", 
     "around", " ", 
     RowBox[{"p", "^", "2"}]}], "=", "0"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"integrandB1temp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"Simplify", "[", 
             RowBox[{"Series", "[", 
              RowBox[{
               RowBox[{"\[Epsilon]", " ", 
                RowBox[{
                 RowBox[{"k\[Mu]", "/", 
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox["k", "2"], "+", 
                   RowBox[{"2", "\[Epsilon]", " ", "kdotp"}], "+", 
                   RowBox[{
                    SuperscriptBox["\[Epsilon]", "2"], "ww", " ", "psq"}], 
                   "+", "m2"}], ")"}]}]}], ",", 
               RowBox[{"{", 
                RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], 
             "]"}], "]"}], "/.", 
           RowBox[{
            SuperscriptBox["\[Epsilon]", "4"], "\[Rule]", "1"}]}], "/.", 
          RowBox[{
           SuperscriptBox["\[Epsilon]", "2"], "\[Rule]", "1"}]}], "/.", 
         RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"kdotp", " ", "k\[Mu]"}], "\[Rule]", 
          RowBox[{
           RowBox[{"1", "/", "d"}], " ", 
           SuperscriptBox["k", "2"], "p\[Mu]"}]}], ",", 
         RowBox[{
          RowBox[{
           SuperscriptBox["kdotp", "3"], " ", "k\[Mu]"}], " ", "\[Rule]", 
          RowBox[{"ww", " ", 
           RowBox[{"3", "/", 
            RowBox[{"(", 
             RowBox[{"d", 
              RowBox[{"(", 
               RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
           SuperscriptBox["k", "4"], "psq", " ", "p\[Mu]"}]}]}], "}"}]}], "/.", 
      RowBox[{"d", "\[Rule]", 
       RowBox[{"4", "-", "\[Delta]"}]}]}], ",", "psq", ",", "FullSimplify"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.575025500320006*^9, 3.575025543620569*^9}, {
  3.575025576572165*^9, 3.575025638525362*^9}, {3.5750256807403316`*^9, 
  3.575025698648065*^9}, {3.575476974978651*^9, 3.575476986762218*^9}, {
  3.575477034686808*^9, 3.575477039334394*^9}, {3.575477128735261*^9, 
  3.575477131375552*^9}, {3.575477234276248*^9, 3.5754772343407793`*^9}, {
  3.5754779422269497`*^9, 3.5754779488187857`*^9}, {3.57547800318672*^9, 
  3.5754780068982143`*^9}, {3.577379162148179*^9, 3.577379162780962*^9}, {
  3.577669736076992*^9, 3.5776697367224092`*^9}, {3.5776699176308537`*^9, 
  3.57766992817467*^9}, {3.577676285357152*^9, 3.577676285827388*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"2", " ", 
    SuperscriptBox["k", "2"], " ", "p\[Mu]"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{"4", " ", 
    SuperscriptBox["k", "2"], " ", "psq", " ", "p\[Mu]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"m2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", "\[Delta]"}]}], ")"}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]]}]], "Output",
 CellChangeTimes->{
  3.575025543963251*^9, {3.5750255771351852`*^9, 3.575025603192131*^9}, 
   3.5750256990448103`*^9, 3.575037159045957*^9, 3.5750393251379843`*^9, 
   3.575043085539357*^9, 3.575071460067535*^9, 3.5751216916812696`*^9, 
   3.5751997742293587`*^9, 3.575410774465536*^9, 3.575414007569174*^9, 
   3.5754140809274817`*^9, 3.575431623470738*^9, 3.575476998152134*^9, 
   3.575477049452777*^9, {3.5754771290641193`*^9, 3.5754771315594378`*^9}, 
   3.5754772526975327`*^9, 3.575477348620954*^9, 3.575478007327166*^9, 
   3.575478044159663*^9, 3.5754819958585577`*^9, 3.5754847604355392`*^9, {
   3.575485369825796*^9, 3.575485390270904*^9}, 3.575490857672619*^9, 
   3.575491029478403*^9, 3.575491188909198*^9, 3.575503795742791*^9, 
   3.575503961916787*^9, 3.575741183738614*^9, 3.575742892133062*^9, 
   3.575818536618863*^9, 3.5759271445291023`*^9, 3.575928273504549*^9, 
   3.5759305662142487`*^9, 3.575931310912696*^9, 3.575932803300067*^9, 
   3.5759330887174253`*^9, 3.575933675744331*^9, 3.5759349557274*^9, 
   3.575935240393528*^9, 3.5759358762011642`*^9, 3.575938357904718*^9, 
   3.575938622019094*^9, 3.57593941407197*^9, 3.575940270353303*^9, 
   3.575940606228457*^9, 3.575943007548552*^9, 3.575944517771682*^9, 
   3.575945937670473*^9, 3.576007705845668*^9, 3.5760086200610027`*^9, 
   3.576015064901465*^9, 3.576097310886014*^9, 3.57609997133689*^9, 
   3.576192436603448*^9, 3.576194999446961*^9, 3.576198777390375*^9, 
   3.576245992851748*^9, {3.576251923137641*^9, 3.5762519384929943`*^9}, 
   3.576253348048314*^9, 3.576359015541642*^9, 3.576363200346848*^9, 
   3.5763640964203997`*^9, 3.576364222442628*^9, 3.57636430351901*^9, 
   3.5763644419588223`*^9, 3.5763725438931026`*^9, 3.576429131961306*^9, 
   3.576536707867393*^9, 3.5765394749895353`*^9, 3.576539810653819*^9, 
   3.576690430441633*^9, 3.576692897063964*^9, 3.576712135560163*^9, 
   3.576795742102047*^9, 3.57698373721441*^9, 3.576983816398879*^9, 
   3.577215458120162*^9, 3.577278671808963*^9, 3.577283264477585*^9, 
   3.577296658419301*^9, 3.577379163255774*^9, 3.577383123754422*^9, 
   3.57740129416148*^9, 3.57766125927941*^9, 3.577662998891724*^9, 
   3.577669928871092*^9, 3.5776753429076853`*^9, 3.577676326464102*^9, 
   3.5776769483210373`*^9, 3.577816811307097*^9, 3.577834475168556*^9, 
   3.5779158497982283`*^9, 3.5779283338434134`*^9, 3.577930796305224*^9, 
   3.577931054883415*^9, 3.577934514475995*^9, 3.578330294211041*^9, 
   3.578342610024601*^9, 3.57834452201344*^9, 3.578494199962214*^9, 
   3.578756818240036*^9, 3.5788499560190077`*^9, 3.578850469063386*^9, 
   3.57886748310608*^9, 3.578953605017568*^9, 3.579005039074963*^9, 
   3.579005896129548*^9, 3.5790060577014513`*^9, 3.579006451448567*^9, 
   3.579007234152692*^9, 3.5790239318008623`*^9, 3.5791951454213667`*^9, 
   3.579216907472144*^9, 3.579282514193433*^9, 3.5792839647140083`*^9, 
   3.579298901943714*^9, 3.579299002146305*^9, {3.5793038886214333`*^9, 
   3.579303906566909*^9}, 3.579452386298285*^9, 3.579456303736022*^9, 
   3.579456350190114*^9, 3.579820881767232*^9, 3.579824008496764*^9, 
   3.5800088108648577`*^9, 3.580439166508716*^9, 3.580596053906897*^9, 
   3.5806429666235027`*^9, 3.580671588555583*^9, 3.5806991913776817`*^9, 
   3.580845934493079*^9, 3.581172076826466*^9, 3.581190359816094*^9, 
   3.5811956146823606`*^9, 3.5812689720563507`*^9, 3.5812709263752127`*^9, 
   3.581276302815827*^9, 3.581291807269143*^9, 3.581295511364944*^9, {
   3.5812976209285707`*^9, 3.5812976479335203`*^9}, 3.581298246668334*^9, 
   3.581298846293037*^9, 3.581346630649757*^9, 3.581363516169319*^9, 
   3.5813691839162827`*^9, 3.581369574150185*^9, 3.58144200185637*^9, 
   3.581450799714048*^9, 3.581757914651925*^9, 3.581915390645562*^9, 
   3.582021739138659*^9, 3.582483458676896*^9, 3.584065496539905*^9, 
   3.5841098250982847`*^9, 3.584110056488325*^9, 3.5842086803501663`*^9, 
   3.584208889197073*^9, 3.584211088461524*^9, 3.584211265355968*^9, 
   3.5842136076073227`*^9, 3.5842141546007147`*^9, 3.584221414007959*^9, 
   3.584224087545247*^9, 3.584279761819942*^9, 3.5843170736920137`*^9, 
   3.584362235290751*^9, 3.5843833519892483`*^9, 3.584448805298078*^9, 
   3.584453300465282*^9, 3.584453382912532*^9, 3.584453589366346*^9, 
   3.5844538288137207`*^9, 3.58447956577803*^9, 3.5844851757513657`*^9, 
   3.584708489805616*^9, 3.58472037557771*^9, 3.58472551039981*^9, 
   3.5848746951418324`*^9, 3.584874749307119*^9, 3.58487600181743*^9, 
   3.5848795457362022`*^9, 3.591549613465872*^9, 3.6037557076252193`*^9, 
   3.603757448220387*^9, 3.603849871509256*^9, 3.603850197430315*^9, 
   3.6038539475703087`*^9, 3.603925574881588*^9, 3.604157594702343*^9, 
   3.604236389690922*^9, 3.605050443496254*^9, 3.605452227648991*^9, 
   3.6054530047336187`*^9, {3.612794818082922*^9, 3.612794820755693*^9}, 
   3.6127949532514553`*^9, 3.612798652658942*^9, 3.6127987892889137`*^9, 
   3.6128004591967983`*^9, 3.6133202911827517`*^9, 3.613578354146324*^9, 
   3.613578436540145*^9, 3.661536870093495*^9, 3.661536927651196*^9, 
   3.661536971285082*^9, 3.6615376606076117`*^9, 3.6615377182398453`*^9, 
   3.6616117820214777`*^9, 3.66161587007526*^9, 3.661637962725705*^9, 
   3.661704569672117*^9, 3.661810156972888*^9, 3.674840071096583*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1temp", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"b1", " ", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"Intfuncgen", "[", "integrandB1temp", "]"}], ",", "p\[Mu]"}],
         "]"}]}], "/.", "subtry"}], ",", 
     RowBox[{"\[Delta]", "\[Rule]", "\[Infinity]"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575481997438965*^9, 3.575482008880863*^9}, {
   3.575482088808155*^9, 3.575482092007248*^9}, {3.575484546161133*^9, 
   3.5754845536794558`*^9}, 3.577296599280238*^9, 3.577675304155553*^9, {
   3.661537669626543*^9, 3.661537692192369*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"12", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], "4"]}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"6", " ", 
     SuperscriptBox["m1", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m1", "2"], "-", 
       RowBox[{"2", " ", "m1", " ", "m2"}], "+", 
       RowBox[{"m2", " ", 
        RowBox[{"(", 
         RowBox[{"m2", "+", 
          RowBox[{"2", " ", "psq"}]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "+", 
    RowBox[{"6", " ", "m2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["m1", "3"]}], "-", 
       RowBox[{"4", " ", "m1", " ", 
        SuperscriptBox["m2", "2"]}], "+", 
       SuperscriptBox["m2", "3"], "+", 
       RowBox[{
        SuperscriptBox["m1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"5", " ", "m2"}], "-", 
          RowBox[{"2", " ", "psq"}]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"Log", "[", "m2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"m1", " ", "m2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "10"}], " ", "psq"}], "+", 
          RowBox[{"3", " ", "m2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "5"}], "+", 
             RowBox[{"6", " ", "EulerGamma"}], "-", 
             RowBox[{"12", " ", 
              RowBox[{"Log", "[", "2", "]"}]}], "-", 
             RowBox[{"6", " ", 
              RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["m1", "3"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "3"}], "+", 
          RowBox[{"2", " ", "EulerGamma"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"Log", "[", "2", "]"}]}], "-", 
          RowBox[{"2", " ", 
           RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["m2", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "psq"}], "+", 
          RowBox[{"m2", " ", 
           RowBox[{"(", 
            RowBox[{"3", "-", 
             RowBox[{"6", " ", "EulerGamma"}], "+", 
             RowBox[{"Log", "[", "4096", "]"}], "+", 
             RowBox[{"6", " ", 
              RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["m1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "4"}], " ", "psq"}], "+", 
          RowBox[{"3", " ", "m2", " ", 
           RowBox[{"(", 
            RowBox[{"7", "-", 
             RowBox[{"6", " ", "EulerGamma"}], "+", 
             RowBox[{"Log", "[", "4096", "]"}], "+", 
             RowBox[{"6", " ", 
              RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}]}], ")"}]}]}], 
      ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.575484554376258*^9, 3.575484761350093*^9, {3.575485369967785*^9, 
   3.575485390366139*^9}, 3.575491033174017*^9, 3.575491188995398*^9, 
   3.57550379588421*^9, 3.5755039620583067`*^9, 3.575741183933632*^9, 
   3.5757428923283*^9, 3.575818536809085*^9, 3.575927144723024*^9, 
   3.575928273695162*^9, 3.575930566402096*^9, 3.5759313111273603`*^9, 
   3.575932803489469*^9, 3.575933088906666*^9, 3.5759336759320107`*^9, 
   3.575934955938051*^9, 3.575935240604005*^9, 3.57593587639072*^9, 
   3.57593835811589*^9, 3.575938622209331*^9, 3.5759394142649193`*^9, 
   3.5759402705408773`*^9, 3.575940606416358*^9, 3.575943007737093*^9, 
   3.57594451795897*^9, 3.575945937858364*^9, 3.576007706042568*^9, 
   3.576008620261612*^9, 3.576015065093461*^9, 3.576097311075554*^9, 
   3.5760999715252666`*^9, 3.5761924367933292`*^9, 3.576194999633471*^9, 
   3.576198777577023*^9, 3.576245993044931*^9, {3.576251923326189*^9, 
   3.576251938679184*^9}, 3.576253348255924*^9, 3.576359015732218*^9, 
   3.576363200552608*^9, 3.576364096636528*^9, 3.576364222652931*^9, 
   3.576364303730906*^9, 3.5763644421498528`*^9, 3.576372544083868*^9, 
   3.576429132148416*^9, 3.576536708058949*^9, 3.576539475179587*^9, 
   3.576539810842581*^9, 3.576690430637081*^9, 3.5766928972525787`*^9, 
   3.576712135750451*^9, 3.5767957422399673`*^9, 3.5769837375225677`*^9, 
   3.5769838165381937`*^9, 3.5772154582570467`*^9, 3.5772786719455967`*^9, 
   3.577283264628951*^9, 3.577296658586594*^9, 3.577379166638294*^9, 
   3.577383124038525*^9, 3.577401294426441*^9, 3.5776612652664824`*^9, 
   3.577663001440487*^9, 3.577669940529717*^9, 3.577675346782977*^9, 
   3.577676333222208*^9, 3.577676951442605*^9, 3.5778168149467373`*^9, 
   3.577834478280175*^9, 3.577915852873177*^9, 3.577928336887247*^9, 
   3.577930799511572*^9, 3.577931055925191*^9, 3.577934515544627*^9, 
   3.578330295310927*^9, 3.5783426110772333`*^9, 3.5783445230866137`*^9, 
   3.578494200959385*^9, 3.578756819422586*^9, 3.578849957043049*^9, 
   3.57885047025478*^9, 3.57886748412964*^9, 3.578953606311413*^9, 
   3.579005040121973*^9, 3.579005897111558*^9, 3.579006058709331*^9, 
   3.5790064520907516`*^9, 3.5790072343650093`*^9, 3.5790239327841578`*^9, 
   3.579195146424066*^9, 3.579216908480912*^9, 3.5792825152260113`*^9, 
   3.579283965676848*^9, 3.5792989045808773`*^9, 3.579299004174643*^9, {
   3.5793038894346848`*^9, 3.579303907581292*^9}, 3.579452387291152*^9, 
   3.579456306820874*^9, 3.579456351031074*^9, 3.579820882752078*^9, 
   3.5798240094721518`*^9, 3.580008811830606*^9, 3.580439167415325*^9, 
   3.580596055036178*^9, 3.580642967571175*^9, 3.580671589487185*^9, 
   3.580699192424582*^9, 3.580845935455229*^9, 3.581172078053103*^9, 
   3.581190360927799*^9, 3.581195615626981*^9, 3.581268973053722*^9, 
   3.581270927400344*^9, 3.581276303844061*^9, 3.5812918082904863`*^9, 
   3.5812955124227657`*^9, {3.581297621911838*^9, 3.58129764886525*^9}, 
   3.58129824765764*^9, 3.581298847294814*^9, 3.581346631847926*^9, 
   3.581363517350533*^9, 3.5813691851870823`*^9, 3.581369575421073*^9, 
   3.5814420029795237`*^9, 3.581450800989945*^9, 3.581757915926536*^9, 
   3.5819153921212683`*^9, 3.582021740352882*^9, 3.582483459950286*^9, 
   3.584065497766983*^9, 3.584109826382741*^9, 3.584110057757106*^9, 
   3.584208681540574*^9, 3.584208890369155*^9, 3.584211089646234*^9, 
   3.584211266547009*^9, 3.584213608776062*^9, 3.58421415578526*^9, 
   3.584221415183177*^9, 3.584224088743505*^9, 3.58427976311554*^9, 
   3.5843170749013367`*^9, 3.584362236495202*^9, 3.584383353179006*^9, 
   3.5844488064951878`*^9, 3.584453301738124*^9, 3.5844533842143087`*^9, 
   3.584453590543954*^9, 3.5844538300099773`*^9, 3.5844795670079403`*^9, 
   3.5844851769452677`*^9, 3.584708491090905*^9, 3.584720376775831*^9, 
   3.584725511606925*^9, 3.584874696355105*^9, 3.584874750481352*^9, 
   3.584876002991663*^9, 3.5848795469110518`*^9, 3.5915496147532177`*^9, 
   3.603755709015025*^9, 3.6037574495268717`*^9, 3.603849872712492*^9, 
   3.603850198717548*^9, 3.6038539488791733`*^9, 3.603925576285527*^9, 
   3.604157595948022*^9, 3.604236391089527*^9, 3.6050504447318163`*^9, 
   3.605452228879327*^9, 3.605453005965335*^9, 3.61279482209026*^9, 
   3.6127949541605673`*^9, 3.612798655212516*^9, 3.6127987908733053`*^9, 
   3.612800460509871*^9, 3.613320292506565*^9, 3.6135783558757*^9, 
   3.6135784366084347`*^9, 3.6615368742525063`*^9, 3.661536931688603*^9, 
   3.661536974607854*^9, {3.661537664419066*^9, 3.661537723012719*^9}, 
   3.661611783568841*^9, 3.66161587152917*^9, 3.661637964196723*^9, 
   3.661704571190151*^9, 3.661810158485057*^9, 3.6748400725399027`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"B0temp0", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"B0temp", "/.", 
      RowBox[{"psq", "\[Rule]", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Log", "[", "m1", "]"}], ",", 
      RowBox[{"Log", "[", "m2", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"B0temp1", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    SubscriptBox["\[PartialD]", "psq"], "B0temp"}], "]"}]}]}], "Input",
 CellChangeTimes->{
  3.66153788186136*^9, {3.661537938648501*^9, 3.6615379417365923`*^9}, {
   3.661537999519723*^9, 3.661538060226807*^9}, {3.661538095681819*^9, 
   3.661538112362927*^9}, {3.6615382067674437`*^9, 3.6615382319240637`*^9}, {
   3.6615382810193987`*^9, 3.661538343250083*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"m1", " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], 
    RowBox[{"m1", "-", "m2"}]]}], "+", 
  FractionBox[
   RowBox[{"m2", " ", 
    RowBox[{"Log", "[", "m2", "]"}]}], 
   RowBox[{"m1", "-", "m2"}]], "+", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], "2"]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"4", " ", "EulerGamma", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "256", "]"}]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.661537882235841*^9, 3.661537942562162*^9, {3.66153804597289*^9, 
   3.661538064193548*^9}, 3.6615380998848743`*^9, {3.661538212383017*^9, 
   3.661538232387362*^9}, {3.66153828677355*^9, 3.661538298029377*^9}, {
   3.661538333138364*^9, 3.661538343682391*^9}, 3.661611783961237*^9, 
   3.661615871905822*^9, 3.6616379648274393`*^9, 3.661704571570312*^9, 
   3.661810158656384*^9, 3.674840073072592*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["m1", "2"], "-", 
   SuperscriptBox["m2", "2"], "-", 
   RowBox[{"2", " ", "m1", " ", "m2", " ", 
    RowBox[{"Log", "[", "m1", "]"}]}], "+", 
   RowBox[{"2", " ", "m1", " ", "m2", " ", 
    RowBox[{"Log", "[", "m2", "]"}]}]}], 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"m1", "-", "m2"}], ")"}], "3"]}]]], "Output",
 CellChangeTimes->{
  3.661537882235841*^9, 3.661537942562162*^9, {3.66153804597289*^9, 
   3.661538064193548*^9}, 3.6615380998848743`*^9, {3.661538212383017*^9, 
   3.661538232387362*^9}, {3.66153828677355*^9, 3.661538298029377*^9}, {
   3.661538333138364*^9, 3.661538343682391*^9}, 3.661611783961237*^9, 
   3.661615871905822*^9, 3.6616379648274393`*^9, 3.661704571570312*^9, 
   3.661810158656384*^9, 3.6748400730745373`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], "2"]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"4", " ", "EulerGamma", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "256", "]"}]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661538356698373*^9, 3.661538367347582*^9}}],

Cell[BoxData[
 RowBox[{"1", "-", "EulerGamma", "+", 
  RowBox[{"Log", "[", 
   RowBox[{"4", " ", "\[Pi]"}], "]"}]}]], "Output",
 CellChangeTimes->{
  3.661537882235841*^9, 3.661537942562162*^9, {3.66153804597289*^9, 
   3.661538064193548*^9}, 3.6615380998848743`*^9, {3.661538212383017*^9, 
   3.661538232387362*^9}, {3.66153828677355*^9, 3.661538298029377*^9}, {
   3.661538333138364*^9, 3.661538367645845*^9}, 3.6616117840280647`*^9, 
   3.661615871961843*^9, 3.66163796489466*^9, 3.661704571636454*^9, 
   3.6618101587576523`*^9, 3.6748400732055817`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"B1temp0", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"B1temp", "/.", 
    RowBox[{"psq", "\[Rule]", "0"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"B1temp1", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    SubscriptBox["\[PartialD]", "psq"], "B1temp"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.661538081111959*^9, 3.661538115852119*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"12", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], "3"]}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"6", " ", 
     SuperscriptBox["m1", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "-", 
    RowBox[{"6", " ", 
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "m1"}], "-", "m2"}], ")"}], " ", "m2", " ", 
     RowBox[{"Log", "[", "m2", "]"}]}], "+", 
    RowBox[{"3", " ", "m1", " ", 
     SuperscriptBox["m2", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "5"}], "+", 
       RowBox[{"6", " ", "EulerGamma"}], "-", 
       RowBox[{"12", " ", 
        RowBox[{"Log", "[", "2", "]"}]}], "-", 
       RowBox[{"6", " ", 
        RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "+", 
    RowBox[{"3", " ", 
     SuperscriptBox["m1", "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "3"}], "+", 
       RowBox[{"2", " ", "EulerGamma"}], "-", 
       RowBox[{"4", " ", 
        RowBox[{"Log", "[", "2", "]"}]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "+", 
    RowBox[{
     SuperscriptBox["m2", "3"], " ", 
     RowBox[{"(", 
      RowBox[{"3", "-", 
       RowBox[{"6", " ", "EulerGamma"}], "+", 
       RowBox[{"Log", "[", "4096", "]"}], "+", 
       RowBox[{"6", " ", 
        RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "+", 
    RowBox[{"3", " ", 
     SuperscriptBox["m1", "2"], " ", "m2", " ", 
     RowBox[{"(", 
      RowBox[{"7", "-", 
       RowBox[{"6", " ", "EulerGamma"}], "+", 
       RowBox[{"Log", "[", "4096", "]"}], "+", 
       RowBox[{"6", " ", 
        RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}]}], ")"}]}]], "Output",\

 CellChangeTimes->{{3.6615380918685493`*^9, 3.66153811632733*^9}, 
   3.6616117840901537`*^9, 3.661615872017013*^9, 3.661637964994103*^9, 
   3.66170457170319*^9, 3.661810158821692*^9, 3.674840073271914*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["m1", "3"]}], "+", 
    RowBox[{"3", " ", 
     SuperscriptBox["m1", "2"], " ", "m2"}], "-", 
    RowBox[{"6", " ", "m1", " ", 
     SuperscriptBox["m2", "2"]}], "+", 
    SuperscriptBox["m2", "3"], "-", 
    RowBox[{"6", " ", 
     SuperscriptBox["m1", "2"], " ", "m2", " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "+", 
    RowBox[{"6", " ", 
     SuperscriptBox["m1", "2"], " ", "m2", " ", 
     RowBox[{"Log", "[", "m2", "]"}]}]}], 
   RowBox[{"6", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m1", "-", "m2"}], ")"}], "4"]}]]}]], "Output",
 CellChangeTimes->{{3.6615380918685493`*^9, 3.66153811632733*^9}, 
   3.6616117840901537`*^9, 3.661615872017013*^9, 3.661637964994103*^9, 
   3.66170457170319*^9, 3.661810158821692*^9, 3.674840073274667*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1func", "[", 
  RowBox[{"0", ",", "x", ",", "x"}], "]"}]], "Input",
 CellChangeTimes->{{3.613578359942388*^9, 3.613578411606007*^9}}],

Cell[BoxData[
 RowBox[{"B1func", "[", 
  RowBox[{"0", ",", "x", ",", "x"}], "]"}]], "Output",
 CellChangeTimes->{{3.613578363428619*^9, 3.6135784366683817`*^9}, 
   3.66153698340871*^9, 3.661611784158329*^9, 3.661615872122971*^9, 
   3.661637965036441*^9, 3.6617045717388353`*^9, 3.661810158863221*^9, 
   3.674840073354556*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"FFF", "=", "B1func1"}], "\[IndentingNewLine]", 
 RowBox[{"Normal", "[", 
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{"FFF", "[", 
     RowBox[{"x", ",", 
      RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"\[Epsilon]", ",", "y"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "\[Epsilon]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Normal", "[", 
      RowBox[{"Series", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Delta]\[Delta]", ",", "\[Epsilon]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
     ",", 
     RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], ",", 
   RowBox[{"\[Delta]\[Delta]", "\[Rule]", "0"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.661538487136751*^9, 3.661538530667296*^9}, {
   3.6615385983560467`*^9, 3.6615386244771357`*^9}, 3.661538661022667*^9, {
   3.661539364910467*^9, 3.661539370343006*^9}, {3.66153957098244*^9, 
   3.661539599142818*^9}, {3.661539898594648*^9, 3.661539908383321*^9}, {
   3.661540213150674*^9, 3.66154024364948*^9}, {3.66154037001757*^9, 
   3.661540386163279*^9}, {3.661540636698042*^9, 3.661540636824669*^9}}],

Cell[BoxData["B1func1"], "Output",
 CellChangeTimes->{
  3.661538531108082*^9, {3.661538598986537*^9, 3.661538661280526*^9}, 
   3.6615393714815493`*^9, {3.661539576626048*^9, 3.661539599500042*^9}, {
   3.661539900505011*^9, 3.661539909332912*^9}, {3.661540221285543*^9, 
   3.6615402442294483`*^9}, 3.661540386943119*^9, 3.6615406370725393`*^9, 
   3.6615410607228937`*^9, 3.661611784193837*^9, 3.6616158721588297`*^9, 
   3.661637965088771*^9, 3.661704571819028*^9, 3.661810158922669*^9, 
   3.674840073439406*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"B1func1", "[", 
   RowBox[{"x", ",", "x"}], "]"}], "+", 
  RowBox[{"\[Epsilon]", " ", 
   RowBox[{
    SuperscriptBox["B1func1", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "x"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[Epsilon]", "2"], " ", 
   RowBox[{
    SuperscriptBox["B1func1", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "2"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "x"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "6"], " ", 
   SuperscriptBox["\[Epsilon]", "3"], " ", 
   RowBox[{
    SuperscriptBox["B1func1", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "3"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "x"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "24"], " ", 
   SuperscriptBox["\[Epsilon]", "4"], " ", 
   RowBox[{
    SuperscriptBox["B1func1", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "4"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "x"}], "]"}]}]}]], "Output",
 CellChangeTimes->{
  3.661538531108082*^9, {3.661538598986537*^9, 3.661538661280526*^9}, 
   3.6615393714815493`*^9, {3.661539576626048*^9, 3.661539599500042*^9}, {
   3.661539900505011*^9, 3.661539909332912*^9}, {3.661540221285543*^9, 
   3.6615402442294483`*^9}, 3.661540386943119*^9, 3.6615406370725393`*^9, 
   3.6615410607228937`*^9, 3.661611784193837*^9, 3.6616158721588297`*^9, 
   3.661637965088771*^9, 3.661704571819028*^9, 3.661810158922669*^9, 
   3.6748400734411287`*^9}],

Cell[BoxData[
 RowBox[{"B1func1", "[", 
  RowBox[{"0", ",", "y"}], "]"}]], "Output",
 CellChangeTimes->{
  3.661538531108082*^9, {3.661538598986537*^9, 3.661538661280526*^9}, 
   3.6615393714815493`*^9, {3.661539576626048*^9, 3.661539599500042*^9}, {
   3.661539900505011*^9, 3.661539909332912*^9}, {3.661540221285543*^9, 
   3.6615402442294483`*^9}, 3.661540386943119*^9, 3.6615406370725393`*^9, 
   3.6615410607228937`*^9, 3.661611784193837*^9, 3.6616158721588297`*^9, 
   3.661637965088771*^9, 3.661704571819028*^9, 3.661810158922669*^9, 
   3.674840073442375*^9}],

Cell[BoxData[
 RowBox[{"B1func1", "[", 
  RowBox[{"x", ",", "0"}], "]"}]], "Output",
 CellChangeTimes->{
  3.661538531108082*^9, {3.661538598986537*^9, 3.661538661280526*^9}, 
   3.6615393714815493`*^9, {3.661539576626048*^9, 3.661539599500042*^9}, {
   3.661539900505011*^9, 3.661539909332912*^9}, {3.661540221285543*^9, 
   3.6615402442294483`*^9}, 3.661540386943119*^9, 3.6615406370725393`*^9, 
   3.6615410607228937`*^9, 3.661611784193837*^9, 3.6616158721588297`*^9, 
   3.661637965088771*^9, 3.661704571819028*^9, 3.661810158922669*^9, 
   3.674840073443411*^9}],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"B1func1", "[", 
    RowBox[{"\[Delta]\[Delta]", ",", "0"}], "]"}], ",", 
   RowBox[{"\[Delta]\[Delta]", "\[Rule]", "0"}]}], "]"}]], "Output",
 CellChangeTimes->{
  3.661538531108082*^9, {3.661538598986537*^9, 3.661538661280526*^9}, 
   3.6615393714815493`*^9, {3.661539576626048*^9, 3.661539599500042*^9}, {
   3.661539900505011*^9, 3.661539909332912*^9}, {3.661540221285543*^9, 
   3.6615402442294483`*^9}, 3.661540386943119*^9, 3.6615406370725393`*^9, 
   3.6615410607228937`*^9, 3.661611784193837*^9, 3.6616158721588297`*^9, 
   3.661637965088771*^9, 3.661704571819028*^9, 3.661810158922669*^9, 
   3.674840073444449*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"B0func0", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"1", "-", "EulerGamma", "+", 
   RowBox[{"Log", "[", 
    RowBox[{"4", " ", "\[Pi]"}], "]"}], "+", 
   FractionBox[
    RowBox[{"x", " ", 
     RowBox[{"Log", "[", "x", "]"}]}], 
    RowBox[{
     RowBox[{"-", "x"}], "+", "y"}]], "+", 
   FractionBox[
    RowBox[{"y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], 
    RowBox[{"x", "-", "y"}]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B0func0d12", "[", 
   RowBox[{"x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "EulerGamma"}], "-", 
   FractionBox["\[Epsilon]", 
    RowBox[{"2", " ", "x"}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "2"], 
    RowBox[{"6", " ", 
     SuperscriptBox["x", "2"]}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "3"], 
    RowBox[{"12", " ", 
     SuperscriptBox["x", "3"]}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "4"], 
    RowBox[{"20", " ", 
     SuperscriptBox["x", "4"]}]], "+", 
   RowBox[{"Log", "[", 
    RowBox[{"4", " ", "\[Pi]"}], "]"}], "-", 
   RowBox[{"Log", "[", "x", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B0func0d1", "[", "x_", "]"}], ":=", 
  RowBox[{"1", "-", "EulerGamma", "+", 
   RowBox[{"Log", "[", 
    RowBox[{"4", " ", "\[Pi]"}], "]"}], "-", 
   RowBox[{"Log", "[", "x", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B0func1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   FractionBox[
    RowBox[{"x", "+", "y"}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"x", " ", "y", " ", 
     RowBox[{"Log", "[", "x", "]"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "3"]], "+", 
   FractionBox[
    RowBox[{"x", " ", "y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "3"]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B0func1d12", "[", 
   RowBox[{"x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"6", " ", "x"}]], "-", 
   FractionBox["\[Epsilon]", 
    RowBox[{"12", " ", 
     SuperscriptBox["x", "2"]}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "2"], 
    RowBox[{"20", " ", 
     SuperscriptBox["x", "3"]}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "3"], 
    RowBox[{"30", " ", 
     SuperscriptBox["x", "4"]}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "4"], 
    RowBox[{"42", " ", 
     SuperscriptBox["x", "5"]}]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B0func1d1", "[", "x_", "]"}], ":=", 
  FractionBox["1", 
   RowBox[{"2", " ", "x"}]]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func0", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "EulerGamma"}], "+", 
      FractionBox[
       RowBox[{"2", " ", "x"}], 
       RowBox[{
        RowBox[{"-", "x"}], "+", "y"}]]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{"x", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["y", "2"]}]}], ")"}], " ", 
     RowBox[{"Log", "[", "2", "]"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "3"]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["y", "3"], " ", 
     RowBox[{"Log", "[", "16", "]"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "2"], " ", "y", " ", 
     RowBox[{"Log", "[", "4096", "]"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "3"]}]], "-", 
   FractionBox[
    RowBox[{"Log", "[", "\[Pi]", "]"}], "2"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "2"], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"y", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", "x"}], "+", "y"}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "2"]}]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func0d12", "[", 
   RowBox[{"x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["\[Epsilon]", 
    RowBox[{"3", " ", "x"}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "2"], 
    RowBox[{"8", " ", 
     SuperscriptBox["x", "2"]}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "3"], 
    RowBox[{"15", " ", 
     SuperscriptBox["x", "3"]}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "4"], 
    RowBox[{"24", " ", 
     SuperscriptBox["x", "4"]}]], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "EulerGamma"}], "-", 
      RowBox[{"Log", "[", "16", "]"}], "-", 
      RowBox[{"2", " ", 
       RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Log", "[", "x", "]"}]}]}], ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func0d1", "[", "x_", "]"}], ":=", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     RowBox[{"2", " ", "EulerGamma"}], "-", 
     RowBox[{"Log", "[", "16", "]"}], "-", 
     RowBox[{"2", " ", 
      RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"Log", "[", "x", "]"}]}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func0d2", "[", "x_", "]"}], ":=", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "3"}], "+", 
     RowBox[{"2", " ", "EulerGamma"}], "-", 
     RowBox[{"4", " ", 
      RowBox[{"Log", "[", "2", "]"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"Log", "[", "x", "]"}]}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{"5", " ", "x", " ", "y"}], "+", 
     SuperscriptBox["y", "2"]}], 
    RowBox[{"6", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "2"], " ", "y", " ", 
     RowBox[{"Log", "[", "x", "]"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "4"]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "2"], " ", "y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "4"]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func1d12", "[", 
   RowBox[{"x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", 
     RowBox[{"12", " ", "x"}]]}], "+", 
   FractionBox["\[Epsilon]", 
    RowBox[{"20", " ", 
     SuperscriptBox["x", "2"]}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "2"], 
    RowBox[{"30", " ", 
     SuperscriptBox["x", "3"]}]], "+", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "3"], 
    RowBox[{"42", " ", 
     SuperscriptBox["x", "4"]}]], "-", 
   FractionBox[
    SuperscriptBox["\[Epsilon]", "4"], 
    RowBox[{"56", " ", 
     SuperscriptBox["x", "5"]}]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func1d1", "[", "x_", "]"}], ":=", 
  RowBox[{"-", 
   FractionBox["1", 
    RowBox[{"6", " ", "x"}]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B1func1d2", "[", "x_", "]"}], ":=", 
  RowBox[{"-", 
   FractionBox["1", 
    RowBox[{"3", " ", "x"}]]}]}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQA2IQ/ea9kPmh168cKxr224HoS9MlXUD044uxviD6yxye
I5eBtNbKfDD9PKnlHIh+tVb4Aoi+8sXvKoj2a9x1F0SzHZvNdAVIb3m1jwdE
+4V+7Y9688pRqUN4IogOEDObCqIjVtpNA9EZjNVdV9xfO3rqVnSDaKP/IRNA
dN2NuZNBdN8JpQUgWuGJyDIQvWjnpVUgesqZhetAtHPoplsgepqX520Q/d56
puxVIP2sbAGY3vFkhgqIXhR7QR1EFz6t9gTRl0LYvUG00DPuDhDN0W7eA6Lf
/n+8FETvKb6+AkSLMxy5DaJ/ba+/B6IN3JxsfgHp4sZHtiD6tn9NLIj+Ij4X
TMdNvTYHRNeYey0E0QBN8Mnv
  "]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"B00funcAcc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S", "=", 
       RowBox[{"x", "+", "y"}]}]}], "}"}], ",", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"S", "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func0d1", "[", "y", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func0d1", "[", "x", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]", "/", "S"}], "]"}], "<", "0.01"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func0d12", "[", 
         RowBox[{"x", ",", "\[Epsilon]"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func0", "[", 
         RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
      "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B01funcAcc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S", "=", 
       RowBox[{"x", "+", "y"}]}]}], "}"}], ",", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"S", "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func1d1", "[", "y", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func1d1", "[", "x", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]", "/", "S"}], "]"}], "<", "0.003"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func1d12", "[", 
         RowBox[{"x", ",", "\[Epsilon]"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B0func1", "[", 
         RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
      "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B10funcAcc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S", "=", 
       RowBox[{"x", "+", "y"}]}]}], "}"}], ",", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"S", "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func0d1", "[", "y", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func0d2", "[", "x", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]", "/", "S"}], "]"}], "<", "0.003"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func0d12", "[", 
         RowBox[{"x", ",", "\[Epsilon]"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func0", "[", 
         RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
      "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B11funcAcc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S", "=", 
       RowBox[{"x", "+", "y"}]}]}], "}"}], ",", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"S", "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func1d1", "[", "y", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func1d2", "[", "x", "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]", "/", "S"}], "]"}], "<", "0.015"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func1d12", "[", 
         RowBox[{"x", ",", "\[Epsilon]"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"B1func1", "[", 
         RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
      "]"}], "]"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.661538903715921*^9, 3.6615390190805264`*^9}, {
   3.661539083689979*^9, 3.661539100825015*^9}, {3.6615392491692753`*^9, 
   3.661539355465824*^9}, {3.6615394078494883`*^9, 3.661539411849002*^9}, {
   3.661539539738447*^9, 3.661539551278204*^9}, {3.661539653023528*^9, 
   3.661539686163849*^9}, {3.661539870326373*^9, 3.661539886179266*^9}, {
   3.661540205423811*^9, 3.66154020581658*^9}, {3.6615405209798307`*^9, 
   3.661540548063777*^9}, 3.6615411481812077`*^9, {3.6615413562367764`*^9, 
   3.661541369937469*^9}, 3.66154151273381*^9, {3.661662037867322*^9, 
   3.661662044635399*^9}, {3.661704471443524*^9, 3.661704551764695*^9}, 
   3.661704643034915*^9, {3.661704716786459*^9, 3.661704753158381*^9}, {
   3.6618100622307568`*^9, 3.661810065944045*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparison", "Subsubsection",
 CellChangeTimes->{{3.661541944192935*^9, 3.661541945642829*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B1func0", "[", 
   RowBox[{
    SuperscriptBox["1000.0000000001", "2"], ",", 
    SuperscriptBox["1000", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B10func", "[", 
   RowBox[{
    SuperscriptBox["1000.0000000001", "2"], ",", 
    SuperscriptBox["1000", "2"]}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661539701323595*^9, 3.6615398620362186`*^9}, {
  3.661539937661075*^9, 3.661539950184641*^9}, {3.6615405616170473`*^9, 
  3.661540594531179*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00007899999999999999578982612380428918`1.9182270045700653", ",", 
   RowBox[{"-", "3.777893186295716`*^22"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661539731475686*^9, 3.6615398625149107`*^9}, {
   3.661539941055376*^9, 3.661539950419406*^9}, {3.661540565638238*^9, 
   3.661540595032103*^9}, 3.661541123583414*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00007899999999999999578982612380428918`1.9182270045700653", ",", 
   "5.930850987948292`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661539731475686*^9, 3.6615398625149107`*^9}, {
   3.661539941055376*^9, 3.661539950419406*^9}, {3.661540565638238*^9, 
   3.661540595032103*^9}, 3.661541123585472*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B1func0", "[", 
   RowBox[{
    SuperscriptBox["1000", "2"], ",", 
    SuperscriptBox["1000.01", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B10func", "[", 
   RowBox[{
    SuperscriptBox["1000.01", "2"], ",", 
    SuperscriptBox["1000", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B10func", "[", 
   RowBox[{
    SuperscriptBox["1000.01", "2"], ",", 
    SuperscriptBox["0.0001", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B10func", "[", 
   RowBox[{
    SuperscriptBox["1000.01", "2"], ",", "0"}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661539957976997*^9, 3.661539987566532*^9}, {
  3.66154007917419*^9, 3.661540083324288*^9}, {3.661540125564685*^9, 
  3.661540137483404*^9}, {3.661540573517632*^9, 3.661540583185655*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00006700000000000000226207941267375645`1.8466747159804504", ",", 
   "5.9901350570753`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661539960042878*^9, 3.6615399878542013`*^9}, {
   3.6615400719051113`*^9, 3.661540083629098*^9}, {3.661540119586774*^9, 
   3.661540137862624*^9}, 3.661540583960063*^9, 3.661541126406097*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00005199999999999999679986886769178511`1.7366032569144234", ",", 
   "5.930854321281592`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661539960042878*^9, 3.6615399878542013`*^9}, {
   3.6615400719051113`*^9, 3.661540083629098*^9}, {3.661540119586774*^9, 
   3.661540137862624*^9}, 3.661540583960063*^9, 3.661541126408658*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00005599999999999999915996015675823116`1.7687879402858244", ",", 
   "5.1808609878985745`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661539960042878*^9, 3.6615399878542013`*^9}, {
   3.6615400719051113`*^9, 3.661540083629098*^9}, {3.661540119586774*^9, 
   3.661540137862624*^9}, 3.661540583960063*^9, 3.661541126411838*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00004100000000000000047401318981066254`1.6333837699993596", ",", 
   "5.180860987898258`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661539960042878*^9, 3.6615399878542013`*^9}, {
   3.6615400719051113`*^9, 3.661540083629098*^9}, {3.661540119586774*^9, 
   3.661540137862624*^9}, 3.661540583960063*^9, 3.6615411264137583`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["1000.", "2"], "-", 
   SuperscriptBox["999.", "2"]}], 
  RowBox[{
   SuperscriptBox["1000.", "2"], "+", 
   SuperscriptBox["999.", "2"]}]]], "Input",
 CellChangeTimes->{{3.661541227593704*^9, 3.661541264364458*^9}, {
  3.6615413031797247`*^9, 3.661541306334529*^9}, {3.6615413391573343`*^9, 
  3.661541340688387*^9}}],

Cell[BoxData["0.00100049999974975`"], "Output",
 CellChangeTimes->{{3.661541260217881*^9, 3.661541264871562*^9}, 
   3.661541306738111*^9, 3.661541340986835*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"B0func1", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}], ",", 
     RowBox[{"B01func", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.", ",", "1001."}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"B1func0", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}], ",", 
     RowBox[{"B10func", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.", ",", "1001."}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"B1func1", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}], ",", 
     RowBox[{"B11func", "[", 
      RowBox[{
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "994.", ",", "1006."}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.6615399934859257`*^9, 3.661540058526366*^9}, {
  3.661540092910667*^9, 3.66154010895737*^9}, {3.661540157740184*^9, 
  3.6615401851766863`*^9}, {3.661540600962411*^9, 3.661540622092663*^9}, {
  3.661541050492454*^9, 3.66154105437284*^9}, {3.6615411633179407`*^9, 
  3.6615411745606203`*^9}}],

Cell[BoxData[{
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"B0func1", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}], "-", 
    RowBox[{"B01func", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "985.", ",", "998."}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"B0func1", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}], "-", 
    RowBox[{"B01func", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "985.", ",", "998."}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"B1func1", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}], "-", 
    RowBox[{"B11func", "[", 
     RowBox[{
      SuperscriptBox["1000.", "2"], ",", 
      SuperscriptBox["x", "2"]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "985.", ",", "996."}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.66154139630807*^9, 3.661541481677722*^9}, 
   3.661541529098984*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FFG", "=", "B11func"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"B1func1", "[", 
  RowBox[{
   SuperscriptBox["0.001", "2"], ",", 
   SuperscriptBox["1000.01", "2"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"FFG", "[", 
  RowBox[{
   SuperscriptBox["0.001", "2"], ",", 
   SuperscriptBox["1000.01", "2"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"FFG", "[", 
  RowBox[{
   SuperscriptBox["0.000", "2"], ",", 
   SuperscriptBox["1000.01", "2"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"FFG", "[", 
  RowBox[{
   SuperscriptBox["1000.01", "2"], ",", 
   SuperscriptBox["0.001", "2"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"FFG", "[", 
  RowBox[{
   SuperscriptBox["1000.01", "2"], ",", 
   SuperscriptBox["0.000", "2"]}], "]"}]}], "Input",
 CellChangeTimes->{{3.661540699328245*^9, 3.661540823843958*^9}, {
  3.661541132654311*^9, 3.661541134142138*^9}}],

Cell[BoxData[
 RowBox[{"-", "1.6666333338299935`*^-7"}]], "Output",
 CellChangeTimes->{{3.661540697111526*^9, 3.661540824493618*^9}, {
   3.661541134463955*^9, 3.6615411519638863`*^9}, 3.661541849845152*^9}],

Cell[BoxData[
 RowBox[{"-", "1.6666333338299935`*^-7"}]], "Output",
 CellChangeTimes->{{3.661540697111526*^9, 3.661540824493618*^9}, {
   3.661541134463955*^9, 3.6615411519638863`*^9}, 3.6615418498477488`*^9}],

Cell[BoxData[
 RowBox[{"-", "1.6666333338333268`*^-7"}]], "Output",
 CellChangeTimes->{{3.661540697111526*^9, 3.661540824493618*^9}, {
   3.661541134463955*^9, 3.6615411519638863`*^9}, 3.6615418498496733`*^9}],

Cell[BoxData[
 RowBox[{"-", "3.3332666674086865`*^-7"}]], "Output",
 CellChangeTimes->{{3.661540697111526*^9, 3.661540824493618*^9}, {
   3.661541134463955*^9, 3.6615411519638863`*^9}, 3.661541849852952*^9}],

Cell[BoxData[
 RowBox[{"-", "3.3332666676666536`*^-7"}]], "Output",
 CellChangeTimes->{{3.661540697111526*^9, 3.661540824493618*^9}, {
   3.661541134463955*^9, 3.6615411519638863`*^9}, 3.661541849855023*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Compilation", "Subsubsection",
 CellChangeTimes->{{3.6615419559763107`*^9, 3.661541957532312*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"B00comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"B00funcAcc", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B01comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"B01funcAcc", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B10comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"B10funcAcc", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B11comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"B11funcAcc", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "]"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.661542064974391*^9, 3.661542121168898*^9}, {
  3.661542189792481*^9, 3.661542205546473*^9}, {3.66170460733854*^9, 
  3.661704620830823*^9}, {3.66170477461548*^9, 3.661704776205708*^9}, {
  3.661810164419045*^9, 3.6618101677210197`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B00comp", "[", 
   RowBox[{"1300.", ",", "1000."}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B01comp", "[", 
   RowBox[{"1300.", ",", "1000."}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B10comp", "[", 
   RowBox[{"1300.", ",", "1000."}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"B11comp", "[", 
   RowBox[{"1300.", ",", "1000."}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6617045792415857`*^9, 3.661704595761674*^9}, {
  3.661704659627853*^9, 3.661704664318894*^9}, {3.661704767844326*^9, 
  3.661704768636148*^9}, {3.661706537063808*^9, 3.661706557073097*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001299999999999999919996721692294628`1.1345432655864605", ",", 
   RowBox[{"-", "5.090858509606843`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661704591331592*^9, 3.661704596476303*^9}, 
   3.6617046296715517`*^9, 3.661704664651885*^9, {3.661704764047078*^9, 
   3.661704781501109*^9}, {3.66170654656849*^9, 3.661706557364978*^9}, {
   3.661810171035014*^9, 3.661810173356036*^9}, {3.6748400901016817`*^9, 
   3.674840093602888*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6.00000000000000015200514458246772`0.7987511636632681*^-6", ",", 
   "0.00014542430341707435`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661704591331592*^9, 3.661704596476303*^9}, 
   3.6617046296715517`*^9, 3.661704664651885*^9, {3.661704764047078*^9, 
   3.661704781501109*^9}, {3.66170654656849*^9, 3.661706557364978*^9}, {
   3.661810171035014*^9, 3.661810173356036*^9}, {3.6748400901016817`*^9, 
   3.674840093605336*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6.00000000000000015200514458246772`0.7987511636632681*^-6", ",", 
   "2.5236156092908715`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661704591331592*^9, 3.661704596476303*^9}, 
   3.6617046296715517`*^9, 3.661704664651885*^9, {3.661704764047078*^9, 
   3.661704781501109*^9}, {3.66170654656849*^9, 3.661706557364978*^9}, {
   3.661810171035014*^9, 3.661810173356036*^9}, {3.6748400901016817`*^9, 
   3.674840093606904*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5.00000000000000040901526957015655`0.7195699176156426*^-6", ",", 
   RowBox[{"-", "0.00007461642591843898`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661704591331592*^9, 3.661704596476303*^9}, 
   3.6617046296715517`*^9, 3.661704664651885*^9, {3.661704764047078*^9, 
   3.661704781501109*^9}, {3.66170654656849*^9, 3.661706557364978*^9}, {
   3.661810171035014*^9, 3.661810173356036*^9}, {3.6748400901016817`*^9, 
   3.6748400936084633`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"BFunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "B0func0", ",", "B0func0d12", ",", "B0func0d1", ",", "B0func1", ",", 
     "B0func1d12", ",", "B0func1d1", ",", "B1func0", ",", "B1func0d12", ",", 
     "B1func0d1", ",", "B1func0d2", ",", "\[IndentingNewLine]", "B1func1", 
     ",", "B1func1d12", ",", "B1func1d1", ",", "B1func1d2", ",", "B00funcAcc",
      ",", "B01funcAcc", ",", "B10funcAcc", ",", "B11funcAcc"}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.661615159974884*^9, 3.6616151603003683`*^9}, {
  3.661637990693103*^9, 3.661638001253957*^9}, {3.661810069387377*^9, 
  3.661810074362661*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Need 3-pt integrals C0, C1, C2, C00 for p1^2 = p2^2 = 0 and general m1, m2, \
m3 and expanded to linear order around p1dotp2=0\
\>", "Subsection",
 CellChangeTimes->{{3.574023579872418*^9, 3.574023598464611*^9}, 
   3.574184334934391*^9, {3.574882525191081*^9, 3.574882540855279*^9}}],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection",
 CellChangeTimes->{{3.6615463808173513`*^9, 3.661546385228072*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"C0", " ", "with", " ", 
     SuperscriptBox["p1", "2"]}], "=", 
    RowBox[{
     SuperscriptBox["p2", "2"], "=", 
     RowBox[{
      RowBox[{
      "0", " ", "and", " ", "expanded", " ", "to", " ", "first", " ", "order",
        " ", "around", " ", "p1dotp2"}], "=", "0"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"integrandC0temp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Normal", "[", 
             RowBox[{"Simplify", "[", 
              RowBox[{"Series", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"1", "/", 
                   RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
                  RowBox[{"(", 
                   RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp1"}], "+", "m2"}], 
                   ")"}]}], "/", 
                 RowBox[{"(", 
                  RowBox[{
                   SuperscriptBox["k", "2"], "+", 
                   RowBox[{"2", "\[Epsilon]", " ", "kdotp2"}], "+", "m3"}], 
                  ")"}]}], ",", 
                RowBox[{"{", 
                 RowBox[{"\[Epsilon]", ",", "0", ",", "2"}], "}"}]}], "]"}], 
              "]"}], "]"}], "/.", 
            RowBox[{
             SuperscriptBox["\[Epsilon]", "2"], "\[Rule]", "1"}]}], "/.", 
           RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "/.", 
          RowBox[{
           SuperscriptBox["kdotp1", "2"], "\[Rule]", "0"}]}], "/.", 
         RowBox[{
          SuperscriptBox["kdotp2", "2"], "\[Rule]", "0"}]}], "/.", 
        RowBox[{
         RowBox[{"kdotp1", " ", "kdotp2"}], "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "d"}], " ", 
          SuperscriptBox["k", "2"], "p1dotp2"}]}]}], "/.", 
       RowBox[{"d", "\[Rule]", 
        RowBox[{"4", "-", "\[Delta]"}]}]}], "/.", 
      RowBox[{"p1dotp2", "\[Rule]", 
       RowBox[{
        RowBox[{"-", "qsq"}], "/", "2"}]}]}], ",", "qsq", ",", 
     "FullSimplify"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.574897145698737*^9, 3.574897187536634*^9}, {
  3.5748973919474897`*^9, 3.574897394916353*^9}, {3.57489873915858*^9, 
  3.574898755726925*^9}, {3.5776699624455433`*^9, 3.5776699712051086`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m3"}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{"2", " ", 
    SuperscriptBox["k", "2"], " ", "qsq"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]]}]], "Output",
 CellChangeTimes->{
  3.5748971480726013`*^9, 3.574897188067897*^9, 3.5748973952454357`*^9, 
   3.574903924283656*^9, 3.575024417169873*^9, 3.575037168602854*^9, 
   3.575039334929381*^9, 3.575043095154052*^9, 3.5750714697154293`*^9, 
   3.5751217012262917`*^9, 3.575199783544715*^9, 3.575410783696611*^9, 
   3.575414016754126*^9, 3.575414090162072*^9, 3.575431632601409*^9, 
   3.575477359278277*^9, 3.575478058302403*^9, 3.5754821043845863`*^9, 
   3.575484766095099*^9, {3.575485370070292*^9, 3.575485390569195*^9}, 
   3.575491189117011*^9, 3.575503795982009*^9, 3.575503962205645*^9, 
   3.5757411840977697`*^9, 3.575742892575355*^9, 3.5758185369649343`*^9, 
   3.575927144930654*^9, 3.575928273837411*^9, 3.575930566567824*^9, 
   3.575931311395584*^9, 3.575932803636202*^9, 3.575933089060338*^9, 
   3.575933676094315*^9, 3.575934956181621*^9, 3.575935240772909*^9, 
   3.575935876564817*^9, 3.575938358295298*^9, 3.575938622362239*^9, 
   3.575939414425799*^9, 3.5759402706837587`*^9, 3.575940606588168*^9, 
   3.575943007891754*^9, 3.575944518071342*^9, 3.575945938071208*^9, 
   3.576007706241067*^9, 3.576008620469935*^9, 3.5760150652336607`*^9, 
   3.576097311299911*^9, 3.576099972140047*^9, 3.576192436945587*^9, 
   3.576194999846745*^9, 3.57619877768997*^9, 3.576245993225621*^9, {
   3.576251923452886*^9, 3.5762519388191223`*^9}, 3.576253348401009*^9, 
   3.576359015873736*^9, 3.576363200732362*^9, 3.576364096810697*^9, 
   3.576364222808535*^9, 3.57636430392715*^9, 3.576364442289371*^9, 
   3.576372544226355*^9, 3.5764291322963467`*^9, 3.576536708251452*^9, 
   3.576539475327636*^9, 3.576539810992508*^9, 3.576690430941389*^9, 
   3.576692897414905*^9, 3.576712135889247*^9, 3.576795742354631*^9, 
   3.576983737685679*^9, 3.576983816630754*^9, 3.577215458396619*^9, 
   3.577278672041518*^9, 3.577283264755928*^9, 3.577296658768396*^9, 
   3.577383124155705*^9, 3.577401294547724*^9, 3.5776629313427362`*^9, 
   3.577663001736299*^9, 3.577669971516794*^9, 3.577675347002562*^9, 
   3.57767695171037*^9, 3.577816815140456*^9, 3.5778344786924067`*^9, 
   3.5779158530885677`*^9, 3.577928337073476*^9, 3.5779307998706627`*^9, 
   3.577931056204967*^9, 3.577934515825535*^9, 3.578330295542039*^9, 
   3.578342611323728*^9, 3.578344523246283*^9, 3.578494201206201*^9, 
   3.578756819578004*^9, 3.5788499573020697`*^9, 3.578850470373294*^9, 
   3.5788674842878942`*^9, 3.578953606529396*^9, 3.579005040321274*^9, 
   3.57900589727258*^9, 3.5790060588886757`*^9, 3.5790064523005123`*^9, 
   3.579007234414875*^9, 3.5790239329833403`*^9, 3.5791951466207848`*^9, 
   3.579216908699592*^9, 3.579282515553876*^9, 3.579283965873014*^9, 
   3.579299398324277*^9, {3.579303890213739*^9, 3.579303908241888*^9}, 
   3.5794523882869387`*^9, 3.579456351734716*^9, 3.579820883707974*^9, 
   3.579824010462659*^9, 3.580008812742504*^9, 3.580439168259222*^9, 
   3.580596056021659*^9, 3.580642968433015*^9, 3.580671590363386*^9, 
   3.580699193424552*^9, 3.580845936507781*^9, 3.581172079100061*^9, 
   3.581190361980523*^9, 3.581195616629908*^9, 3.58126897398454*^9, 
   3.581270928294229*^9, 3.5812763047743607`*^9, 3.5812918091517*^9, 
   3.5812955132767487`*^9, {3.581297622610643*^9, 3.581297649749373*^9}, 
   3.581298248507359*^9, 3.581298848183468*^9, 3.581346633140215*^9, 
   3.58136351998829*^9, 3.581369186696898*^9, 3.581369576723339*^9, 
   3.581442003963849*^9, 3.581450802313902*^9, 3.581757917514182*^9, 
   3.581915393746087*^9, 3.582021741739171*^9, 3.58248346145119*^9, 
   3.584065499096163*^9, 3.5841098278122883`*^9, 3.584110059134514*^9, 
   3.584208682873699*^9, 3.584208891683762*^9, 3.584211091009077*^9, 
   3.584211267892787*^9, 3.584213610121166*^9, 3.5842141571299553`*^9, 
   3.5842214164966717`*^9, 3.584224090116034*^9, 3.5842797645059013`*^9, 
   3.584317076267603*^9, 3.584362237893867*^9, 3.584383354524911*^9, 
   3.584448807849901*^9, 3.5844533037334003`*^9, 3.584453386222417*^9, 
   3.58445359193292*^9, 3.584453831366433*^9, 3.584479568427314*^9, 
   3.5844851783066*^9, 3.584708492465097*^9, 3.584720378083254*^9, 
   3.584725512986198*^9, 3.5848746977595453`*^9, 3.5848747518258057`*^9, 
   3.58487600432936*^9, 3.584879548269713*^9, 3.591549616229661*^9, 
   3.6037557106493397`*^9, 3.6037574510589027`*^9, 3.603849874154615*^9, 
   3.6038502002969112`*^9, 3.603853950355564*^9, 3.6039255777520323`*^9, 
   3.604157597433237*^9, 3.6042363928341427`*^9, 3.605050446218274*^9, 
   3.605452230285342*^9, 3.605453007370535*^9, 3.612794824518709*^9, 
   3.612794955721298*^9, 3.612798666470744*^9, 3.612798791123231*^9, 
   3.612800462127021*^9, 3.6133202942316713`*^9, 3.613578436790884*^9, 
   3.661542383701274*^9, 3.6616119636022377`*^9, 3.661615876044319*^9, 
   3.6616380108853197`*^9, 3.6617048060054502`*^9, 3.661810186285831*^9, 
   3.674840103596711*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0temp", "=", 
  RowBox[{"c0", " ", 
   RowBox[{"Intfuncgen", "[", "integrandC0temp", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.575479584854576*^9, 3.5754795944663277`*^9}, {
  3.575479648296536*^9, 3.575479648959216*^9}, {3.575482121217411*^9, 
  3.5754821213526897`*^9}, {3.575482209627096*^9, 3.5754822099219923`*^9}, {
  3.575484562751299*^9, 3.575484563679084*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "m2", " ", "m3"}], "-", 
        RowBox[{"m1", " ", 
         RowBox[{"(", 
          RowBox[{"m2", "+", "m3"}], ")"}]}]}], ")"}], " ", "qsq"}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["m1", 
        RowBox[{"16", " ", 
         RowBox[{"(", 
          RowBox[{"m1", "-", "m2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{"m1", "-", "m3"}], ")"}], " ", 
         SuperscriptBox["\[Pi]", "2"]}]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["m1", "2"], " ", "qsq"}], 
        RowBox[{"32", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["m2", 
         RowBox[{"16", " ", 
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], " ", 
          SuperscriptBox["\[Pi]", "2"]}]]}], "-", 
       FractionBox[
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["m2", "2"], "-", 
           RowBox[{"2", " ", "m1", " ", "m3"}], "+", 
           RowBox[{"m2", " ", "m3"}]}], ")"}], " ", "qsq"}], 
        RowBox[{"32", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "3"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["m3", 
         RowBox[{"16", " ", 
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "m2"}], "+", "m3"}], ")"}], " ", 
          SuperscriptBox["\[Pi]", "2"]}]]}], "-", 
       FractionBox[
        RowBox[{"m3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", "m1", " ", "m2"}], "+", 
           RowBox[{"m3", " ", 
            RowBox[{"(", 
             RowBox[{"m2", "+", "m3"}], ")"}]}]}], ")"}], " ", "qsq"}], 
        RowBox[{"32", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "m2"}], "+", "m3"}], ")"}], "3"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m3", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.575479594847238*^9, 3.575479637396659*^9}, {
   3.575479715748292*^9, 3.575479732502356*^9}, 3.57547988105416*^9, {
   3.575482107915501*^9, 3.575482122095173*^9}, {3.575482191210232*^9, 
   3.5754822177962933`*^9}, 3.575482704098876*^9, 3.57548456738776*^9, 
   3.5754847670328007`*^9, {3.5754853703476973`*^9, 3.575485390675375*^9}, 
   3.5754911894838142`*^9, 3.57550379626178*^9, 3.575503962490556*^9, 
   3.575741184475408*^9, 3.575742892989759*^9, 3.57581853733773*^9, 
   3.5759271453126383`*^9, 3.575928274223325*^9, 3.575930566940432*^9, 
   3.5759313118375893`*^9, 3.575932804019916*^9, 3.575933090120858*^9, 
   3.575933676469391*^9, 3.575934956606379*^9, 3.575935241184094*^9, 
   3.575935877001172*^9, 3.575938358720821*^9, 3.575938622759902*^9, 
   3.575939414825679*^9, 3.575940271057618*^9, 3.575940606963523*^9, 
   3.5759430082667437`*^9, 3.5759445184522457`*^9, 3.5759459384478273`*^9, 
   3.576007706625781*^9, 3.576008620860713*^9, 3.576015065613367*^9, 
   3.576097311673244*^9, 3.576099972690576*^9, 3.576192437342389*^9, 
   3.576195000221588*^9, 3.576198778134845*^9, 3.576245993602582*^9, {
   3.576251923828061*^9, 3.576251939183343*^9}, 3.576253348809284*^9, 
   3.576359016315908*^9, 3.5763632011287413`*^9, 3.576364097246559*^9, 
   3.576364223215713*^9, 3.576364304346876*^9, 3.5763644426735573`*^9, 
   3.576372544607975*^9, 3.5764291326780653`*^9, 3.5765367086225653`*^9, 
   3.576539475699189*^9, 3.57653981136693*^9, 3.576690431320619*^9, 
   3.576692897788866*^9, 3.576712136267928*^9, 3.576795742710799*^9, 
   3.576983737951235*^9, 3.5769838169248037`*^9, 3.577215458808367*^9, 
   3.5772786723101873`*^9, 3.577283265180861*^9, 3.57729665905128*^9, 
   3.5773831244479094`*^9, 3.57740129482316*^9, 3.577662935620831*^9, 
   3.5776630051354313`*^9, 3.577669983482614*^9, 3.577675351636354*^9, 
   3.5776769564662027`*^9, 3.5778168205116673`*^9, 3.577834483614785*^9, 
   3.577915857642144*^9, 3.577928342278665*^9, 3.577930804653612*^9, 
   3.577931057316371*^9, 3.577934516946933*^9, 3.5783302966676493`*^9, 
   3.57834261248985*^9, 3.578344524351026*^9, 3.578494202282445*^9, 
   3.5787568208804483`*^9, 3.5788499584453173`*^9, 3.578850471656389*^9, 
   3.578867485397768*^9, 3.578953607970195*^9, 3.57900504153522*^9, 
   3.5790058983788147`*^9, 3.579006060057941*^9, 3.579006453021908*^9, 
   3.579007234487406*^9, 3.579023934064056*^9, 3.579195147750002*^9, 
   3.579216909799162*^9, 3.579282516632875*^9, 3.5792839684285917`*^9, 
   3.5792994004215*^9, {3.57930389141523*^9, 3.579303908331154*^9}, 
   3.579452389391515*^9, 3.579456352840342*^9, 3.579820884794875*^9, 
   3.579824011609057*^9, 3.580008813877556*^9, 3.580439169312058*^9, 
   3.5805960573187943`*^9, 3.580642969534569*^9, 3.580671591428751*^9, 
   3.580699194673365*^9, 3.5808459378832283`*^9, 3.581172080439109*^9, 
   3.5811903633109217`*^9, 3.581195617717355*^9, 3.581268975102098*^9, 
   3.581270929403399*^9, 3.581276305885219*^9, 3.581291810332354*^9, 
   3.581295514388138*^9, {3.581297623722518*^9, 3.58129765080888*^9}, 
   3.581298249620181*^9, 3.5812988493083057`*^9, 3.581346634492612*^9, 
   3.581363521528816*^9, 3.581369188119364*^9, 3.5813695781981277`*^9, 
   3.581442005273967*^9, 3.581450803810444*^9, 3.581757919038844*^9, 
   3.5819153954210653`*^9, 3.582021743106496*^9, 3.582483462879867*^9, 
   3.584065500605393*^9, 3.584109829463416*^9, 3.584110060654933*^9, 
   3.584208684398136*^9, 3.584208893206996*^9, 3.584211092538364*^9, 
   3.584211269393071*^9, 3.584213611619278*^9, 3.584214158666337*^9, 
   3.5842214180116*^9, 3.584224091642034*^9, 3.584279766069544*^9, 
   3.5843170778312597`*^9, 3.584362239454967*^9, 3.584383356792844*^9, 
   3.584448809364979*^9, 3.584453305382552*^9, 3.5844533878854647`*^9, 
   3.584453593475223*^9, 3.584453832891305*^9, 3.584479569997817*^9, 
   3.584485179830591*^9, 3.584708494017764*^9, 3.5847203796176453`*^9, 
   3.584725514524268*^9, 3.584874699294819*^9, 3.5848747533339233`*^9, 
   3.584876005834486*^9, 3.584879549787281*^9, 3.5915496178775377`*^9, 
   3.603755712216674*^9, 3.6037574524949913`*^9, 3.603849877536108*^9, 
   3.6038502022792587`*^9, 3.603853951865204*^9, 3.6039255791825447`*^9, 
   3.6041575988498907`*^9, 3.6042363944476547`*^9, 3.60505044780527*^9, 
   3.6054522318852987`*^9, 3.605453008965131*^9, 3.61279482621525*^9, 
   3.6127949563401527`*^9, 3.612798669996023*^9, 3.612798792847636*^9, 
   3.612800463592757*^9, 3.613320295703375*^9, 3.613578438200706*^9, 
   3.66154238683458*^9, 3.6616119654571123`*^9, 3.661615877935609*^9, 
   3.6616380129908667`*^9, 3.6617048078379097`*^9, 3.661810187995844*^9, 
   3.674840105569612*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"C1", " ", "and", " ", "C2", " ", "with", " ", 
     SuperscriptBox["p1", "2"]}], "=", 
    RowBox[{
     SuperscriptBox["p2", "2"], "=", 
     RowBox[{
      RowBox[{
      "0", " ", "and", " ", "expanded", " ", "to", " ", "first", " ", "order",
        " ", "around", " ", "p1dotp2"}], "=", "0"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Use", " ", 
     RowBox[{"identity", ":", " ", 
      RowBox[{"\[Integral]", " ", 
       RowBox[{
        SuperscriptBox["d", "4"], "k", " ", 
        SuperscriptBox["k", "\[Mu]"], 
        SuperscriptBox["k", "\[Nu]"], 
        SuperscriptBox["k", "\[Rho]"], 
        SuperscriptBox["k", "\[Sigma]"], 
        RowBox[{"f", "[", 
         SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "24"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Eta]", "\[Mu]\[Nu]"], 
        SuperscriptBox["\[Eta]", "\[Rho]\[Sigma]"]}], "+", 
       RowBox[{
        SuperscriptBox["\[Eta]", "\[Mu]\[Rho]"], 
        SuperscriptBox["\[Eta]", "\[Nu]\[Sigma]"]}], "+", 
       RowBox[{
        SuperscriptBox["\[Eta]", "\[Mu]\[Sigma]"], 
        SuperscriptBox["\[Eta]", "\[Nu]\[Rho]"]}]}], ")"}], 
     RowBox[{"\[Integral]", " ", 
      RowBox[{
       SuperscriptBox["d", "4"], "k", " ", 
       SuperscriptBox["k", "4"], 
       RowBox[{"f", "[", 
        SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "*)"}], 
  RowBox[{"integrandCitemp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Expand", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Normal", "[", 
                 RowBox[{"Series", "[", 
                  RowBox[{
                   RowBox[{"\[Epsilon]", " ", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"k\[Mu]", "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp1"}], "+", "m2"}], 
                    ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp2"}], "+", "m3"}], 
                    ")"}]}]}], ",", 
                   RowBox[{"{", 
                    RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], 
                  "]"}], "]"}], "/.", 
                RowBox[{
                 SuperscriptBox["\[Epsilon]", "k_"], "\[RuleDelayed]", 
                 RowBox[{"1", "/;", 
                  RowBox[{"EvenQ", "[", "k", "]"}]}]}]}], "/.", 
               RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "/.", 
             RowBox[{
              SuperscriptBox["kdotp2", "3"], "\[Rule]", "0"}]}], "/.", 
            RowBox[{
             SuperscriptBox["kdotp1", "3"], "\[Rule]", "0"}]}], "/.", 
           RowBox[{
            RowBox[{"kdotp1", " ", 
             SuperscriptBox["kdotp2", "2"], " ", "k\[Mu]"}], "\[Rule]", 
            RowBox[{
             RowBox[{
              RowBox[{"1", "/", "d"}], "/", 
              RowBox[{"(", 
               RowBox[{"d", "+", "2"}], ")"}]}], 
             RowBox[{"(", 
              RowBox[{"2", "p2\[Mu]", " ", "p1dotp2", " ", 
               SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
          RowBox[{
           RowBox[{"kdotp2", " ", 
            SuperscriptBox["kdotp1", "2"], " ", "k\[Mu]"}], "\[Rule]", 
           RowBox[{
            RowBox[{
             RowBox[{"1", "/", "d"}], "/", 
             RowBox[{"(", 
              RowBox[{"d", "+", "2"}], ")"}]}], 
            RowBox[{"(", 
             RowBox[{"2", "p1\[Mu]", " ", "p1dotp2", " ", 
              SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
         RowBox[{
          RowBox[{"kdotp1", " ", "k\[Mu]"}], "\[Rule]", 
          RowBox[{
           RowBox[{"1", "/", "d"}], " ", "p1\[Mu]", " ", 
           SuperscriptBox["k", "2"]}]}]}], "/.", 
        RowBox[{
         RowBox[{"kdotp2", " ", "k\[Mu]"}], "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "d"}], " ", "p2\[Mu]", " ", 
          SuperscriptBox["k", "2"]}]}]}], "/.", 
       RowBox[{"d", "\[Rule]", 
        RowBox[{"4", "-", "\[Delta]"}]}]}], "/.", 
      RowBox[{"p1dotp2", "\[Rule]", 
       RowBox[{
        RowBox[{"-", "qsq"}], "/", "2"}]}]}], ",", "qsq", ",", 
     "FullSimplify"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5748976889729156`*^9, 3.574897691258231*^9}, {
  3.574897729583708*^9, 3.574897823658194*^9}, {3.574898685850213*^9, 
  3.574898691201762*^9}, {3.574898766358665*^9, 3.574898876629177*^9}, {
  3.574900204022066*^9, 3.574900207477779*^9}, {3.574901764123354*^9, 
  3.574901768636355*^9}, {3.577670198760721*^9, 3.57767021254214*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"2", " ", 
    SuperscriptBox["k", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"m3", " ", "p1\[Mu]"}], "+", 
      RowBox[{"m2", " ", "p2\[Mu]"}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"p1\[Mu]", "+", "p2\[Mu]"}], ")"}]}]}], ")"}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{"8", " ", 
    SuperscriptBox["k", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"m3", " ", "p1\[Mu]"}], "+", 
      RowBox[{"m2", " ", "p2\[Mu]"}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"p1\[Mu]", "+", "p2\[Mu]"}], ")"}]}]}], ")"}], " ", "qsq"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "3"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]]}]], "Output",
 CellChangeTimes->{
  3.574897694554819*^9, {3.574897729876338*^9, 3.574897823914639*^9}, {
   3.5748988114166813`*^9, 3.5748988769702797`*^9}, 3.57490020866156*^9, 
   3.574901769246704*^9, 3.574903937167047*^9, 3.57502443088159*^9, 
   3.575037182696754*^9, 3.57503934947077*^9, 3.575043109302827*^9, 
   3.575071484006803*^9, 3.5751217153244877`*^9, 3.575199796767825*^9, 
   3.575410797278654*^9, 3.5754140298781557`*^9, 3.5754141037017193`*^9, 
   3.575431645508463*^9, 3.575477373195034*^9, 3.575478072218609*^9, 
   3.5754822248334103`*^9, 3.575484775111083*^9, {3.575485370506456*^9, 
   3.575485390752357*^9}, 3.575491189627239*^9, 3.575503796365918*^9, 
   3.575503962653985*^9, 3.57574118467192*^9, 3.57574289319025*^9, 
   3.575818537523657*^9, 3.575927145435795*^9, 3.575928274413784*^9, 
   3.5759305671405888`*^9, 3.5759313119664917`*^9, 3.575932804234084*^9, 
   3.575933090297289*^9, 3.575933676669478*^9, 3.5759349567293777`*^9, 
   3.5759352414580107`*^9, 3.575935877126739*^9, 3.575938358931305*^9, 
   3.575938623039089*^9, 3.575939415101828*^9, 3.5759402712534122`*^9, 
   3.5759406070897217`*^9, 3.575943008453589*^9, 3.57594451862199*^9, 
   3.575945938568503*^9, 3.576007706748529*^9, 3.5760086209792643`*^9, 
   3.5760150657921124`*^9, 3.576097311799138*^9, 3.5760999735385723`*^9, 
   3.576192437510195*^9, 3.576195000340208*^9, 3.576198778248827*^9, 
   3.57624599371978*^9, {3.5762519240073347`*^9, 3.576251939372117*^9}, 
   3.576253348987006*^9, 3.576359016435689*^9, 3.576363201318983*^9, 
   3.5763640974400053`*^9, 3.576364223409853*^9, 3.576364304520144*^9, 
   3.576364442864559*^9, 3.576372544765291*^9, 3.576429132851812*^9, 
   3.5765367087431107`*^9, 3.576539475864398*^9, 3.576539811564187*^9, 
   3.576690431449093*^9, 3.5766928979532948`*^9, 3.576712136428516*^9, 
   3.576795742824913*^9, 3.5769837381052437`*^9, 3.576983817035984*^9, 
   3.577215459074924*^9, 3.577278672412168*^9, 3.577283265357583*^9, 
   3.5772966592646923`*^9, 3.5773831245587587`*^9, 3.577401294928899*^9, 
   3.5776629383588*^9, 3.577663005440504*^9, 3.577670212849998*^9, 
   3.577675351897521*^9, 3.57767695681089*^9, 3.5778168207099953`*^9, 
   3.577834483874399*^9, 3.57791585791654*^9, 3.5779283425828047`*^9, 
   3.5779308049184237`*^9, 3.5779310575775757`*^9, 3.577934517226025*^9, 
   3.578330296898347*^9, 3.578342612735879*^9, 3.578344524540621*^9, 
   3.578494202537354*^9, 3.578756821045464*^9, 3.578849958665052*^9, 
   3.578850471811002*^9, 3.5788674855820704`*^9, 3.578953608175374*^9, 
   3.579005041736212*^9, 3.579005898553217*^9, 3.5790060603043118`*^9, 
   3.579006453132428*^9, 3.579007234564199*^9, 3.579023934278008*^9, 
   3.579195147971525*^9, 3.579216910038802*^9, 3.579282516921422*^9, 
   3.5792839686229887`*^9, 3.579299401804196*^9, {3.5793038921470823`*^9, 
   3.579303908407893*^9}, 3.579452390278232*^9, 3.579456353592074*^9, 
   3.579820885534111*^9, 3.57982401265423*^9, 3.5800088146292973`*^9, 
   3.5804391699545727`*^9, 3.580596058097227*^9, 3.58064297022059*^9, 
   3.58067159211171*^9, 3.580699195685524*^9, 3.580845938742909*^9, 
   3.58117208122118*^9, 3.581190364170301*^9, 3.581195618592299*^9, 
   3.581268975834857*^9, 3.5812709300569477`*^9, 3.581276306591577*^9, 
   3.581291811083349*^9, 3.581295515056727*^9, {3.581297624098563*^9, 
   3.581297651480626*^9}, 3.5812982502675734`*^9, 3.581298849985106*^9, 
   3.581346635612508*^9, 3.5813635228091583`*^9, 3.581369189298553*^9, 
   3.58136957940625*^9, 3.581442006132201*^9, 3.581450805062904*^9, 
   3.581757920801672*^9, 3.5819153971578007`*^9, 3.582021744551075*^9, 
   3.5824834644723*^9, 3.5840655020594177`*^9, 3.584109831076742*^9, 
   3.584110062108734*^9, 3.584208685854022*^9, 3.584208894649384*^9, 
   3.584211093978402*^9, 3.584211270855173*^9, 3.584213613100418*^9, 
   3.584214160144719*^9, 3.584221419498605*^9, 3.584224093195972*^9, 
   3.584279767611848*^9, 3.5843170795001497`*^9, 3.5843622409924726`*^9, 
   3.584383358297223*^9, 3.5844488108549213`*^9, 3.584453307542584*^9, 
   3.584453389994856*^9, 3.5844535949845057`*^9, 3.5844538343982363`*^9, 
   3.5844795714936657`*^9, 3.58448518131697*^9, 3.584708495535576*^9, 
   3.5847203810761623`*^9, 3.584725516048696*^9, 3.584874700742361*^9, 
   3.5848747548209333`*^9, 3.584876007280059*^9, 3.584879551290291*^9, 
   3.591549619509108*^9, 3.603755713957312*^9, 3.603757454323119*^9, 
   3.603849879068503*^9, 3.6038502040267076`*^9, 3.603853953518382*^9, 
   3.603925580705411*^9, 3.604157600421795*^9, 3.604236396150535*^9, 
   3.605050449409163*^9, 3.6054522334173193`*^9, 3.605453010486803*^9, 
   3.612794827051921*^9, 3.612794957750637*^9, 3.612798675942835*^9, 
   3.612798793201537*^9, 3.612800463868018*^9, 3.613320295952928*^9, 
   3.613578438420072*^9, 3.6615423895182447`*^9, 3.661611965947885*^9, 
   3.66161587841294*^9, 3.6616380136856403`*^9, 3.6617048082921*^9, 
   3.66181018827216*^9, 3.674840106148406*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C1temp", "=", 
  RowBox[{
   RowBox[{"c1", " ", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"Intfuncgen", "[", "integrandCitemp", "]"}], ",", "p1\[Mu]"}], 
     "]"}]}], "/.", "subtry"}]}], "\[IndentingNewLine]", 
 RowBox[{"C2temp", "=", 
  RowBox[{
   RowBox[{"c1", " ", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"Intfuncgen", "[", "integrandCitemp", "]"}], ",", "p2\[Mu]"}], 
     "]"}]}], "/.", "subtry"}]}]}], "Input",
 CellChangeTimes->{{3.5754822310664673`*^9, 3.575482268068571*^9}, {
  3.575482933862275*^9, 3.575482963701426*^9}, {3.575484574024372*^9, 
  3.5754845790335503`*^9}, {3.577296613684626*^9, 3.577296616681299*^9}, {
  3.577675310653458*^9, 3.577675311019451*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["m2", 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m3"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["m2", "2"]}], " ", "m3", " ", 
         RowBox[{"(", 
          RowBox[{"m2", "+", 
           RowBox[{"5", " ", "m3"}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["m1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["m2", "2"], "-", 
           RowBox[{"5", " ", "m2", " ", "m3"}], "-", 
           RowBox[{"2", " ", 
            SuperscriptBox["m3", "2"]}]}], ")"}]}], "+", 
        RowBox[{"m1", " ", "m2", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["m2", "2"], "+", 
           RowBox[{"2", " ", "m2", " ", "m3"}], "+", 
           RowBox[{"9", " ", 
            SuperscriptBox["m3", "2"]}]}], ")"}]}]}], ")"}], " ", "qsq"}], 
     RowBox[{"96", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "3"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["m1", "2"], 
         RowBox[{"32", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], " ", 
          SuperscriptBox["\[Pi]", "2"]}]]}], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["m1", "3"], " ", "qsq"}], 
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m1", " ", 
            RowBox[{"(", 
             RowBox[{"m2", "-", 
              RowBox[{"2", " ", "m3"}]}], ")"}]}], "+", 
           RowBox[{"m2", " ", "m3"}]}], ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]], "+", 
       FractionBox[
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", 
            SuperscriptBox["m1", "2"], " ", 
            SuperscriptBox["m3", "2"]}], "+", 
           RowBox[{
            SuperscriptBox["m2", "2"], " ", "m3", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", "m2"}], "+", "m3"}], ")"}]}], "+", 
           RowBox[{"m1", " ", "m2", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["m2", "2"], "-", 
              RowBox[{"4", " ", "m2", " ", "m3"}], "-", 
              RowBox[{"3", " ", 
               SuperscriptBox["m3", "2"]}]}], ")"}]}]}], ")"}], " ", "qsq"}], 
        
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "4"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox["m3", "2"], 
        RowBox[{"32", " ", 
         RowBox[{"(", 
          RowBox[{"m1", "-", "m3"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["m3", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "3"}], " ", "m1", " ", "m2"}], "+", 
           RowBox[{"m3", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", "m2"}], "+", "m3"}], ")"}]}]}], ")"}], " ", 
         "qsq"}], 
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "4"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m3", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.5754822595354967`*^9, 3.575482717204527*^9, {3.575482807648204*^9, 
   3.5754828291296463`*^9}, {3.575482926918315*^9, 3.5754829639233828`*^9}, 
   3.5754846116965017`*^9, 3.5754847767699614`*^9, {3.5754853725296*^9, 
   3.5754853912184153`*^9}, 3.575491192157715*^9, 3.575503798482194*^9, 
   3.57550396472117*^9, 3.575741187585586*^9, 3.5757428961350927`*^9, 
   3.5758185404090137`*^9, 3.5759271483539124`*^9, 3.5759282773912086`*^9, 
   3.5759305700029917`*^9, 3.575931315097371*^9, 3.5759328071753893`*^9, 
   3.5759330932282963`*^9, 3.575933679567024*^9, 3.575934959895438*^9, 
   3.575935244549008*^9, 3.5759358799834557`*^9, 3.575938362003059*^9, 
   3.575938625997038*^9, 3.575939417973022*^9, 3.5759402741447687`*^9, 
   3.575940609992372*^9, 3.575943011361413*^9, 3.57594452150957*^9, 
   3.575945941472361*^9, 3.576007709742931*^9, 3.576008623967984*^9, 
   3.576015068808301*^9, 3.5760973146753902`*^9, 3.57609997651552*^9, 
   3.576192440466675*^9, 3.5761950031835613`*^9, 3.5761987811281347`*^9, 
   3.5762459966112213`*^9, {3.576251924422138*^9, 3.5762519422455482`*^9}, 
   3.576253352033868*^9, 3.5763590192992487`*^9, 3.576363204355481*^9, 
   3.576364100571808*^9, 3.57636422648999*^9, 3.576364307623331*^9, 
   3.576364445816478*^9, 3.576372547678439*^9, 3.576429135697431*^9, 
   3.576536711625822*^9, 3.576539478730389*^9, 3.576539814448132*^9, 
   3.5766904343489923`*^9, 3.576692900841967*^9, 3.57671213935498*^9, 
   3.576795745080434*^9, 3.57698374008147*^9, 3.576983820169694*^9, 
   3.577215461037088*^9, 3.5772786743476343`*^9, 3.5772832674156647`*^9, 
   3.5772966612778463`*^9, 3.5773831267020407`*^9, 3.577401296953652*^9, 
   3.5776629561269207`*^9, 3.577663019512335*^9, 3.5776702375799913`*^9, 
   3.577675371432786*^9, 3.577676977917088*^9, 3.577816844099495*^9, 
   3.5778345045779037`*^9, 3.5779158768503304`*^9, 3.5779283625629*^9, 
   3.577930821920948*^9, 3.577931061805299*^9, 3.577934521153883*^9, 
   3.578330301057623*^9, 3.578342616668209*^9, 3.5783445284833193`*^9, 
   3.5784942064398603`*^9, 3.578756825766122*^9, 3.578849962604244*^9, 
   3.578850476534955*^9, 3.5788674895758657`*^9, 3.578953613505374*^9, 
   3.579005045872016*^9, 3.5790059026683292`*^9, 3.5790060643795013`*^9, 
   3.579006455663219*^9, 3.579007235278796*^9, 3.5790239382197237`*^9, 
   3.5791951519631643`*^9, 3.5792169141460133`*^9, 3.579282520795487*^9, 
   3.579283972563745*^9, 3.5792994063719378`*^9, {3.579303895818735*^9, 
   3.579303908744062*^9}, 3.579452394135118*^9, 3.579456357618094*^9, 
   3.579820889567319*^9, 3.579824017239972*^9, 3.580008818816884*^9, 
   3.580439173732257*^9, 3.580596062790663*^9, 3.580642974106855*^9, 
   3.580671595960641*^9, 3.5806991996503887`*^9, 3.5808459427931967`*^9, 
   3.581172085950387*^9, 3.581190368255628*^9, 3.5811956226175737`*^9, 
   3.5812689798636627`*^9, 3.581270934018257*^9, 3.5812763107211437`*^9, 
   3.5812918153480177`*^9, 3.581295518972372*^9, {3.581297627305236*^9, 
   3.581297655450973*^9}, 3.581298254243938*^9, 3.58129885393552*^9, 
   3.581346640517611*^9, 3.5813635282755117`*^9, 3.581369194427376*^9, 
   3.581369584583631*^9, 3.581442010864999*^9, 3.5814508100382347`*^9, 
   3.58175792566466*^9, 3.5819154025892973`*^9, 3.582021749337229*^9, 
   3.58248346948032*^9, 3.5840655075074673`*^9, 3.5841098368240957`*^9, 
   3.584110067617313*^9, 3.584208691350688*^9, 3.584208900072899*^9, 
   3.584211099428808*^9, 3.584211276263479*^9, 3.584213618522097*^9, 
   3.584214165575383*^9, 3.584221424928316*^9, 3.584224098813429*^9, 
   3.584279773275197*^9, 3.584317085048935*^9, 3.584362246833229*^9, 
   3.584383363826952*^9, 3.584448816984511*^9, 3.5844533133343973`*^9, 
   3.584453395715345*^9, 3.58445360049219*^9, 3.584453839903584*^9, 
   3.584479577138777*^9, 3.58448518679587*^9, 3.584708501027782*^9, 
   3.584720386653994*^9, 3.58472552158075*^9, 3.584874706321282*^9, 
   3.584874760302898*^9, 3.584876012730761*^9, 3.5848795567464657`*^9, 
   3.591549625403561*^9, 3.603755719931258*^9, 3.6037574602951*^9, 
   3.603849884535861*^9, 3.603850210010413*^9, 3.603853958686137*^9, 
   3.603925585986068*^9, 3.604157605227705*^9, 3.604236401511125*^9, 
   3.6050504551693277`*^9, 3.605452239190099*^9, 3.6054530162399063`*^9, 
   3.612794832599655*^9, 3.612794959160706*^9, 3.6127986826620197`*^9, 
   3.612798799803097*^9, 3.6128004688369017`*^9, 3.613320301080969*^9, 
   3.613578443430953*^9, 3.661542399014533*^9, 3.6616119715673227`*^9, 
   3.661615884446692*^9, 3.66163801939565*^9, 3.661704813603603*^9, 
   3.661810193874118*^9, 3.674840111716119*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["m3", 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m3"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"m2", " ", 
         SuperscriptBox["m3", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"5", " ", "m2"}], "+", "m3"}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["m1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            SuperscriptBox["m2", "2"]}], "+", 
           RowBox[{"5", " ", "m2", " ", "m3"}], "-", 
           SuperscriptBox["m3", "2"]}], ")"}]}], "-", 
        RowBox[{"m1", " ", "m3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"9", " ", 
            SuperscriptBox["m2", "2"]}], "+", 
           RowBox[{"2", " ", "m2", " ", "m3"}], "+", 
           SuperscriptBox["m3", "2"]}], ")"}]}]}], ")"}], " ", "qsq"}], 
     RowBox[{"96", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "3"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["m1", "2"], 
         RowBox[{"32", " ", 
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
          SuperscriptBox["\[Pi]", "2"]}]]}], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["m1", "3"], " ", "qsq"}], 
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "3"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox["m2", "2"], 
        RowBox[{"32", " ", 
         RowBox[{"(", 
          RowBox[{"m1", "-", "m2"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["m2", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["m2", "2"], "-", 
           RowBox[{"3", " ", "m1", " ", "m3"}], "+", 
           RowBox[{"2", " ", "m2", " ", "m3"}]}], ")"}], " ", "qsq"}], 
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "4"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"m3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m2", " ", "m3"}], "+", 
           RowBox[{"m1", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", "m2"}], "+", "m3"}], ")"}]}]}], 
          ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]], "+", 
       FractionBox[
        RowBox[{"m3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", 
            SuperscriptBox["m1", "2"], " ", 
            SuperscriptBox["m2", "2"]}], "+", 
           RowBox[{"m2", " ", 
            SuperscriptBox["m3", "2"], " ", 
            RowBox[{"(", 
             RowBox[{"m2", "+", 
              RowBox[{"2", " ", "m3"}]}], ")"}]}], "+", 
           RowBox[{"m1", " ", "m3", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "3"}], " ", 
               SuperscriptBox["m2", "2"]}], "-", 
              RowBox[{"4", " ", "m2", " ", "m3"}], "+", 
              SuperscriptBox["m3", "2"]}], ")"}]}]}], ")"}], " ", "qsq"}], 
        RowBox[{"48", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m1", "-", "m3"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m2", "-", "m3"}], ")"}], "4"], " ", 
         SuperscriptBox["\[Pi]", "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "m3", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.5754822595354967`*^9, 3.575482717204527*^9, {3.575482807648204*^9, 
   3.5754828291296463`*^9}, {3.575482926918315*^9, 3.5754829639233828`*^9}, 
   3.5754846116965017`*^9, 3.5754847767699614`*^9, {3.5754853725296*^9, 
   3.5754853912184153`*^9}, 3.575491192157715*^9, 3.575503798482194*^9, 
   3.57550396472117*^9, 3.575741187585586*^9, 3.5757428961350927`*^9, 
   3.5758185404090137`*^9, 3.5759271483539124`*^9, 3.5759282773912086`*^9, 
   3.5759305700029917`*^9, 3.575931315097371*^9, 3.5759328071753893`*^9, 
   3.5759330932282963`*^9, 3.575933679567024*^9, 3.575934959895438*^9, 
   3.575935244549008*^9, 3.5759358799834557`*^9, 3.575938362003059*^9, 
   3.575938625997038*^9, 3.575939417973022*^9, 3.5759402741447687`*^9, 
   3.575940609992372*^9, 3.575943011361413*^9, 3.57594452150957*^9, 
   3.575945941472361*^9, 3.576007709742931*^9, 3.576008623967984*^9, 
   3.576015068808301*^9, 3.5760973146753902`*^9, 3.57609997651552*^9, 
   3.576192440466675*^9, 3.5761950031835613`*^9, 3.5761987811281347`*^9, 
   3.5762459966112213`*^9, {3.576251924422138*^9, 3.5762519422455482`*^9}, 
   3.576253352033868*^9, 3.5763590192992487`*^9, 3.576363204355481*^9, 
   3.576364100571808*^9, 3.57636422648999*^9, 3.576364307623331*^9, 
   3.576364445816478*^9, 3.576372547678439*^9, 3.576429135697431*^9, 
   3.576536711625822*^9, 3.576539478730389*^9, 3.576539814448132*^9, 
   3.5766904343489923`*^9, 3.576692900841967*^9, 3.57671213935498*^9, 
   3.576795745080434*^9, 3.57698374008147*^9, 3.576983820169694*^9, 
   3.577215461037088*^9, 3.5772786743476343`*^9, 3.5772832674156647`*^9, 
   3.5772966612778463`*^9, 3.5773831267020407`*^9, 3.577401296953652*^9, 
   3.5776629561269207`*^9, 3.577663019512335*^9, 3.5776702375799913`*^9, 
   3.577675371432786*^9, 3.577676977917088*^9, 3.577816844099495*^9, 
   3.5778345045779037`*^9, 3.5779158768503304`*^9, 3.5779283625629*^9, 
   3.577930821920948*^9, 3.577931061805299*^9, 3.577934521153883*^9, 
   3.578330301057623*^9, 3.578342616668209*^9, 3.5783445284833193`*^9, 
   3.5784942064398603`*^9, 3.578756825766122*^9, 3.578849962604244*^9, 
   3.578850476534955*^9, 3.5788674895758657`*^9, 3.578953613505374*^9, 
   3.579005045872016*^9, 3.5790059026683292`*^9, 3.5790060643795013`*^9, 
   3.579006455663219*^9, 3.579007235278796*^9, 3.5790239382197237`*^9, 
   3.5791951519631643`*^9, 3.5792169141460133`*^9, 3.579282520795487*^9, 
   3.579283972563745*^9, 3.5792994063719378`*^9, {3.579303895818735*^9, 
   3.579303908744062*^9}, 3.579452394135118*^9, 3.579456357618094*^9, 
   3.579820889567319*^9, 3.579824017239972*^9, 3.580008818816884*^9, 
   3.580439173732257*^9, 3.580596062790663*^9, 3.580642974106855*^9, 
   3.580671595960641*^9, 3.5806991996503887`*^9, 3.5808459427931967`*^9, 
   3.581172085950387*^9, 3.581190368255628*^9, 3.5811956226175737`*^9, 
   3.5812689798636627`*^9, 3.581270934018257*^9, 3.5812763107211437`*^9, 
   3.5812918153480177`*^9, 3.581295518972372*^9, {3.581297627305236*^9, 
   3.581297655450973*^9}, 3.581298254243938*^9, 3.58129885393552*^9, 
   3.581346640517611*^9, 3.5813635282755117`*^9, 3.581369194427376*^9, 
   3.581369584583631*^9, 3.581442010864999*^9, 3.5814508100382347`*^9, 
   3.58175792566466*^9, 3.5819154025892973`*^9, 3.582021749337229*^9, 
   3.58248346948032*^9, 3.5840655075074673`*^9, 3.5841098368240957`*^9, 
   3.584110067617313*^9, 3.584208691350688*^9, 3.584208900072899*^9, 
   3.584211099428808*^9, 3.584211276263479*^9, 3.584213618522097*^9, 
   3.584214165575383*^9, 3.584221424928316*^9, 3.584224098813429*^9, 
   3.584279773275197*^9, 3.584317085048935*^9, 3.584362246833229*^9, 
   3.584383363826952*^9, 3.584448816984511*^9, 3.5844533133343973`*^9, 
   3.584453395715345*^9, 3.58445360049219*^9, 3.584453839903584*^9, 
   3.584479577138777*^9, 3.58448518679587*^9, 3.584708501027782*^9, 
   3.584720386653994*^9, 3.58472552158075*^9, 3.584874706321282*^9, 
   3.584874760302898*^9, 3.584876012730761*^9, 3.5848795567464657`*^9, 
   3.591549625403561*^9, 3.603755719931258*^9, 3.6037574602951*^9, 
   3.603849884535861*^9, 3.603850210010413*^9, 3.603853958686137*^9, 
   3.603925585986068*^9, 3.604157605227705*^9, 3.604236401511125*^9, 
   3.6050504551693277`*^9, 3.605452239190099*^9, 3.6054530162399063`*^9, 
   3.612794832599655*^9, 3.612794959160706*^9, 3.6127986826620197`*^9, 
   3.612798799803097*^9, 3.6128004688369017`*^9, 3.613320301080969*^9, 
   3.613578443430953*^9, 3.661542399014533*^9, 3.6616119715673227`*^9, 
   3.661615884446692*^9, 3.66163801939565*^9, 3.661704813603603*^9, 
   3.661810193874118*^9, 3.674840115298094*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Cij", " ", "with", " ", 
     SuperscriptBox["p1", "2"]}], "=", 
    RowBox[{
     SuperscriptBox["p2", "2"], "=", 
     RowBox[{
      RowBox[{
      "0", " ", "and", " ", "expanded", " ", "to", " ", "first", " ", "order",
        " ", "around", " ", "p1dotp2"}], "=", "0"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Use", " ", 
     RowBox[{"identity", ":", " ", 
      RowBox[{"\[Integral]", " ", 
       RowBox[{
        SuperscriptBox["d", "4"], "k", " ", 
        SuperscriptBox["k", "\[Mu]"], 
        SuperscriptBox["k", "\[Nu]"], 
        SuperscriptBox["k", "\[Rho]"], 
        SuperscriptBox["k", "\[Sigma]"], 
        SuperscriptBox["k", "\[Alpha]"], 
        SuperscriptBox["k", "\[Beta]"], 
        RowBox[{"f", "[", 
         SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "192"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Eta]", "\[Mu]\[Nu]"], 
        SuperscriptBox["\[Eta]", "\[Rho]\[Sigma]"], 
        SuperscriptBox["\[Eta]", "\[Alpha]\[Beta]"]}], "+", "perms"}], ")"}], 
     
     RowBox[{"\[Integral]", " ", 
      RowBox[{
       SuperscriptBox["d", "4"], "k", " ", 
       SuperscriptBox["k", "6"], 
       RowBox[{"f", "[", 
        SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"integrandCijtemp", "=", 
   RowBox[{"Collect", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Expand", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"Normal", "[", 
                    RowBox[{"Series", "[", 
                    RowBox[{
                    RowBox[{"k\[Mu]", " ", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"k\[Nu]", "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp1"}], "+", "m2"}], 
                    ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp2"}], "+", "m3"}], 
                    ")"}]}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], 
                    "]"}], "]"}], "/.", 
                   RowBox[{
                    SuperscriptBox["\[Epsilon]", "k_"], "\[RuleDelayed]", 
                    RowBox[{"1", "/;", 
                    RowBox[{"EvenQ", "[", "k", "]"}]}]}]}], "/.", 
                  RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "/.", 
                 RowBox[{
                  SuperscriptBox["kdotp2", "4"], "\[Rule]", "0"}]}], "/.", 
                RowBox[{
                 SuperscriptBox["kdotp1", "4"], "\[Rule]", "0"}]}], "]"}], "/.", 
              RowBox[{
               RowBox[{"kdotp1", " ", 
                SuperscriptBox["kdotp2", "3"], " ", "k\[Mu]", " ", "k\[Nu]"}],
                "\[Rule]", 
               RowBox[{
                RowBox[{"1", "/", 
                 RowBox[{"(", 
                  RowBox[{"d", 
                   RowBox[{"(", 
                    RowBox[{"d", "+", "2"}], ")"}], 
                   RowBox[{"(", 
                    RowBox[{"d", "+", "4"}], ")"}]}], ")"}]}], " ", 
                RowBox[{"(", 
                 RowBox[{
                 "6", "p1dotp2", " ", "p2\[Mu]", " ", "p2\[Nu]", " ", 
                  SuperscriptBox["k", "6"]}], ")"}]}]}]}], "/.", 
             RowBox[{
              RowBox[{
               SuperscriptBox["kdotp1", "3"], " ", "kdotp2", " ", "k\[Mu]", 
               " ", "k\[Nu]"}], "\[Rule]", 
              RowBox[{
               RowBox[{"1", "/", 
                RowBox[{"(", 
                 RowBox[{"d", 
                  RowBox[{"(", 
                   RowBox[{"d", "+", "2"}], ")"}], 
                  RowBox[{"(", 
                   RowBox[{"d", "+", "4"}], ")"}]}], ")"}]}], " ", 
               RowBox[{"(", 
                RowBox[{"6", "p1dotp2", " ", "p1\[Mu]", " ", "p1\[Nu]", " ", 
                 SuperscriptBox["k", "6"]}], ")"}]}]}]}], "/.", 
            RowBox[{
             RowBox[{
              SuperscriptBox["kdotp1", "2"], " ", 
              SuperscriptBox["kdotp2", "2"], "k\[Mu]", " ", "k\[Nu]"}], 
             "\[Rule]", 
             RowBox[{
              RowBox[{"1", "/", 
               RowBox[{"(", 
                RowBox[{"d", 
                 RowBox[{"(", 
                  RowBox[{"d", "+", "2"}], ")"}], 
                 RowBox[{"(", 
                  RowBox[{"d", "+", "4"}], ")"}]}], ")"}]}], 
              RowBox[{"(", 
               RowBox[{"4", "p1dotp2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"p1\[Mu]", " ", "p2\[Nu]"}], "+", 
                  RowBox[{"p2\[Mu]", " ", "p1\[Nu]"}]}], ")"}]}], ")"}]}]}]}],
            "/.", 
           RowBox[{
            RowBox[{
             SuperscriptBox["kdotp2", "2"], " ", "k\[Mu]", " ", "k\[Nu]"}], 
            "\[Rule]", 
            RowBox[{
             RowBox[{"1", "/", 
              RowBox[{"(", 
               RowBox[{"d", 
                RowBox[{"(", 
                 RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
             RowBox[{"(", 
              RowBox[{"2", "p2\[Mu]", " ", "p2\[Nu]", " ", 
               SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
          RowBox[{
           RowBox[{
            SuperscriptBox["kdotp1", "2"], " ", "k\[Mu]", " ", "k\[Nu]"}], 
           "\[Rule]", 
           RowBox[{
            RowBox[{"1", "/", 
             RowBox[{"(", 
              RowBox[{"d", 
               RowBox[{"(", 
                RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
            RowBox[{"(", 
             RowBox[{"2", "p1\[Mu]", " ", "p1\[Nu]", " ", 
              SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
         RowBox[{
          RowBox[{"kdotp1", " ", "kdotp2", " ", "k\[Mu]", " ", "k\[Nu]"}], 
          "\[Rule]", 
          RowBox[{
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"d", 
              RowBox[{"(", 
               RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"p1dotp2", " ", "\[Eta]\[Mu]\[Nu]"}], "+", 
             RowBox[{"p1\[Mu]", " ", "p2\[Nu]"}], "+", 
             RowBox[{"p2\[Mu]", " ", "p1\[Nu]"}]}], ")"}], 
           SuperscriptBox["k", "4"]}]}]}], "/.", 
        RowBox[{
         RowBox[{"k\[Mu]", " ", "k\[Nu]"}], "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", "d"}], " ", "\[Eta]\[Mu]\[Nu]", " ", 
          SuperscriptBox["k", "2"]}]}]}], "/.", 
       RowBox[{"d", "\[Rule]", 
        RowBox[{"4", "-", "\[Delta]"}]}]}], "/.", 
      RowBox[{"p1dotp2", "\[Rule]", 
       RowBox[{
        RowBox[{"-", "qsq"}], "/", "2"}]}]}], ",", "qsq", ",", 
     "FullSimplify"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.574900226988413*^9, 3.574900264108863*^9}, {
  3.574900328087408*^9, 3.574900396959166*^9}, {3.574901224333816*^9, 
  3.574901243102832*^9}, {3.574901702649322*^9, 3.5749017754695377`*^9}, {
  3.574901849582118*^9, 3.5749019113833723`*^9}, {3.5749026103462667`*^9, 
  3.574902628406314*^9}, {3.574903114210644*^9, 3.5749032650500603`*^9}, {
  3.574903473994944*^9, 3.574903484149876*^9}, {3.5776702440754642`*^9, 
  3.577670273015163*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", " ", "qsq", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Nu]", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", 
           SuperscriptBox["k", "6"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Mu]"}], 
          "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", "p2\[Mu]"}]}],
          ")"}]}], "+", 
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Mu]"}], 
          "+", 
          RowBox[{"3", " ", 
           SuperscriptBox["k", "6"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", "p2\[Mu]"}]}],
          ")"}], " ", "p2\[Nu]"}], "-", 
       RowBox[{
        SuperscriptBox["k", "4"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "8"}], "+", "\[Delta]"}], ")"}], " ", 
        "\[Eta]\[Mu]\[Nu]"}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "4"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "8"}], "+", "\[Delta]"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}], ")"}]}], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"4", " ", 
     SuperscriptBox["k", "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Nu]", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Mu]"}], 
          "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", "p2\[Mu]"}]}],
          ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m3"}], ")"}], " ", "p1\[Mu]"}], 
          "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["k", "2"], "+", "m2"}], ")"}], " ", "p2\[Mu]"}]}],
          ")"}], " ", "p2\[Nu]"}]}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["k", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}], " ", 
     "\[Eta]\[Mu]\[Nu]"}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m1"}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "3"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "6"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "4"}], "+", "\[Delta]"}], ")"}]}]]}]], "Output",
 CellChangeTimes->{{3.5749003482155657`*^9, 3.574900397282271*^9}, {
   3.574901224849764*^9, 3.574901243739356*^9}, 3.5749017572444277`*^9, 
   3.574901892182584*^9, 3.5749031047355757`*^9, {3.5749031588367558`*^9, 
   3.574903169441737*^9}, {3.574903212988481*^9, 3.574903265493743*^9}, {
   3.5749034744616623`*^9, 3.574903485226489*^9}, 3.5749039615577087`*^9, 
   3.575024456561544*^9, 3.575037209136154*^9, 3.575039377105245*^9, 
   3.575043135943554*^9, 3.575071512099782*^9, 3.575121741757475*^9, 
   3.5751998212685747`*^9, 3.5754108225789013`*^9, 3.575414054634507*^9, 
   3.575414128647138*^9, 3.575431670068425*^9, 3.5754773996950607`*^9, 
   3.575478098534864*^9, 3.57548294393124*^9, 3.575484780813212*^9, {
   3.5754853737552567`*^9, 3.575485391985918*^9}, 3.5754911936709414`*^9, 
   3.57550379969281*^9, 3.5755039658637466`*^9, 3.5757411887625723`*^9, 
   3.575742897496097*^9, 3.575818541541054*^9, 3.5759271495634117`*^9, 
   3.575928278556176*^9, 3.5759305711289454`*^9, 3.575931316295546*^9, 
   3.575932808318329*^9, 3.5759330943719387`*^9, 3.575933680707403*^9, 
   3.575934961166813*^9, 3.5759352456907883`*^9, 3.575935881102968*^9, 
   3.575938363185979*^9, 3.57593862714336*^9, 3.5759394191002817`*^9, 
   3.575940275314741*^9, 3.57594061112265*^9, 3.575943012507538*^9, 
   3.575944522638999*^9, 3.575945942590623*^9, 3.576007710919854*^9, 
   3.5760086251129637`*^9, 3.576015069983117*^9, 3.576097315799981*^9, 
   3.576099977665805*^9, 3.576192441595172*^9, 3.5761950042973127`*^9, 
   3.576198782232785*^9, 3.5762459977491426`*^9, 3.576251943399746*^9, 
   3.576253353245831*^9, 3.576359020433177*^9, 3.576363205562458*^9, 
   3.57636410178935*^9, 3.576364227724944*^9, 3.576364308860017*^9, 
   3.576364446972472*^9, 3.57637254881353*^9, 3.576429136814*^9, 
   3.576536712799259*^9, 3.576539479854055*^9, 3.576539815640511*^9, 
   3.5766904354354067`*^9, 3.5766929019772463`*^9, 3.576712140493822*^9, 
   3.57679574650597*^9, 3.576983741330303*^9, 3.576983821364303*^9, 
   3.577215462248797*^9, 3.577278675518199*^9, 3.5772832686071587`*^9, 
   3.5772966624807568`*^9, 3.577383127924925*^9, 3.577401298147332*^9, 
   3.577662957543198*^9, 3.577663020492921*^9, 3.577670274641251*^9, 
   3.577675373136271*^9, 3.577676979702888*^9, 3.577816846100312*^9, 
   3.5778345063464518`*^9, 3.577915878491005*^9, 3.5779283641962748`*^9, 
   3.577930912878531*^9, 3.577931063535254*^9, 3.577934522851131*^9, 
   3.578330302846408*^9, 3.5783426183159227`*^9, 3.578344530044507*^9, 
   3.578494208063204*^9, 3.578756827600235*^9, 3.578849964227969*^9, 
   3.5788504783024483`*^9, 3.5788674911408978`*^9, 3.578953615727113*^9, 
   3.57900504755075*^9, 3.5790059043699903`*^9, 3.579006066069569*^9, 
   3.579006456900304*^9, 3.579007235737961*^9, 3.579023939816044*^9, 
   3.5791951535440187`*^9, 3.5792169157602053`*^9, 3.579282522406*^9, 
   3.579283974116014*^9, 3.579299470395026*^9, 3.579303911111755*^9, 
   3.579452397353187*^9, 3.5794563610592823`*^9, 3.57982089283971*^9, 
   3.579824020863618*^9, 3.580008824525156*^9, 3.580439176724121*^9, 
   3.580596066566815*^9, 3.5806429771996317`*^9, 3.580671599098812*^9, 
   3.5806992030059547`*^9, 3.58084594637869*^9, 3.581172089734984*^9, 
   3.581190371910972*^9, 3.581195626095956*^9, 3.5812689831645393`*^9, 
   3.58127093733146*^9, 3.5812763139178743`*^9, 3.5812918185811872`*^9, 
   3.581295522052093*^9, 3.5812976585469513`*^9, 3.5812982574475927`*^9, 
   3.5812988570268373`*^9, 3.581346644773817*^9, 3.581363536394248*^9, 
   3.581369198925891*^9, 3.5813695894487953`*^9, 3.5814420144810534`*^9, 
   3.581450814461768*^9, 3.581757931633772*^9, 3.5819154083993607`*^9, 
   3.582021754668507*^9, 3.582483475137041*^9, 3.584065516833695*^9, 
   3.584109846412675*^9, 3.584110077018704*^9, 3.58420870069386*^9, 
   3.584208909268894*^9, 3.584211108609633*^9, 3.5842112854720497`*^9, 
   3.584213627830373*^9, 3.5842141747538633`*^9, 3.5842214341906147`*^9, 
   3.58422410861558*^9, 3.584279782756023*^9, 3.58431709450109*^9, 
   3.584362256837635*^9, 3.58438337316427*^9, 3.584448826916868*^9, 
   3.5844533235998087`*^9, 3.584453405830868*^9, 3.584453609839066*^9, 
   3.5844538493873053`*^9, 3.5844795866703053`*^9, 3.584485196087447*^9, 
   3.584708510379219*^9, 3.5847203961501427`*^9, 3.5847255309862623`*^9, 
   3.584874769515232*^9, 3.5848760219644012`*^9, 3.58487956604624*^9, 
   3.591549635471657*^9, 3.603755726414506*^9, 3.603757468095469*^9, 
   3.603849890612577*^9, 3.603850216852721*^9, 3.603853965167159*^9, 
   3.6039255915325947`*^9, 3.604157610672423*^9, 3.6042364080129137`*^9, 
   3.605050464947919*^9, 3.60545224902628*^9, 3.605453025993805*^9, 
   3.6127948366700287`*^9, 3.612794963967465*^9, 3.612798702897789*^9, 
   3.612798801750823*^9, 3.6128004708488827`*^9, 3.6133203037659283`*^9, 
   3.613578445393116*^9, 3.661542404500905*^9, 3.661611977352729*^9, 
   3.66161589015784*^9, 3.661638025309791*^9, 3.661704819165927*^9, 
   3.661810199344685*^9, 3.674840117403556*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cijtemp", "=", 
   RowBox[{"Intfuncgen", "[", "integrandCijtemp", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5754829785813828`*^9, 3.575482980173191*^9}, {
  3.5754847863059807`*^9, 3.5754847893206453`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C00temp", "=", 
   RowBox[{
    RowBox[{"c00", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Cijtemp", ",", "\[Eta]\[Mu]\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11temp", "=", 
   RowBox[{
    RowBox[{"c11", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Cijtemp", ",", "p1\[Mu]", ",", "p1\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12temp", "=", 
   RowBox[{
    RowBox[{"c12", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Cijtemp", ",", "p1\[Mu]", ",", "p2\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22temp", "=", 
   RowBox[{
    RowBox[{"c22", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Cijtemp", ",", "p2\[Mu]", ",", "p2\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.575482955815819*^9, 3.575482975374505*^9}, {
  3.575484938445628*^9, 3.575484976569477*^9}, {3.575498108963306*^9, 
  3.575498120837516*^9}, {3.57729663952792*^9, 3.577296651078128*^9}, {
  3.577675322044446*^9, 3.5776753315322533`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"B0temp0", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"B0temp", "/.", 
      RowBox[{"psq", "\[Rule]", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Log", "[", "m1", "]"}], ",", 
      RowBox[{"Log", "[", "m2", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"B0temp1", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    SubscriptBox["\[PartialD]", "psq"], "B0temp"}], "]"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"m1", " ", 
     RowBox[{"Log", "[", "m1", "]"}]}], 
    RowBox[{"m1", "-", "m2"}]]}], "+", 
  FractionBox[
   RowBox[{"m2", " ", 
    RowBox[{"Log", "[", "m2", "]"}]}], 
   RowBox[{"m1", "-", "m2"}]], "+", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], "2"]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m1", "2"]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"4", " ", "EulerGamma", " ", "m1", " ", "m2"}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"2", " ", "EulerGamma", " ", 
      SuperscriptBox["m2", "2"]}], "-", 
     RowBox[{"m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "256", "]"}]}], "-", 
     RowBox[{"4", " ", "m1", " ", "m2", " ", 
      RowBox[{"Log", "[", "\[Pi]", "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["m2", "2"], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"4", " ", "\[Pi]"}], "]"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.661611995360095*^9, 3.6616159081607656`*^9, 
  3.66163804409754*^9, 3.661704837658531*^9, 3.661810216623901*^9, 
  3.674840135342556*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["m1", "2"], "-", 
   SuperscriptBox["m2", "2"], "-", 
   RowBox[{"2", " ", "m1", " ", "m2", " ", 
    RowBox[{"Log", "[", "m1", "]"}]}], "+", 
   RowBox[{"2", " ", "m1", " ", "m2", " ", 
    RowBox[{"Log", "[", "m2", "]"}]}]}], 
  RowBox[{"2", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"m1", "-", "m2"}], ")"}], "3"]}]]], "Output",
 CellChangeTimes->{3.661611995360095*^9, 3.6616159081607656`*^9, 
  3.66163804409754*^9, 3.661704837658531*^9, 3.661810216623901*^9, 
  3.6748401353435583`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
  3.6615427066237087`*^9, 3.6615427367168493`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "y", " ", "z"}], "-", 
        RowBox[{"x", " ", 
         RowBox[{"(", 
          RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["x", "2"]}], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox["x", 
        RowBox[{"16", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["y", 
         RowBox[{"16", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", "y", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "-", 
           RowBox[{"2", " ", "x", " ", "z"}], "+", 
           RowBox[{"y", " ", "z"}]}], ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["z", 
         RowBox[{"16", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "2"}], " ", "x", " ", "y"}], "+", 
           RowBox[{"z", " ", 
            RowBox[{"(", 
             RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661611995428934*^9, 3.661615908207347*^9, 3.6616380441598473`*^9, 
   3.66170483770654*^9, 3.6618102166563663`*^9, 3.674840135378092*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C0funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "y", " ", "z"}], "-", 
         RowBox[{"x", " ", 
          RowBox[{"(", 
           RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], 
      RowBox[{"32", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["x", "2"]}], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox["x", 
         RowBox[{"16", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["y", 
          RowBox[{"16", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", "y", " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", "2"], "-", 
            RowBox[{"2", " ", "x", " ", "z"}], "+", 
            RowBox[{"y", " ", "z"}]}], ")"}]}], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["z", 
          RowBox[{"16", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "2"}], " ", "x", " ", "y"}], "+", 
            RowBox[{"z", " ", 
             RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
  3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6748378979070053`*^9, 
  3.674837898369111*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C0funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C0funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C0funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C0funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C0funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661542768637866*^9, 3.661542973070032*^9}, {
  3.6615430048044777`*^9, 3.661543009268293*^9}, {3.661543066886743*^9, 
  3.661543338316929*^9}, {3.6615434173357563`*^9, 3.66154342191934*^9}, {
  3.661543491159232*^9, 3.661543664209738*^9}, {3.661543705907539*^9, 
  3.661543747667486*^9}, {3.661543965677484*^9, 3.661544123667473*^9}, {
  3.66154565476906*^9, 3.661545655347352*^9}, {3.66170482090112*^9, 
  3.661704839497237*^9}, {3.674837900267687*^9, 3.674837927407371*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C0funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C0funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C0funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661704848828199*^9, 3.661704854712596*^9}, {
   3.6617048904380693`*^9, 3.6617049509300823`*^9}, {3.661706568919217*^9, 
   3.661706569829359*^9}, {3.661810084925221*^9, 3.6618100864420652`*^9}, 
   3.6748380280202007`*^9, {3.674838082696406*^9, 3.674838264280786*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{
  3.6615441635317793`*^9, {3.661544195654359*^9, 3.6615442810376577`*^9}, {
   3.661544314265877*^9, 3.661544556829431*^9}, {3.661544590695019*^9, 
   3.66154461345207*^9}, {3.6615446441380873`*^9, 3.6615449339022818`*^9}, 
   3.6615456520764923`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C1temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
   3.6615427066237087`*^9, 3.6615427367168493`*^9}, 3.661545675141267*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["y", 
     RowBox[{"32", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"y", "-", "z"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["y", "2"]}], " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{"y", "+", 
           RowBox[{"5", " ", "z"}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["x", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "-", 
           RowBox[{"5", " ", "y", " ", "z"}], "-", 
           RowBox[{"2", " ", 
            SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
        RowBox[{"x", " ", "y", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "+", 
           RowBox[{"2", " ", "y", " ", "z"}], "+", 
           RowBox[{"9", " ", 
            SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["x", "3"]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], "-", 
       FractionBox[
        SuperscriptBox["x", "2"], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"y", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{"y", "-", 
              RowBox[{"2", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"y", " ", "z"}]}], ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox[
        RowBox[{"q", " ", "y", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", 
            SuperscriptBox["x", "2"], " ", 
            SuperscriptBox["z", "2"]}], "+", 
           RowBox[{
            SuperscriptBox["y", "2"], " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", "y"}], "+", "z"}], ")"}]}], "+", 
           RowBox[{"x", " ", "y", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["y", "2"], "-", 
              RowBox[{"4", " ", "y", " ", "z"}], "-", 
              RowBox[{"3", " ", 
               SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox["z", "2"], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["z", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "3"}], " ", "x", " ", "y"}], "+", 
           RowBox[{"z", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}]}], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.661612000190675*^9, 3.66161591310326*^9, 
   3.661638049465725*^9, 3.661704842724553*^9, 3.661810221243088*^9, 
   3.674840140668674*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C1funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["y", 
      RowBox[{"32", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox["y", "2"]}], " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{"y", "+", 
            RowBox[{"5", " ", "z"}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["x", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", "2"], "-", 
            RowBox[{"5", " ", "y", " ", "z"}], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
         RowBox[{"x", " ", "y", " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", "2"], "+", 
            RowBox[{"2", " ", "y", " ", "z"}], "+", 
            RowBox[{"9", " ", 
             SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"q", " ", 
           SuperscriptBox["x", "3"]}], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], "-", 
        FractionBox[
         SuperscriptBox["x", "2"], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"y", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{"y", "-", 
               RowBox[{"2", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"y", " ", "z"}]}], ")"}]}], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox[
         RowBox[{"q", " ", "y", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             SuperscriptBox["x", "2"], " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{
             SuperscriptBox["y", "2"], " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "y"}], "+", "z"}], ")"}]}], "+", 
            RowBox[{"x", " ", "y", " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["y", "2"], "-", 
               RowBox[{"4", " ", "y", " ", "z"}], "-", 
               RowBox[{"3", " ", 
                SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         SuperscriptBox["z", "2"], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["z", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", "x", " ", "y"}], "+", 
            RowBox[{"z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
  3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
  3.6615456906623697`*^9}, {3.674838325958639*^9, 3.6748383265017014`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C1funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C1funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C1funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C1funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C1funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
   3.6617050171452208`*^9, 3.661705031561706*^9}, {3.661705236735703*^9, 
   3.6617052776928062`*^9}, 3.67483833220221*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C1funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"x", "\[Equal]", "0"}], "||", 
          RowBox[{"z", "\[Equal]", "0"}]}], ")"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C1funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C1funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{
  3.661705280160005*^9, {3.6617053639055233`*^9, 3.661705387012764*^9}, 
   3.6618100904264727`*^9, {3.67483833940053*^9, 3.674838407297415*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C1funcz12", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C1funcz13", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C1funcz23", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661704968383883*^9, 3.661704971798695*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"DirectedInfinity", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "q"}], "+", 
    RowBox[{"3", " ", "z"}]}], "]"}], 
  SuperscriptBox[
   RowBox[{"Sign", "[", "z", "]"}], "2"]]], "Output",
 CellChangeTimes->{
  3.661704972039997*^9, 3.6617051217311993`*^9, {3.6617051598874063`*^9, 
   3.661705179191448*^9}, {3.661705217625696*^9, 3.66170524251197*^9}, 
   3.661810224508112*^9, 3.674840145339077*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"q", "-", 
    RowBox[{"3", " ", "y"}]}], 
   RowBox[{"6", " ", 
    SuperscriptBox["y", "2"]}]]}]], "Output",
 CellChangeTimes->{
  3.661704972039997*^9, 3.6617051217311993`*^9, {3.6617051598874063`*^9, 
   3.661705179191448*^9}, {3.661705217625696*^9, 3.66170524251197*^9}, 
   3.661810224508112*^9, 3.674840145340127*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"Sign", "[", "q", "]"}], 
    RowBox[{"Sign", "[", "x", "]"}]]}], "]"}]], "Output",
 CellChangeTimes->{
  3.661704972039997*^9, 3.6617051217311993`*^9, {3.6617051598874063`*^9, 
   3.661705179191448*^9}, {3.661705217625696*^9, 3.66170524251197*^9}, 
   3.661810224508112*^9, 3.674840145340909*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C2temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
   3.6615427066237087`*^9, 3.6615427367168493`*^9}, 3.661545675141267*^9, 
   3.6615459611157913`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["z", 
     RowBox[{"32", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "y"}], "+", "z"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         SuperscriptBox["z", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"5", " ", "y"}], "+", "z"}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["x", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", 
            SuperscriptBox["y", "2"]}], "+", 
           RowBox[{"5", " ", "y", " ", "z"}], "-", 
           SuperscriptBox["z", "2"]}], ")"}]}], "-", 
        RowBox[{"x", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"9", " ", 
            SuperscriptBox["y", "2"]}], "+", 
           RowBox[{"2", " ", "y", " ", "z"}], "+", 
           SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["x", "3"]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], "-", 
       FractionBox[
        SuperscriptBox["x", "2"], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox["y", "2"], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["y", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "-", 
           RowBox[{"3", " ", "x", " ", "z"}], "+", 
           RowBox[{"2", " ", "y", " ", "z"}]}], ")"}]}], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"y", " ", "z"}], "+", 
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", "y"}], "+", "z"}], ")"}]}]}], ")"}]}], 
        RowBox[{"32", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox[
        RowBox[{"q", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", 
            SuperscriptBox["x", "2"], " ", 
            SuperscriptBox["y", "2"]}], "+", 
           RowBox[{"y", " ", 
            SuperscriptBox["z", "2"], " ", 
            RowBox[{"(", 
             RowBox[{"y", "+", 
              RowBox[{"2", " ", "z"}]}], ")"}]}], "+", 
           RowBox[{"x", " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "3"}], " ", 
               SuperscriptBox["y", "2"]}], "-", 
              RowBox[{"4", " ", "y", " ", "z"}], "+", 
              SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.66154596139646*^9, 3.6616120035728292`*^9, 
   3.66161591655196*^9, 3.6616380532193403`*^9, 3.661704846248418*^9, 
   3.661810224547364*^9, 3.6748401453911457`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C2funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["z", 
      RowBox[{"32", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"y", " ", 
          SuperscriptBox["z", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"5", " ", "y"}], "+", "z"}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["x", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox["y", "2"]}], "+", 
            RowBox[{"5", " ", "y", " ", "z"}], "-", 
            SuperscriptBox["z", "2"]}], ")"}]}], "-", 
         RowBox[{"x", " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"9", " ", 
             SuperscriptBox["y", "2"]}], "+", 
            RowBox[{"2", " ", "y", " ", "z"}], "+", 
            SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"q", " ", 
           SuperscriptBox["x", "3"]}], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], "-", 
        FractionBox[
         SuperscriptBox["x", "2"], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         SuperscriptBox["y", "2"], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["y", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", "2"], "-", 
            RowBox[{"3", " ", "x", " ", "z"}], "+", 
            RowBox[{"2", " ", "y", " ", "z"}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"y", " ", "z"}], "+", 
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", "y"}], "+", "z"}], ")"}]}]}], 
           ")"}]}], 
         RowBox[{"32", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox[
         RowBox[{"q", " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             SuperscriptBox["x", "2"], " ", 
             SuperscriptBox["y", "2"]}], "+", 
            RowBox[{"y", " ", 
             SuperscriptBox["z", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"y", "+", 
               RowBox[{"2", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"x", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", 
                SuperscriptBox["y", "2"]}], "-", 
               RowBox[{"4", " ", "y", " ", "z"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "4"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
  3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
  3.6615456906623697`*^9}, {3.6615459684671373`*^9, 3.661545974336686*^9}, {
  3.674838416649414*^9, 3.6748384170963984`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C2funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C2funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C2funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C2funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C2funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
  3.661545976723319*^9, 3.6615460255038643`*^9}, {3.661705404162435*^9, 
  3.6617054067309837`*^9}, {3.6748384192506638`*^9, 3.6748384213124647`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C2funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"y", "\[Equal]", "0"}], "||", 
          RowBox[{"x", "\[Equal]", "0"}]}], ")"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"y", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz12", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C2funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C2funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66170545903759*^9, 3.661705526187743*^9}, 
   3.6618100947254744`*^9, {3.67483842695341*^9, 3.674838450382214*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C2funcz12", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C2funcz13", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C2funcz23", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661705425887252*^9, 3.661705440830681*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"q", "-", 
    RowBox[{"3", " ", "z"}]}], 
   RowBox[{"6", " ", 
    SuperscriptBox["z", "2"]}]]}]], "Output",
 CellChangeTimes->{{3.661705432634037*^9, 3.6617054410508633`*^9}, 
   3.661810227464518*^9, 3.674840149386179*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"DirectedInfinity", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "q"}], "+", 
    RowBox[{"3", " ", "y"}]}], "]"}], 
  SuperscriptBox[
   RowBox[{"Sign", "[", "y", "]"}], "2"]]], "Output",
 CellChangeTimes->{{3.661705432634037*^9, 3.6617054410508633`*^9}, 
   3.661810227464518*^9, 3.674840149387162*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  RowBox[{"-", 
   FractionBox[
    RowBox[{"Sign", "[", "q", "]"}], 
    RowBox[{"Sign", "[", "x", "]"}]]}], "]"}]], "Output",
 CellChangeTimes->{{3.661705432634037*^9, 3.6617054410508633`*^9}, 
   3.661810227464518*^9, 3.6748401493879642`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"C00temp", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"qsq", "\[Rule]", "q"}], ",", 
      RowBox[{"m1", "\[Rule]", "x"}], ",", 
      RowBox[{"m2", "\[Rule]", "y"}], ",", 
      RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}], ",", 
   RowBox[{"\[Delta]", "\[Rule]", "\[Infinity]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
   3.6615427066237087`*^9, 3.6615427367168493`*^9}, 3.661545675141267*^9, 
   3.6615459611157913`*^9, {3.661546127788756*^9, 3.661546148505188*^9}}],

Cell[BoxData[
 RowBox[{"16", " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["3", 
     RowBox[{"128", " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox["EulerGamma", 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", 
      SuperscriptBox["y", "2"]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", 
      SuperscriptBox["y", "2"], " ", "z"}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", 
      SuperscriptBox["z", "2"]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", "y", " ", 
      SuperscriptBox["z", "2"]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
    FractionBox[
     RowBox[{"Log", "[", "2", "]"}], 
     RowBox[{"32", " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{"Log", "[", "\[Pi]", "]"}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", 
      SuperscriptBox["x", "3"], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["x", "2"], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", 
      SuperscriptBox["y", "3"], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["y", "2"], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"y", "-", "z"}], ")"}]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", 
      SuperscriptBox["y", "2"], " ", "z", " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", 
      SuperscriptBox["y", "3"], " ", "z", " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", "y", " ", 
      SuperscriptBox["z", "2"], " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", "x", " ", 
      SuperscriptBox["z", "3"], " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", "y", " ", 
      SuperscriptBox["z", "3"], " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["z", "2"], " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.66154596139646*^9, {3.661546130459319*^9, 
   3.661546149404558*^9}, 3.6616120105036297`*^9, 3.661615919659317*^9, 
   3.661638056655446*^9, 3.661704849629943*^9, 3.6618102275448093`*^9, 
   3.674840149513826*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C00funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"16", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["3", 
      RowBox[{"128", " ", 
       SuperscriptBox["\[Pi]", "2"]}]], "-", 
     FractionBox["EulerGamma", 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", 
       SuperscriptBox["y", "2"]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{"q", " ", 
       SuperscriptBox["y", "2"], " ", "z"}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", 
       SuperscriptBox["z", "2"]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{"q", " ", "y", " ", 
       SuperscriptBox["z", "2"]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{"Log", "[", "2", "]"}], 
      RowBox[{"32", " ", 
       SuperscriptBox["\[Pi]", "2"]}]], "+", 
     FractionBox[
      RowBox[{"Log", "[", "\[Pi]", "]"}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"]}]], "-", 
     FractionBox[
      RowBox[{"q", " ", 
       SuperscriptBox["x", "3"], " ", 
       RowBox[{"Log", "[", "x", "]"}]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], " ", 
       RowBox[{"Log", "[", "x", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", 
       SuperscriptBox["y", "3"], " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["y", "2"], " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", 
       SuperscriptBox["y", "2"], " ", "z", " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", 
       SuperscriptBox["y", "3"], " ", "z", " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "3"]}]], "-", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", "y", " ", 
       SuperscriptBox["z", "2"], " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", "x", " ", 
       SuperscriptBox["z", "3"], " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"192", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
     FractionBox[
      RowBox[{"q", " ", "y", " ", 
       SuperscriptBox["z", "3"], " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
   3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
   3.6615456906623697`*^9}, {3.6615459684671373`*^9, 3.661545974336686*^9}, {
   3.6615461621997137`*^9, 3.661546164751027*^9}, 3.6748384992134943`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C00funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C00funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C00funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C00funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C00funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
   3.661545976723319*^9, 3.6615460255038643`*^9}, {3.661546170055657*^9, 
   3.661546214562175*^9}, {3.66170554278883*^9, 3.6617055440221567`*^9}, 
   3.674838499670514*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C00funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"y", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz12", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C00funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C00funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661705597065672*^9, 3.6617056542082987`*^9}, 
   3.661810100658122*^9, 3.6748385011982937`*^9, {3.674838621332958*^9, 
   3.674838657826973*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"C00funcz12", "[", 
    RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{"C00funcz23", "[", 
    RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{"C00funcz13", "[", 
    RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.661705559725108*^9, 3.66170556948503*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "q"}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"(", 
     RowBox[{"9", "-", 
      RowBox[{"6", " ", "EulerGamma"}], "+", 
      RowBox[{"Log", "[", "4096", "]"}], "+", 
      RowBox[{"6", " ", 
       RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "-", 
   RowBox[{"6", " ", "z", " ", 
    RowBox[{"Log", "[", "z", "]"}]}]}], 
  RowBox[{"24", " ", "z"}]]], "Output",
 CellChangeTimes->{3.6617055708398323`*^9, 3.661810231503509*^9, 
  3.674840153953733*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"DirectedInfinity", "[", 
   RowBox[{"-", 
    RowBox[{"Sign", "[", "q", "]"}]}], "]"}], 
  RowBox[{"Sign", "[", "x", "]"}]]], "Output",
 CellChangeTimes->{3.6617055708398323`*^9, 3.661810231503509*^9, 
  3.6748401539546957`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "q"}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"(", 
     RowBox[{"9", "-", 
      RowBox[{"6", " ", "EulerGamma"}], "+", 
      RowBox[{"Log", "[", "4096", "]"}], "+", 
      RowBox[{"6", " ", 
       RowBox[{"Log", "[", "\[Pi]", "]"}]}]}], ")"}]}], "-", 
   RowBox[{"6", " ", "y", " ", 
    RowBox[{"Log", "[", "y", "]"}]}]}], 
  RowBox[{"24", " ", "y"}]]], "Output",
 CellChangeTimes->{3.6617055708398323`*^9, 3.661810231503509*^9, 
  3.6748401539555407`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C11temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
   3.6615427066237087`*^9, 3.6615427367168493`*^9}, 3.661545675141267*^9, 
   3.6615459611157913`*^9, {3.661546127788756*^9, 3.661546148505188*^9}, {
   3.661546349543713*^9, 3.6615463568205423`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"y", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "3"}], " ", "x", " ", "y"}], "+", 
        SuperscriptBox["y", "2"], "+", 
        RowBox[{"5", " ", "x", " ", "z"}], "-", 
        RowBox[{"3", " ", "y", " ", "z"}]}], ")"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["y", "3"], " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox["y", "2"]}], "+", 
           RowBox[{"8", " ", "y", " ", "z"}], "+", 
           RowBox[{"17", " ", 
            SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
        RowBox[{"x", " ", 
         SuperscriptBox["y", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "3"], "-", 
           RowBox[{"3", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "-", 
           RowBox[{"21", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "-", 
           RowBox[{"49", " ", 
            SuperscriptBox["z", "3"]}]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["x", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "3"], "-", 
           RowBox[{"5", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "+", 
           RowBox[{"13", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           RowBox[{"3", " ", 
            SuperscriptBox["z", "3"]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["x", "2"], " ", "y", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "5"}], " ", 
            SuperscriptBox["y", "3"]}], "+", 
           RowBox[{"12", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "+", 
           RowBox[{"21", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           RowBox[{"44", " ", 
            SuperscriptBox["z", "3"]}]}], ")"}]}]}], ")"}]}], 
     RowBox[{"384", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "4"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["x", "4"]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "+", 
       FractionBox[
        SuperscriptBox["x", "3"], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"y", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"x", " ", "y", " ", 
             RowBox[{"(", 
              RowBox[{"y", "-", 
               RowBox[{"3", " ", "z"}]}], ")"}], " ", "z"}], "+", 
            RowBox[{
             SuperscriptBox["y", "2"], " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{
             SuperscriptBox["x", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["y", "2"], "-", 
               RowBox[{"3", " ", "y", " ", "z"}], "+", 
               RowBox[{"3", " ", 
                SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", "y", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "4"}], " ", 
            SuperscriptBox["x", "3"], " ", 
            SuperscriptBox["z", "3"]}], "+", 
           RowBox[{
            SuperscriptBox["y", "3"], " ", 
            SuperscriptBox["z", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}], "+", 
           RowBox[{"2", " ", "x", " ", 
            SuperscriptBox["y", "2"], " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["y", "2"], "-", 
              RowBox[{"5", " ", "y", " ", "z"}], "-", 
              RowBox[{"2", " ", 
               SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
           RowBox[{
            SuperscriptBox["x", "2"], " ", "y", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["y", "3"], "-", 
              RowBox[{"5", " ", 
               SuperscriptBox["y", "2"], " ", "z"}], "+", 
              RowBox[{"10", " ", "y", " ", 
               SuperscriptBox["z", "2"]}], "+", 
              RowBox[{"6", " ", 
               SuperscriptBox["z", "3"]}]}], ")"}]}]}], ")"}]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["z", "3"], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["z", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "4"}], " ", "x", " ", "y"}], "+", 
           RowBox[{"z", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.66154596139646*^9, {3.661546130459319*^9, 
   3.661546149404558*^9}, {3.66154635052914*^9, 3.661546359423377*^9}, 
   3.661612014682691*^9, 3.661615923603366*^9, 3.6616380609954653`*^9, 
   3.6617048535563917`*^9, 3.6618102315389853`*^9, 3.674840153999425*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C11funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"y", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "3"}], " ", "x", " ", "y"}], "+", 
         SuperscriptBox["y", "2"], "+", 
         RowBox[{"5", " ", "x", " ", "z"}], "-", 
         RowBox[{"3", " ", "y", " ", "z"}]}], ")"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"q", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["y", "3"], " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              SuperscriptBox["y", "2"]}], "+", 
             RowBox[{"8", " ", "y", " ", "z"}], "+", 
             RowBox[{"17", " ", 
              SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
          RowBox[{"x", " ", 
           SuperscriptBox["y", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "3"], "-", 
             RowBox[{"3", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "-", 
             RowBox[{"21", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "-", 
             RowBox[{"49", " ", 
              SuperscriptBox["z", "3"]}]}], ")"}]}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["x", "3"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "3"], "-", 
             RowBox[{"5", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "+", 
             RowBox[{"13", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             RowBox[{"3", " ", 
              SuperscriptBox["z", "3"]}]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["x", "2"], " ", "y", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "5"}], " ", 
              SuperscriptBox["y", "3"]}], "+", 
             RowBox[{"12", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "+", 
             RowBox[{"21", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             RowBox[{"44", " ", 
              SuperscriptBox["z", "3"]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"384", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"y", "-", "z"}], ")"}], "4"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["x", "4"]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"]}]], "+", 
        FractionBox[
         SuperscriptBox["x", "3"], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"y", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"x", " ", "y", " ", 
              RowBox[{"(", 
               RowBox[{"y", "-", 
                RowBox[{"3", " ", "z"}]}], ")"}], " ", "z"}], "+", 
             RowBox[{
              SuperscriptBox["y", "2"], " ", 
              SuperscriptBox["z", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["x", "2"], " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["y", "2"], "-", 
                RowBox[{"3", " ", "y", " ", "z"}], "+", 
                RowBox[{"3", " ", 
                 SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", "y", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", 
             SuperscriptBox["x", "3"], " ", 
             SuperscriptBox["z", "3"]}], "+", 
            RowBox[{
             SuperscriptBox["y", "3"], " ", 
             SuperscriptBox["z", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}], "+", 
            RowBox[{"2", " ", "x", " ", 
             SuperscriptBox["y", "2"], " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["y", "2"], "-", 
               RowBox[{"5", " ", "y", " ", "z"}], "-", 
               RowBox[{"2", " ", 
                SuperscriptBox["z", "2"]}]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox["x", "2"], " ", "y", " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["y", "3"], "-", 
               RowBox[{"5", " ", 
                SuperscriptBox["y", "2"], " ", "z"}], "+", 
               RowBox[{"10", " ", "y", " ", 
                SuperscriptBox["z", "2"]}], "+", 
               RowBox[{"6", " ", 
                SuperscriptBox["z", "3"]}]}], ")"}]}]}], ")"}]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          SuperscriptBox["z", "3"], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["z", "3"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", "x", " ", "y"}], "+", 
            RowBox[{"z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
   3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
   3.6615456906623697`*^9}, {3.6615459684671373`*^9, 3.661545974336686*^9}, {
   3.6615461621997137`*^9, 3.661546164751027*^9}, {3.66154636481201*^9, 
   3.6615463693061132`*^9}, 3.67483850239244*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C11funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C11funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C11funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C11funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C11funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
   3.661545976723319*^9, 3.6615460255038643`*^9}, {3.661546170055657*^9, 
   3.661546214562175*^9}, {3.6615466857858562`*^9, 3.661546750117469*^9}, {
   3.661705665627289*^9, 3.661705666605506*^9}, 3.674838503764721*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C11funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"x", "\[Equal]", "0"}], "||", 
          RowBox[{"z", "\[Equal]", "0"}]}], ")"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C11funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C11funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661705698684506*^9, 3.661705770185101*^9}, 
   3.661810104998807*^9, 3.6748385075066423`*^9, {3.674838665666027*^9, 
   3.674838687662073*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C11funcz1d", "[", 
  RowBox[{"q", ",", "x", ",", "y"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz2d", "[", 
  RowBox[{"q", ",", "x", ",", "y"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz3d", "[", 
  RowBox[{"q", ",", "x", ",", "y"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz1", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz2", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz3", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661706106481801*^9, 3.661706155481874*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"9", " ", "q"}], "-", 
    RowBox[{"80", " ", "x"}]}], 
   RowBox[{"720", " ", 
    SuperscriptBox["x", "2"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "3"}], " ", "q"}], "+", 
      RowBox[{"10", " ", "x"}]}], ")"}], " ", "y"}], 
   RowBox[{"360", " ", 
    SuperscriptBox["x", "3"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"27", " ", "q"}], "-", 
      RowBox[{"56", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "2"]}], 
   RowBox[{"5040", " ", 
    SuperscriptBox["x", "4"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "9"}], " ", "q"}], "+", 
      RowBox[{"14", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "3"]}], 
   RowBox[{"2520", " ", 
    SuperscriptBox["x", "5"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"25", " ", "q"}], "-", 
      RowBox[{"32", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "4"]}], 
   RowBox[{"10080", " ", 
    SuperscriptBox["x", "6"]}]]}]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158396147*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"3", " ", "q"}], "-", 
    RowBox[{"8", " ", "x"}]}], 
   RowBox[{"24", " ", 
    SuperscriptBox["x", "2"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "q"}], "+", "x"}], ")"}], " ", "y"}], 
   RowBox[{"6", " ", 
    SuperscriptBox["x", "3"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"27", " ", "q"}], "-", 
      RowBox[{"16", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "2"]}], 
   RowBox[{"144", " ", 
    SuperscriptBox["x", "4"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "12"}], " ", "q"}], "+", 
      RowBox[{"5", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "3"]}], 
   RowBox[{"60", " ", 
    SuperscriptBox["x", "5"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"25", " ", "q"}], "-", 
      RowBox[{"8", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "4"]}], 
   RowBox[{"120", " ", 
    SuperscriptBox["x", "6"]}]]}]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158397222*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"3", " ", "q"}], "-", 
    RowBox[{"16", " ", "x"}]}], 
   RowBox[{"144", " ", 
    SuperscriptBox["x", "2"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "q"}], "+", 
      RowBox[{"5", " ", "x"}]}], ")"}], " ", "y"}], 
   RowBox[{"60", " ", 
    SuperscriptBox["x", "3"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"5", " ", "q"}], "-", 
      RowBox[{"8", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "2"]}], 
   RowBox[{"120", " ", 
    SuperscriptBox["x", "4"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "6"}], " ", "q"}], "+", 
      RowBox[{"7", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "3"]}], 
   RowBox[{"126", " ", 
    SuperscriptBox["x", "5"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"35", " ", "q"}], "-", 
      RowBox[{"32", " ", "x"}]}], ")"}], " ", 
    SuperscriptBox["y", "4"]}], 
   RowBox[{"672", " ", 
    SuperscriptBox["x", "6"]}]]}]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158398266*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"y", "-", "z"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "4"}], " ", 
       RowBox[{"(", 
        RowBox[{"y", "-", 
         RowBox[{"3", " ", "z"}]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}], "+", 
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["y", "2"], "-", 
         RowBox[{"8", " ", "y", " ", "z"}], "-", 
         RowBox[{"17", " ", 
          SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["z", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "4"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}], "+", 
      RowBox[{"3", " ", "q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}], " ", 
    RowBox[{"Log", "[", "y", "]"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["z", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}], "-", 
      RowBox[{"3", " ", "q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "y"}], "+", "z"}], ")"}]}]}], ")"}], " ", 
    RowBox[{"Log", "[", "z", "]"}]}]}], 
  RowBox[{"24", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"y", "-", "z"}], ")"}], "5"]}]]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158400083*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"3", " ", "q", " ", 
    RowBox[{"(", 
     RowBox[{"x", "-", "z"}], ")"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "3"}], " ", "q"}], "-", 
      RowBox[{"4", " ", "x"}], "+", 
      RowBox[{"4", " ", "z"}]}], ")"}], " ", 
    RowBox[{"Log", "[", "x", "]"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "q"}], "+", 
      RowBox[{"4", " ", "x"}], "-", 
      RowBox[{"4", " ", "z"}]}], ")"}], " ", "z", " ", 
    RowBox[{"Log", "[", "z", "]"}]}]}], 
  RowBox[{"12", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "-", "z"}], ")"}], "2"], " ", "z"}]]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158401039*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox["x", "2"]}], "+", 
         RowBox[{"5", " ", "x", " ", "y"}], "-", 
         SuperscriptBox["y", "2"]}], ")"}]}], "+", 
      RowBox[{"4", " ", "y", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          SuperscriptBox["x", "2"]}], "-", 
         RowBox[{"4", " ", "x", " ", "y"}], "+", 
         SuperscriptBox["y", "2"]}], ")"}]}]}], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "q"}], "+", 
      RowBox[{"4", " ", "x"}], "-", 
      RowBox[{"4", " ", "y"}]}], ")"}], " ", "y", " ", 
    RowBox[{"Log", "[", "x", "]"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "q"}], "+", 
      RowBox[{"4", " ", "x"}], "-", 
      RowBox[{"4", " ", "y"}]}], ")"}], " ", "y", " ", 
    RowBox[{"Log", "[", "y", "]"}]}]}], 
  RowBox[{"24", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], "4"], " ", "y"}]]], "Output",
 CellChangeTimes->{{3.661706115580217*^9, 3.661706155818274*^9}, 
   3.661810235593298*^9, 3.674840158402018*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C11funcz12", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz23", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C11funcz13", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.66170568111313*^9, 3.6617056885522127`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"DirectedInfinity", "[", 
   RowBox[{
    RowBox[{"6", " ", "q"}], "-", 
    RowBox[{"8", " ", "z"}]}], "]"}], 
  SuperscriptBox[
   RowBox[{"Sign", "[", "z", "]"}], "2"]]], "Output",
 CellChangeTimes->{3.6617056890982647`*^9, 3.661706086417391*^9, 
  3.661706208055338*^9, 3.661810235692062*^9, 3.674840158425932*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{"Sign", "[", "x", "]"}]], "]"}]], "Output",
 CellChangeTimes->{3.6617056890982647`*^9, 3.661706086417391*^9, 
  3.661706208055338*^9, 3.661810235692062*^9, 3.6748401584270563`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"q", "-", 
   RowBox[{"4", " ", "y"}]}], 
  RowBox[{"24", " ", 
   SuperscriptBox["y", "2"]}]]], "Output",
 CellChangeTimes->{3.6617056890982647`*^9, 3.661706086417391*^9, 
  3.661706208055338*^9, 3.661810235692062*^9, 3.674840158427842*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C12temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6615426551051693`*^9, 3.6615426615567617`*^9}, {
   3.6615427066237087`*^9, 3.6615427367168493`*^9}, 3.661545675141267*^9, 
   3.6615459611157913`*^9, {3.661546127788756*^9, 3.661546148505188*^9}, {
   3.661546349543713*^9, 3.6615463568205423`*^9}, {3.661547051080598*^9, 
   3.661547051183381*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"y", " ", "z", " ", 
       RowBox[{"(", 
        RowBox[{"y", "+", "z"}], ")"}]}], "-", 
      RowBox[{"x", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["y", "2"], "+", 
         SuperscriptBox["z", "2"]}], ")"}]}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "+", 
           RowBox[{"10", " ", "y", " ", "z"}], "+", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{"y", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "3"], "-", 
           RowBox[{"7", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "-", 
           RowBox[{"7", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           SuperscriptBox["z", "3"]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["x", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "3"], "+", 
           RowBox[{"8", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "+", 
           RowBox[{"8", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           SuperscriptBox["z", "3"]}], ")"}]}], "+", 
        RowBox[{"x", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "4"], "+", 
           RowBox[{"3", " ", 
            SuperscriptBox["y", "3"], " ", "z"}], "+", 
           RowBox[{"28", " ", 
            SuperscriptBox["y", "2"], " ", 
            SuperscriptBox["z", "2"]}], "+", 
           RowBox[{"3", " ", "y", " ", 
            SuperscriptBox["z", "3"]}], "+", 
           SuperscriptBox["z", "4"]}], ")"}]}]}], ")"}]}], 
     RowBox[{"192", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], "2"], " ", "y", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "4"], " ", "z"}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"q", " ", "x"}], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], "+", 
       FractionBox[
        SuperscriptBox["x", "3"], 
        RowBox[{"96", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["y", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{"y", "-", 
               RowBox[{"3", " ", "z"}]}], ")"}]}], "+", 
            RowBox[{"2", " ", "y", " ", "z"}]}], ")"}]}], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "+", 
       FractionBox[
        RowBox[{"q", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"6", " ", 
            SuperscriptBox["y", "3"]}], "+", 
           RowBox[{"3", " ", 
            SuperscriptBox["x", "2"], " ", 
            RowBox[{"(", 
             RowBox[{"y", "+", "z"}], ")"}]}], "+", 
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "8"}], " ", 
               SuperscriptBox["y", "2"]}], "-", 
              RowBox[{"5", " ", "y", " ", "z"}], "+", 
              SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
        RowBox[{"96", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["z", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "y", " ", "z"}], "+", 
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", "y"}], "+", "z"}], ")"}]}]}], 
           ")"}]}], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "+", 
       FractionBox[
        RowBox[{"q", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"6", " ", 
            SuperscriptBox["z", "3"]}], "+", 
           RowBox[{"3", " ", 
            SuperscriptBox["x", "2"], " ", 
            RowBox[{"(", 
             RowBox[{"y", "+", "z"}], ")"}]}], "+", 
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{
              SuperscriptBox["y", "2"], "-", 
              RowBox[{"5", " ", "y", " ", "z"}], "-", 
              RowBox[{"8", " ", 
               SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
        RowBox[{"96", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.66154596139646*^9, {3.661546130459319*^9, 
   3.661546149404558*^9}, {3.66154635052914*^9, 3.661546359423377*^9}, 
   3.661547051971322*^9, 3.6616120188731117`*^9, 3.661615927682551*^9, 
   3.661638065501511*^9, 3.661704857857039*^9, 3.661810235722754*^9, 
   3.6748401584637747`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C12funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"y", " ", "z", " ", 
        RowBox[{"(", 
         RowBox[{"y", "+", "z"}], ")"}]}], "-", 
       RowBox[{"x", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "2"]}], ")"}]}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"q", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["x", "3"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "2"], "+", 
             RowBox[{"10", " ", "y", " ", "z"}], "+", 
             SuperscriptBox["z", "2"]}], ")"}]}], "+", 
          RowBox[{"y", " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "3"], "-", 
             RowBox[{"7", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "-", 
             RowBox[{"7", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             SuperscriptBox["z", "3"]}], ")"}]}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["x", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "3"], "+", 
             RowBox[{"8", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "+", 
             RowBox[{"8", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             SuperscriptBox["z", "3"]}], ")"}]}], "+", 
          RowBox[{"x", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["y", "4"], "+", 
             RowBox[{"3", " ", 
              SuperscriptBox["y", "3"], " ", "z"}], "+", 
             RowBox[{"28", " ", 
              SuperscriptBox["y", "2"], " ", 
              SuperscriptBox["z", "2"]}], "+", 
             RowBox[{"3", " ", "y", " ", 
              SuperscriptBox["z", "3"]}], "+", 
             SuperscriptBox["z", "4"]}], ")"}]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"192", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], "2"], " ", "y", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"y", "-", "z"}], ")"}], "4"], " ", "z"}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"q", " ", "x"}], 
          RowBox[{"96", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], "+", 
        FractionBox[
         SuperscriptBox["x", "3"], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "2"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["y", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"x", " ", 
              RowBox[{"(", 
               RowBox[{"y", "-", 
                RowBox[{"3", " ", "z"}]}], ")"}]}], "+", 
             RowBox[{"2", " ", "y", " ", "z"}]}], ")"}]}], 
          RowBox[{"96", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "+", 
        FractionBox[
         RowBox[{"q", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"6", " ", 
             SuperscriptBox["y", "3"]}], "+", 
            RowBox[{"3", " ", 
             SuperscriptBox["x", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}]}], "+", 
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "8"}], " ", 
                SuperscriptBox["y", "2"]}], "-", 
               RowBox[{"5", " ", "y", " ", "z"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["z", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", " ", "y", " ", "z"}], "+", 
             RowBox[{"x", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "3"}], " ", "y"}], "+", "z"}], ")"}]}]}], 
            ")"}]}], 
          RowBox[{"96", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "+", 
        FractionBox[
         RowBox[{"q", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"6", " ", 
             SuperscriptBox["z", "3"]}], "+", 
            RowBox[{"3", " ", 
             SuperscriptBox["x", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}]}], "+", 
            RowBox[{"x", " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["y", "2"], "-", 
               RowBox[{"5", " ", "y", " ", "z"}], "-", 
               RowBox[{"8", " ", 
                SuperscriptBox["z", "2"]}]}], ")"}]}]}], ")"}]}], 
         RowBox[{"96", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
   3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
   3.6615456906623697`*^9}, {3.6615459684671373`*^9, 3.661545974336686*^9}, {
   3.6615461621997137`*^9, 3.661546164751027*^9}, {3.66154636481201*^9, 
   3.6615463693061132`*^9}, {3.66154705907458*^9, 3.661547061341311*^9}, 
   3.6748385089464827`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C12funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C12funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C12funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C12funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C12funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
   3.661545976723319*^9, 3.6615460255038643`*^9}, {3.661546170055657*^9, 
   3.661546214562175*^9}, {3.6615466857858562`*^9, 3.661546750117469*^9}, {
   3.661547064854897*^9, 3.661547118369287*^9}, {3.661705837531633*^9, 
   3.6617058594585876`*^9}, 3.674838510083284*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C12funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "||", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C12funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C12funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661705867886525*^9, 3.6617058732051992`*^9}, {
   3.661705955213575*^9, 3.661705964577623*^9}, {3.661706013873543*^9, 
   3.661706061114262*^9}, 3.6618101125920143`*^9, 3.6748385109515457`*^9, {
   3.674838696938717*^9, 3.6748387173410587`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C12funcz1d", "[", 
  RowBox[{"q", ",", "y", ",", 
   RowBox[{"z", "-", "y"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C12funcz2d", "[", 
  RowBox[{"q", ",", "x", ",", 
   RowBox[{"z", "-", "x"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C12funcz3d", "[", 
  RowBox[{"q", ",", "x", ",", 
   RowBox[{"y", "-", "x"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C12funcz1", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C12funcz2", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C12funcz3", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661705824067*^9, 3.661705826403372*^9}, {
  3.661705881918892*^9, 3.661705939162651*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"3", " ", "q"}], "-", 
    RowBox[{"5", " ", 
     SuperscriptBox["y", "4"]}]}], 
   RowBox[{"90", " ", 
    SuperscriptBox["y", "5"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "3"}], " ", "q"}], "+", 
      SuperscriptBox["y", "4"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "y"}], "+", "z"}], ")"}]}], 
   RowBox[{"36", " ", 
    SuperscriptBox["y", "6"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"60", " ", "q"}], "-", 
      RowBox[{"7", " ", 
       SuperscriptBox["y", "4"]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "y"}], "+", "z"}], ")"}], "2"]}], 
   RowBox[{"420", " ", 
    SuperscriptBox["y", "7"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "75"}], " ", "q"}], "+", 
      RowBox[{"4", " ", 
       SuperscriptBox["y", "4"]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}], 
   RowBox[{"360", " ", 
    SuperscriptBox["y", "8"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"35", " ", "q"}], "-", 
      SuperscriptBox["y", "4"]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "y"}], "+", "z"}], ")"}], "4"]}], 
   RowBox[{"126", " ", 
    SuperscriptBox["y", "9"]}]]}]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.674840161994484*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{
    RowBox[{"Sign", "[", "x", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Sign", "[", "z", "]"}], "3"]}]], "]"}]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.674840161995673*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{
    RowBox[{"Sign", "[", "x", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Sign", "[", "y", "]"}], "3"]}]], "]"}]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.674840161996563*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["y", "2"], "-", 
      SuperscriptBox["z", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox["z", "2"]}], "+", 
      RowBox[{"q", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["y", "2"], "-", 
         RowBox[{"8", " ", "y", " ", "z"}], "+", 
         SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SuperscriptBox["y", "2"], " ", 
    SuperscriptBox["z", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "q"}], "+", 
      RowBox[{"y", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"], " ", "z"}]}], ")"}], " ", 
    RowBox[{"Log", "[", "y", "]"}]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox["y", "2"], " ", 
    SuperscriptBox["z", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "q"}], "+", 
      RowBox[{"y", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"], " ", "z"}]}], ")"}], " ", 
    RowBox[{"Log", "[", "z", "]"}]}]}], 
  RowBox[{"12", " ", 
   SuperscriptBox["y", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"y", "-", "z"}], ")"}], "5"], " ", 
   SuperscriptBox["z", "2"]}]]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.674840161997609*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{
    RowBox[{"Sign", "[", "x", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Sign", "[", "z", "]"}], "3"]}]], "]"}]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.67484016199858*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{
    RowBox[{"Sign", "[", "x", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Sign", "[", "y", "]"}], "3"]}]], "]"}]], "Output",
 CellChangeTimes->{
  3.661705826787137*^9, {3.6617058640334806`*^9, 3.661705939526907*^9}, 
   3.661705971589356*^9, 3.66181023886381*^9, 3.6748401619994707`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C22temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"qsq", "\[Rule]", "q"}], ",", 
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.661705808221244*^9, 3.6617058183984737`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"z", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"5", " ", "x", " ", "y"}], "-", 
        RowBox[{"3", " ", "x", " ", "z"}], "-", 
        RowBox[{"3", " ", "y", " ", "z"}], "+", 
        SuperscriptBox["z", "2"]}], ")"}]}], 
     RowBox[{"96", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
    FractionBox[
     RowBox[{"q", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", " ", 
         SuperscriptBox["z", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"17", " ", 
            SuperscriptBox["y", "2"]}], "+", 
           RowBox[{"8", " ", "y", " ", "z"}], "-", 
           SuperscriptBox["z", "2"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["x", "2"], " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"44", " ", 
            SuperscriptBox["y", "3"]}], "+", 
           RowBox[{"21", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "+", 
           RowBox[{"12", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "-", 
           RowBox[{"5", " ", 
            SuperscriptBox["z", "3"]}]}], ")"}]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["x", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", 
            SuperscriptBox["y", "3"]}], "+", 
           RowBox[{"13", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "-", 
           RowBox[{"5", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           SuperscriptBox["z", "3"]}], ")"}]}], "+", 
        RowBox[{"x", " ", 
         SuperscriptBox["z", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "49"}], " ", 
            SuperscriptBox["y", "3"]}], "-", 
           RowBox[{"21", " ", 
            SuperscriptBox["y", "2"], " ", "z"}], "-", 
           RowBox[{"3", " ", "y", " ", 
            SuperscriptBox["z", "2"]}], "+", 
           SuperscriptBox["z", "3"]}], ")"}]}]}], ")"}]}], 
     RowBox[{"384", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}], "4"]}]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["x", "4"]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "4"]}]], "+", 
       FractionBox[
        SuperscriptBox["x", "3"], 
        RowBox[{"48", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "x", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["y", "3"], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", 
         SuperscriptBox["y", "3"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["y", "2"], "-", 
           RowBox[{"4", " ", "x", " ", "z"}], "+", 
           RowBox[{"3", " ", "y", " ", "z"}]}], ")"}]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SuperscriptBox["y", "2"], " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{"x", " ", "y", " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "3"}], " ", "y"}], "+", "z"}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox["x", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"3", " ", 
                SuperscriptBox["y", "2"]}], "-", 
               RowBox[{"3", " ", "y", " ", "z"}], "+", 
               SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "-", 
       FractionBox[
        RowBox[{"q", " ", "z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "4"}], " ", 
            SuperscriptBox["x", "3"], " ", 
            SuperscriptBox["y", "3"]}], "+", 
           RowBox[{
            SuperscriptBox["y", "2"], " ", 
            SuperscriptBox["z", "3"], " ", 
            RowBox[{"(", 
             RowBox[{"y", "+", 
              RowBox[{"3", " ", "z"}]}], ")"}]}], "-", 
           RowBox[{"2", " ", "x", " ", "y", " ", 
            SuperscriptBox["z", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               SuperscriptBox["y", "2"]}], "+", 
              RowBox[{"5", " ", "y", " ", "z"}], "-", 
              SuperscriptBox["z", "2"]}], ")"}]}], "+", 
           RowBox[{
            SuperscriptBox["x", "2"], " ", "z", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"6", " ", 
               SuperscriptBox["y", "3"]}], "+", 
              RowBox[{"10", " ", 
               SuperscriptBox["y", "2"], " ", "z"}], "-", 
              RowBox[{"5", " ", "y", " ", 
               SuperscriptBox["z", "2"]}], "+", 
              SuperscriptBox["z", "3"]}], ")"}]}]}], ")"}]}], 
        RowBox[{"64", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "z"}], ")"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
     RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.661542662465109*^9, {3.6615427281960783`*^9, 3.661542737470388*^9}, 
   3.661545684133011*^9, 3.66154596139646*^9, {3.661546130459319*^9, 
   3.661546149404558*^9}, {3.66154635052914*^9, 3.661546359423377*^9}, 
   3.661547051971322*^9, 3.661547244053384*^9, 3.6616120222085752`*^9, 
   3.6616159310007668`*^9, 3.661638069241501*^9, 3.66170486108834*^9, 
   3.661705819917737*^9, 3.6618102389294243`*^9, 3.6748401620308113`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"C22funcbase", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"z", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x", " ", "y"}], "-", 
         RowBox[{"3", " ", "x", " ", "z"}], "-", 
         RowBox[{"3", " ", "y", " ", "z"}], "+", 
         SuperscriptBox["z", "2"]}], ")"}]}], 
      RowBox[{"96", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "z"}], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"y", "-", "z"}], ")"}], "2"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"q", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"y", " ", 
           SuperscriptBox["z", "3"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"17", " ", 
              SuperscriptBox["y", "2"]}], "+", 
             RowBox[{"8", " ", "y", " ", "z"}], "-", 
             SuperscriptBox["z", "2"]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["x", "2"], " ", "z", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"44", " ", 
              SuperscriptBox["y", "3"]}], "+", 
             RowBox[{"21", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "+", 
             RowBox[{"12", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "-", 
             RowBox[{"5", " ", 
              SuperscriptBox["z", "3"]}]}], ")"}]}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["x", "3"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", " ", 
              SuperscriptBox["y", "3"]}], "+", 
             RowBox[{"13", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "-", 
             RowBox[{"5", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             SuperscriptBox["z", "3"]}], ")"}]}], "+", 
          RowBox[{"x", " ", 
           SuperscriptBox["z", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "49"}], " ", 
              SuperscriptBox["y", "3"]}], "-", 
             RowBox[{"21", " ", 
              SuperscriptBox["y", "2"], " ", "z"}], "-", 
             RowBox[{"3", " ", "y", " ", 
              SuperscriptBox["z", "2"]}], "+", 
             SuperscriptBox["z", "3"]}], ")"}]}]}], ")"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"384", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"y", "-", "z"}], ")"}], "4"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["x", "4"]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "4"]}]], "+", 
        FractionBox[
         SuperscriptBox["x", "3"], 
         RowBox[{"48", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"x", "-", "y"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "3"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          SuperscriptBox["y", "3"], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"y", "-", "z"}], ")"}], "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", 
          SuperscriptBox["y", "3"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", "2"], "-", 
            RowBox[{"4", " ", "x", " ", "z"}], "+", 
            RowBox[{"3", " ", "y", " ", "z"}]}], ")"}]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"y", "-", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"z", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["y", "2"], " ", 
              SuperscriptBox["z", "2"]}], "+", 
             RowBox[{"x", " ", "y", " ", "z", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "3"}], " ", "y"}], "+", "z"}], ")"}]}], "+", 
             RowBox[{
              SuperscriptBox["x", "2"], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"3", " ", 
                 SuperscriptBox["y", "2"]}], "-", 
                RowBox[{"3", " ", "y", " ", "z"}], "+", 
                SuperscriptBox["z", "2"]}], ")"}]}]}], ")"}]}], 
          RowBox[{"48", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "z"}], ")"}], "3"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "y"}], "+", "z"}], ")"}], "3"]}]]}], "-", 
        FractionBox[
         RowBox[{"q", " ", "z", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "4"}], " ", 
             SuperscriptBox["x", "3"], " ", 
             SuperscriptBox["y", "3"]}], "+", 
            RowBox[{
             SuperscriptBox["y", "2"], " ", 
             SuperscriptBox["z", "3"], " ", 
             RowBox[{"(", 
              RowBox[{"y", "+", 
               RowBox[{"3", " ", "z"}]}], ")"}]}], "-", 
            RowBox[{"2", " ", "x", " ", "y", " ", 
             SuperscriptBox["z", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox["y", "2"]}], "+", 
               RowBox[{"5", " ", "y", " ", "z"}], "-", 
               SuperscriptBox["z", "2"]}], ")"}]}], "+", 
            RowBox[{
             SuperscriptBox["x", "2"], " ", "z", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"6", " ", 
                SuperscriptBox["y", "3"]}], "+", 
               RowBox[{"10", " ", 
                SuperscriptBox["y", "2"], " ", "z"}], "-", 
               RowBox[{"5", " ", "y", " ", 
                SuperscriptBox["z", "2"]}], "+", 
               SuperscriptBox["z", "3"]}], ")"}]}]}], ")"}]}], 
         RowBox[{"64", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "z"}], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "y"}], "+", "z"}], ")"}], "5"]}]]}], ")"}], " ", 
      RowBox[{"Log", "[", "z", "]"}]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.6615426917786627`*^9, 3.661542698564436*^9}, {
   3.6615427448114433`*^9, 3.6615427565450573`*^9}, {3.6615456780248404`*^9, 
   3.6615456906623697`*^9}, {3.6615459684671373`*^9, 3.661545974336686*^9}, {
   3.6615461621997137`*^9, 3.661546164751027*^9}, {3.66154636481201*^9, 
   3.6615463693061132`*^9}, {3.66154705907458*^9, 3.661547061341311*^9}, {
   3.661547251520034*^9, 3.661547254191555*^9}, 3.674838511833344*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "C22funcbase"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"two", " ", "zeros"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz12", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "\[Epsilon]1", ",", "\[Epsilon]2", ",", "z"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{
       RowBox[{"FFF", "[", 
        RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "\[Epsilon]2"}], 
        "]"}], ",", 
       RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22funcz13", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "\[Epsilon]2"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"\[Epsilon]2", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz1d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "\[Epsilon]1", ",", "x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz2d", "[", 
   RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22funcz3d", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"q", ",", "x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz1", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "\[Epsilon]1", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcz2", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"q", ",", "x", ",", "\[Epsilon]1", ",", "z"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22funcz3", "[", 
    RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22funcd123", "[", 
    RowBox[{"q_", ",", "x_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"q", ",", "x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"single", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minimize", " ", "this", " ", "error",
      " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcd13", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcd12", "[", 
   RowBox[{"q_", ",", "x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22funcd23", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.66154571208228*^9, 3.66154576518797*^9}, {
   3.661545976723319*^9, 3.6615460255038643`*^9}, {3.661546170055657*^9, 
   3.661546214562175*^9}, {3.6615466857858562`*^9, 3.661546750117469*^9}, {
   3.661547064854897*^9, 3.661547118369287*^9}, {3.661547257260109*^9, 
   3.661547304441201*^9}, {3.6617060735626802`*^9, 3.66170607421562*^9}, 
   3.674838512665822*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C22funcAcc", "[", 
   RowBox[{"q_", ",", "x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"x", "\[Equal]", "0"}], "||", 
          RowBox[{"y", "\[Equal]", "0"}]}], ")"}], "&&", 
        RowBox[{"z", "\[Equal]", "0"}]}], ",", "0", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{"y", "\[Equal]", "0"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz12", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz1d", "[", 
         RowBox[{"q", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz2d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz3d", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz1", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz2", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz3", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd123", "[", 
         RowBox[{"q", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd23", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd13", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd12", "[", 
         RowBox[{"q", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcbase", "[", 
         RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22func0Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz1d", "[", 
         RowBox[{"0", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz2d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz3d", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz1", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz2", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcz3", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd123", "[", 
         RowBox[{"0", ",", "x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd23", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd13", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcd12", "[", 
         RowBox[{"0", ",", "x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"C22funcbase", "[", 
         RowBox[{"0", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22func1Acc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"S12", "*", "S23", "*", "S13"}], "\[Equal]", "0"}], ",", "0", 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz1d", "[", 
           RowBox[{"qsq", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz2d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz3d", "[", 
           RowBox[{"qsq", ",", "x", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz1", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz2", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcz3", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcd123", "[", 
           RowBox[{"0", ",", "x", ",", 
            RowBox[{"y", "-", "x"}], ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcd23", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "y"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcd13", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", 
            RowBox[{"z", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcd12", "[", 
           RowBox[{"qsq", ",", "x", ",", "z", ",", 
            RowBox[{"y", "-", "x"}]}], "]"}], ",", "qsq"}], "]"}], "]"}], ",",
        "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"Coefficient", "[", 
         RowBox[{
          RowBox[{"C22funcbase", "[", 
           RowBox[{"qsq", ",", "x", ",", "y", ",", "z"}], "]"}], ",", "qsq"}],
          "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.661706295493123*^9, 3.661706374268281*^9}, 
   3.661810118165506*^9, 3.674838513491253*^9, {3.6748387241444597`*^9, 
   3.674838742498227*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"C22funcz12", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C22funcz23", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"C22funcz13", "[", 
  RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6617061683797607`*^9, 3.661706173965008*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"q", "-", 
   RowBox[{"4", " ", "z"}]}], 
  RowBox[{"24", " ", 
   SuperscriptBox["z", "2"]}]]], "Output",
 CellChangeTimes->{3.6617061746956873`*^9, 3.661810243095656*^9, 
  3.6748401663826933`*^9}],

Cell[BoxData[
 RowBox[{"DirectedInfinity", "[", 
  FractionBox[
   RowBox[{"Sign", "[", "q", "]"}], 
   RowBox[{"Sign", "[", "x", "]"}]], "]"}]], "Output",
 CellChangeTimes->{3.6617061746956873`*^9, 3.661810243095656*^9, 
  3.674840166385558*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"DirectedInfinity", "[", 
   RowBox[{
    RowBox[{"6", " ", "q"}], "-", 
    RowBox[{"8", " ", "y"}]}], "]"}], 
  SuperscriptBox[
   RowBox[{"Sign", "[", "y", "]"}], "2"]]], "Output",
 CellChangeTimes->{3.6617061746956873`*^9, 3.661810243095656*^9, 
  3.674840166388994*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Comparison", "Subsubsection",
 CellChangeTimes->{{3.66154560027285*^9, 3.661545601624076*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0funcbase", "[", 
   RowBox[{"0", ",", 
    SuperscriptBox["1000.", "2"], ",", 
    SuperscriptBox["10000.01", "2"], ",", 
    SuperscriptBox["100.0001", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0func", "[", 
   RowBox[{"0", ",", 
    SuperscriptBox["1000.", "2"], ",", 
    SuperscriptBox["10000.01", "2"], ",", 
    SuperscriptBox["100.0001", "2"]}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661544957576106*^9, 3.6615450552476673`*^9}, 
   3.674838515305444*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.00007700000000000000138604405730546887`1.907090638452106", ",", 
   RowBox[{"-", "4.605623390453036`*^-8"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661544969719858*^9, 3.661544987096648*^9}, {
  3.6615450192379*^9, 3.661545071385734*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00011100000000000000111802928026705217`2.0659228920662813", ",", 
   RowBox[{"-", "4.605623390453036`*^-8"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661544969719858*^9, 3.661544987096648*^9}, {
  3.6615450192379*^9, 3.6615450713906803`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"B00comp", "=", 
  RowBox[{"Compile", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", "_Real"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", "_Real"}], "}"}]}], "}"}], ",", 
    RowBox[{"B00func", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0comp", "=", 
  RowBox[{"Compile", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "_Real"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "_Real"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", "_Real"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
    RowBox[{"C0func", "[", 
     RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661544993648793*^9, 3.661544999455859*^9}, {
  3.6615451019391403`*^9, 3.6615451398490343`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"CompiledFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "}"}], ",", 
    RowBox[{"C0func", "[", 
     RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], 
    ",", "\<\"-CompiledCode-\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{3.661545140658762*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0comp", "[", 
   RowBox[{"0", ",", 
    SuperscriptBox["1000.", "2"], ",", "1020", ",", 
    SuperscriptBox["100.00001", "2"]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.661545352426629*^9, 3.6615453566249104`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00015699999999999999437776121435916821`2.2164995656888573", ",", 
   RowBox[{"-", "4.396880167176939`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.66154535759678*^9, 3.661545362127247*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C0funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["100.00001", "2"]}], "]"}], ",", 
     RowBox[{"C0comp", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["100.00001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.9998", ",", "1000.00015"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.661545144657215*^9, 3.661545335643244*^9}, 
   3.674838519073848*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ATtest", "=", 
  RowBox[{"Compile", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "}"}], ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{"C0comp", "[", 
      RowBox[{"q", ",", "x", ",", "z", ",", "y"}], "]"}], "+", 
     RowBox[{"2", "*", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "x", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{"C0comp", "[", 
      RowBox[{"q", ",", "x", ",", "z", ",", "x"}], "]"}], "+", 
     RowBox[{"2", "*", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "z", ",", "z"}], "]"}]}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.66154543052872*^9, 3.661545490092451*^9}, {
  3.6615455463861237`*^9, 3.661545557563158*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"CompiledFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "}"}], ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{"C0comp", "[", 
      RowBox[{"q", ",", "x", ",", "z", ",", "y"}], "]"}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "x", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{"C0comp", "[", 
      RowBox[{"q", ",", "x", ",", "z", ",", "x"}], "]"}], "+", 
     RowBox[{"2", " ", 
      RowBox[{"C0comp", "[", 
       RowBox[{"q", ",", "y", ",", "z", ",", "z"}], "]"}]}]}], 
    ",", "\<\"-CompiledCode-\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{3.661545490518776*^9, 3.6615455580523367`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"ATtest", "[", 
   RowBox[{"0.", ",", "1000.", ",", "1000.", ",", "1000.1"}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"ATtest", "[", 
   RowBox[{"0.", ",", "1000.", ",", "1000.2", ",", "1000.1"}], "]"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.661545496419858*^9, 3.661545535002555*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.00070399999999999998169519788149273154`2.868172572421736", ",", 
   RowBox[{"-", "0.0034998500091660174`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661545510682378*^9, 3.661545560453339*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.00061200000000000002228078832544611032`2.807351335425185", ",", 
   RowBox[{"-", "0.0034996333799935013`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661545510682378*^9, 3.661545560456411*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C1funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["101.00001", "2"]}], "]"}], ",", 
     RowBox[{"C1func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["101.00001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.95", ",", "1000.05"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661545802116233*^9, 3.661545853813985*^9}, 
   3.6748385199922323`*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C2funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["101.00001", "2"]}], "]"}], ",", 
     RowBox[{"C2func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["101.00001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.9995", ",", "1000.0005"}], "}"}]}], "]"}]], "Input",\

 CellChangeTimes->{{3.661546043864704*^9, 3.6615460745454283`*^9}, 
   3.674838520579412*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C00funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1001.001", "2"]}], "]"}], ",", 
     RowBox[{"C00func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1001.001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.95", ",", "1000.05"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661546228770274*^9, 3.661546310573491*^9}, 
   3.674838521078703*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Abs", "[", 
  FractionBox[
   RowBox[{
    SuperscriptBox["1000.", "2"], "-", 
    SuperscriptBox["1021.001", "2"]}], 
   RowBox[{
    SuperscriptBox["1000.", "2"], "+", 
    SuperscriptBox["1021.001", "2"]}]], "]"}]], "Input",
 CellChangeTimes->{{3.661546882779463*^9, 3.661546915162229*^9}}],

Cell[BoxData["0.020780526618464756`"], "Output",
 CellChangeTimes->{{3.661546891971958*^9, 3.661546915745988*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C11funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}], ",", 
     RowBox[{"C11func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "990.95", ",", "1020.05"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661546767262216*^9, 3.661546853373556*^9}, {
   3.6615469192981358`*^9, 3.6615470100361767`*^9}, {3.6615471334883347`*^9, 
   3.661547181403379*^9}, 3.674838521941348*^9}],

Cell[BoxData["\.05"], "Input",
 CellChangeTimes->{3.661546977822176*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C12funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}], ",", 
     RowBox[{"C12func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "990.95", ",", "1020.05"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661547189524043*^9, 3.6615471942190638`*^9}, 
   3.674838522622669*^9}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C22funcbase", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.00", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}], ",", 
     RowBox[{"C22func", "[", 
      RowBox[{"0", ",", 
       SuperscriptBox["1000.", "2"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["1011.001", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "990.95", ",", "1020.05"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661547321110606*^9, 3.661547323750737*^9}, 
   3.674838523550548*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Compilation", "Subsubsection",
 CellChangeTimes->{{3.661545607032065*^9, 3.6615456089440393`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C0comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C0funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C1funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C2funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C00funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C11funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C12funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22comp", "=", 
    RowBox[{"Compile", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"q", ",", "_Real"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", "_Real"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"y", ",", "_Real"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
      RowBox[{"Evaluate", "[", 
       RowBox[{"C22funcAcc", "[", 
        RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C0func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C1func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C2func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C00func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C11func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12comp0", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C12func0Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"C22comp0", "=", 
    RowBox[{"Compile", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "_Real"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"y", ",", "_Real"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
      RowBox[{"Evaluate", "[", 
       RowBox[{"C22func0Acc", "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C0comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C0func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C1func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C2func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C00comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C00func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C11comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C11func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C12comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C12func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C22comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"C22func1Acc", "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6615459006624813`*^9, 3.661545910271014*^9}, 
   3.661546091720519*^9, {3.661547344239437*^9, 3.661547364576479*^9}, {
   3.661706381156064*^9, 3.661706408703272*^9}, {3.661810126039771*^9, 
   3.661810132031966*^9}, {3.6748387910416193`*^9, 3.6748388754542217`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C1comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C2comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C00comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C11comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C12comp", "[", 
   RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"AbsoluteTiming", "[", 
   RowBox[{"C22comp", "[", 
    RowBox[{"0.", ",", "12309.", ",", "1203.110", ",", "13290."}], "]"}], 
   "]"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C1comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C2comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C00comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C11comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C12comp0", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"AbsoluteTiming", "[", 
   RowBox[{"C22comp0", "[", 
    RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C0comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C1comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C2comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C00comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C11comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C12comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"C22comp1", "[", 
   RowBox[{"12309.", ",", "1203.110", ",", "13290."}], "]"}], "]"}]}], "Input",\

 CellChangeTimes->{{3.661706414598501*^9, 3.6617065074289103`*^9}, {
  3.661706589307081*^9, 3.661706590574889*^9}, {3.66170720212015*^9, 
  3.661707225808731*^9}, {3.6748401267738132`*^9, 3.674840159153184*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0000250000000000000011980434005964824`1.4185399219516617", ",", 
   RowBox[{"-", "0.00006510490520376496`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800618687`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00002000000000000000163606107828062619`1.321629908943605", ",", 
   "0.000028724103628096133`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800631523`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001699999999999999986599261148079165`1.2510488346578978", ",", 
   "0.000017971923638203245`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180064603*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001699999999999999986599261148079165`1.2510488346578978", ",", 
   RowBox[{"-", "1.7703590747469393`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180065566*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001900000000000000104603825601401468`1.2993535142324528", ",", 
   RowBox[{"-", "0.000017326407024950084`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180066688*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001900000000000000104603825601401468`1.2993535142324528", ",", 
   RowBox[{"-", "5.633889523706949`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180069305*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.00001800000000000000045601543374740316`1.27587241838293", ",", 
   RowBox[{"-", "8.174939782384516`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180071437*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0000100000000000000008180305391403131`1.0205999132796237", ",", 
   RowBox[{"-", "0.00006510490520376496`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800725193`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001200000000000000030401028916493544`1.0997811593272493", ",", 
   "0.000028724103628096133`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180073497*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001200000000000000030401028916493544`1.0997811593272493", ",", 
   "0.000017971923638203245`"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180074656*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001200000000000000030401028916493544`1.0997811593272493", ",", 
   RowBox[{"-", "1.7703590747469393`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.67484018007598*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001399999999999999978999003918955779`1.1667279489578624", ",", 
   RowBox[{"-", "0.000017326407024950084`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180077035*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001299999999999999919996721692294628`1.1345432655864605", ",", 
   RowBox[{"-", "5.633889523706949`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800779037`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001299999999999999919996721692294628`1.1345432655864605", ",", 
   RowBox[{"-", "8.174939782384516`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800787973`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001200000000000000030401028916493544`1.0997811593272493", ",", 
   "8.895737439395885`*^-10"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.6748401800796747`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001299999999999999919996721692294628`1.1345432655864605", ",", 
   RowBox[{"-", "5.012561525359515`*^-10"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180080552*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001299999999999999919996721692294628`1.1345432655864605", ",", 
   RowBox[{"-", "2.558837935155852`*^-10"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180081896*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001200000000000000030401028916493544`1.0997811593272493", ",", 
   RowBox[{"-", "2.8169447618534744`*^-6"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180083709*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001699999999999999986599261148079165`1.2510488346578978", ",", 
   "3.257113866159637`*^-10"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180084988*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0000150000000000000003800128614561693`1.1966911723353055", ",", 
   "1.0516906688195232`*^-21"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180086349*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00001399999999999999978999003918955779`1.1667279489578624", ",", 
   "1.043637790911911`*^-10"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706582496998*^9, 3.661706592185377*^9}, 
   3.66170722653815*^9, {3.661810246353076*^9, 3.661810250046763*^9}, {
   3.674840170866036*^9, 3.674840180087508*^9}}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C0FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C0funcbase", ",", "C0funcz12", ",", "C0funcz23", ",", "C0funcz13", ",", 
     "C0funcz1d", ",", "C0funcz2d", ",", "C0funcz3d", ",", "C0funcz1", ",", 
     "C0funcz2", ",", "C0funcz3", ",", "C0funcd123", ",", "C0funcd13", ",", 
     "C0funcd12", ",", "C0funcd23", ",", "C0func0Acc", ",", "C0func1Acc"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C1FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C1funcbase", ",", "C1funcz12", ",", "C1funcz23", ",", "C1funcz13", ",", 
     "C1funcz1d", ",", "C1funcz2d", ",", "C1funcz3d", ",", "C1funcz1", ",", 
     "C1funcz2", ",", "C1funcz3", ",", "C1funcd123", ",", "C1funcd13", ",", 
     "C1funcd12", ",", "C1funcd23", ",", "C1func0Acc", ",", "C1func1Acc"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C2FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C2funcbase", ",", "C2funcz12", ",", "C2funcz23", ",", "C2funcz13", ",", 
     "C2funcz1d", ",", "C2funcz2d", ",", "C2funcz3d", ",", "C2funcz1", ",", 
     "C2funcz2", ",", "C2funcz3", ",", "C2funcd123", ",", "C2funcd13", ",", 
     "C2funcd12", ",", "C2funcd23", ",", "C2func0Acc", ",", "C2func1Acc"}], 
    "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"C00FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C00funcbase", ",", "C00funcz12", ",", "C00funcz23", ",", "C00funcz13", 
     ",", "C00funcz1d", ",", "C00funcz2d", ",", "C00funcz3d", ",", 
     "C00funcz1", ",", "C00funcz2", ",", "C00funcz3", ",", "C00funcd123", ",",
      "C00funcd13", ",", "C00funcd12", ",", "C00funcd23", ",", "C00func0Acc", 
     ",", "C00func1Acc"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"C11FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C11funcbase", ",", "C11funcz12", ",", "C11funcz23", ",", "C11funcz13", 
     ",", "C11funcz1d", ",", "C11funcz2d", ",", "C11funcz3d", ",", 
     "C11funcz1", ",", "C11funcz2", ",", "C11funcz3", ",", "C11funcd123", ",",
      "C11funcd13", ",", "C11funcd12", ",", "C11funcd23", ",", "C11func0Acc", 
     ",", "C11func1Acc"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"C12FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C12funcbase", ",", "C12funcz12", ",", "C12funcz23", ",", "C12funcz13", 
     ",", "C12funcz1d", ",", "C12funcz2d", ",", "C12funcz3d", ",", 
     "C12funcz1", ",", "C12funcz2", ",", "C12funcz3", ",", "C12funcd123", ",",
      "C12funcd13", ",", "C12funcd12", ",", "C12funcd23", ",", "C12func0Acc", 
     ",", "C12func1Acc"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"C22FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "C22funcbase", ",", "C22funcz12", ",", "C22funcz23", ",", "C22funcz13", 
     ",", "C22funcz1d", ",", "C22funcz2d", ",", "C22funcz3d", ",", 
     "C22funcz1", ",", "C22funcz2", ",", "C22funcz3", ",", "C22funcd123", ",",
      "C22funcd13", ",", "C22funcd12", ",", "C22funcd23", ",", "C22func0Acc", 
     ",", "C22func1Acc"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CFunctionList", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
    "C0FunctionList", ",", "C1FunctionList", ",", "C2FunctionList", ",", 
     "C00FunctionList", ",", "C11FunctionList", ",", "C12FunctionList", ",", 
     "C22FunctionList"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.661611982926502*^9, 3.661611989730653*^9}, {
  3.661612040106251*^9, 3.6616121212348833`*^9}, {3.6616121975957747`*^9, 
  3.661612197767552*^9}, {3.661612231119443*^9, 3.661612294743023*^9}, {
  3.661615214054413*^9, 3.6616152315944023`*^9}, {3.661810135841127*^9, 
  3.6618101453448677`*^9}, {3.67483853456426*^9, 3.674838543342618*^9}, {
  3.674839929931638*^9, 3.674839934471311*^9}, {3.674839970124661*^9, 
  3.674840022362699*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Need 4-pt integrals D0, D1, D2, D3, D00 for p1 = p2 = 0 and general m1, m2, m3\
\>", "Subsection",
 CellChangeTimes->{{3.574023579872418*^9, 3.574023598464611*^9}, {
  3.5741818782741623`*^9, 3.5741819009536943`*^9}}],

Cell[CellGroupData[{

Cell["Definitons", "Subsubsection",
 CellChangeTimes->{{3.661612033246275*^9, 3.6616120345569077`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"integrandD0temp", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m2"}], ")"}]}], "/", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "+", "m3"}], ")"}]}], "/", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "+", "m4"}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5754850105398417`*^9, 3.57548502523773*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0temp", "=", 
  RowBox[{
   RowBox[{"dd", " ", 
    RowBox[{"Intfuncgen", "[", "integrandD0temp", "]"}]}], "/.", 
   "subtry"}]}]], "Input",
 CellChangeTimes->{{3.575485028417182*^9, 3.5754850538713303`*^9}}],

Cell[BoxData[
 RowBox[{"16", " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"m1", " ", 
       RowBox[{"Log", "[", "m1", "]"}]}], 
      RowBox[{"16", " ", 
       RowBox[{"(", 
        RowBox[{"m1", "-", "m2"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"m1", "-", "m3"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"m1", "-", "m4"}], ")"}], " ", 
       SuperscriptBox["\[Pi]", "2"]}]]}], "+", 
    FractionBox[
     RowBox[{"m2", " ", 
      RowBox[{"Log", "[", "m2", "]"}]}], 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{"m3", " ", 
      RowBox[{"Log", "[", "m3", "]"}]}], 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m3", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{"m4", " ", 
      RowBox[{"Log", "[", "m4", "]"}]}], 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m3"}], "+", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.575485034711784*^9, 3.575485054103401*^9}, 
   3.575485413817238*^9, 3.575491226221959*^9, 3.5755038264652767`*^9, 
   3.575503991148119*^9, 3.5757412280422297`*^9, 3.57574293636331*^9, 
   3.5758185795984793`*^9, 3.575927188956048*^9, 3.575928317938182*^9, 
   3.575930609213266*^9, 3.575931355500208*^9, 3.575932846093258*^9, 
   3.575933132131032*^9, 3.575933719312051*^9, 3.5759350001622257`*^9, 
   3.575935285181583*^9, 3.575935918975185*^9, 3.575938402292893*^9, 
   3.575938666295182*^9, 3.575939457149445*^9, 3.575940313512594*^9, 
   3.5759406495429783`*^9, 3.575943051259469*^9, 3.5759445614308987`*^9, 
   3.575945981030498*^9, 3.57600775083683*^9, 3.576008664870023*^9, 
   3.576015108933619*^9, 3.576097353931408*^9, 3.576100016121139*^9, 
   3.5761924799172792`*^9, 3.576195043952577*^9, 3.576198822475027*^9, 
   3.576246036883634*^9, 3.576251981230803*^9, 3.57625339363242*^9, 
   3.576359058749271*^9, 3.576363246196858*^9, 3.576364141914379*^9, 
   3.576364268229197*^9, 3.576364349533031*^9, 3.57636448635546*^9, 
   3.576372588724536*^9, 3.576429174572885*^9, 3.576536750779069*^9, 
   3.5765395187679253`*^9, 3.5765398537493343`*^9, 3.5766904744846497`*^9, 
   3.576692940037777*^9, 3.57671217927271*^9, 3.576795773185478*^9, 
   3.576983766508754*^9, 3.576983846356717*^9, 3.577215487831141*^9, 
   3.577278700046921*^9, 3.5772832938042593`*^9, 3.57729668975837*^9, 
   3.577383153476657*^9, 3.577401323262219*^9, 3.577663086150381*^9, 
   3.5776750171936607`*^9, 3.5776754857005997`*^9, 3.577677099079544*^9, 
   3.577816977530044*^9, 3.577834619522955*^9, 3.577915988768738*^9, 
   3.5779284737242813`*^9, 3.577931075802031*^9, 3.577934535352497*^9, 
   3.5783303155097*^9, 3.578342630224782*^9, 3.57834454155907*^9, 
   3.578494219655011*^9, 3.578756841682351*^9, 3.578849976026948*^9, 
   3.578850492428749*^9, 3.578867503095461*^9, 3.578953631562373*^9, 
   3.579005060543173*^9, 3.5790059160877733`*^9, 3.579006078504531*^9, 
   3.579006464886964*^9, 3.5790072377267523`*^9, 3.579008086840527*^9, 
   3.579016660171298*^9, 3.5790166995312243`*^9, 3.579023951656969*^9, 
   3.5791951657534*^9, 3.579216927595499*^9, 3.579282533987309*^9, 
   3.57928398606114*^9, 3.579303930170581*^9, 3.579452413951302*^9, 
   3.579456378931066*^9, 3.5798209101685457`*^9, 3.579824038956008*^9, 
   3.5800088424829082`*^9, 3.5804391932110023`*^9, 3.5805960869866858`*^9, 
   3.580642994299836*^9, 3.580671616622801*^9, 3.580699221028075*^9, 
   3.580845963692535*^9, 3.5811721100498323`*^9, 3.581190390407308*^9, 
   3.581195643541976*^9, 3.5812690004101667`*^9, 3.5812709542716837`*^9, 
   3.581276331241444*^9, 3.5812918359994*^9, 3.581295539263784*^9, 
   3.581297675258947*^9, 3.581298274495956*^9, 3.5812988739779577`*^9, 
   3.5813466674650583`*^9, 3.5813635629899187`*^9, 3.58136922300775*^9, 
   3.581369613268178*^9, 3.581442035208076*^9, 3.58145083787671*^9, 
   3.581757960496751*^9, 3.58191543666152*^9, 3.582021782036036*^9, 
   3.58248350331444*^9, 3.584065547214787*^9, 3.584109877414509*^9, 
   3.5841101074267282`*^9, 3.5842087312262297`*^9, 3.584208939337288*^9, 
   3.584211315626597*^9, 3.5842136579906263`*^9, 3.584214204710218*^9, 
   3.584221464244378*^9, 3.584224139451276*^9, 3.5842798134860983`*^9, 
   3.5843171251474867`*^9, 3.5843622878564997`*^9, 3.58438340364123*^9, 
   3.584448857625448*^9, 3.584453355578541*^9, 3.584453438050743*^9, 
   3.58445364061477*^9, 3.584453879684752*^9, 3.584479617309289*^9, 
   3.5844852265097513`*^9, 3.584708540963167*^9, 3.584720427223782*^9, 
   3.584725561668372*^9, 3.5848747996849422`*^9, 3.584876052486671*^9, 
   3.5848795965281477`*^9, 3.603755757809717*^9, 3.603757501272311*^9, 
   3.603849920957456*^9, 3.603850247450832*^9, 3.603853994029736*^9, 
   3.603925620320712*^9, 3.6041576383738937`*^9, 3.604236439642579*^9, 
   3.605050497170291*^9, 3.6054522806100082`*^9, 3.605453057614736*^9, 
   3.612794860080533*^9, 3.6127949835700407`*^9, 3.612798726872349*^9, 
   3.612798817558694*^9, 3.612800504954626*^9, 3.6133199373820257`*^9, 
   3.613320343767077*^9, 3.613578477691184*^9, 3.6148130481950607`*^9, 
   3.661547400619729*^9, 3.6616123410966787`*^9, 3.6616159361634903`*^9, 
   3.66163814520599*^9, 3.661706608623961*^9, 3.6618102606058493`*^9, 
   3.6748402137227497`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"integrandD1temp", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"2", "/", "d"}], " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["k", "2"], "/", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m2"}], ")"}], "2"]}], "/", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["k", "2"], "+", "m3"}], ")"}]}], "/", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m4"}], ")"}]}]}], "/.", 
    RowBox[{"d", "\[Rule]", 
     RowBox[{"4", "-", "\[Delta]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"integrandD2temp", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"2", "/", "d"}], " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["k", "2"], "/", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m2"}], ")"}]}], "/", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m3"}], ")"}], "2"]}], "/", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "+", "m4"}], ")"}]}]}], "/.", 
    RowBox[{"d", "\[Rule]", 
     RowBox[{"4", "-", "\[Delta]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"integrandD3temp", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"2", "/", "d"}], " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["k", "2"], "/", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["k", "2"], "+", "m2"}], ")"}]}], "/", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["k", "2"], "+", "m3"}], ")"}]}], "/", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["k", "2"], "+", "m4"}], ")"}], "2"]}]}], "/.", 
    RowBox[{"d", "\[Rule]", 
     RowBox[{"4", "-", "\[Delta]"}]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.575485068835237*^9, 3.575485120738412*^9}, {
  3.57767519894655*^9, 3.577675227490499*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"D1temp", "=", 
  RowBox[{
   RowBox[{"-", "dd"}], " ", 
   RowBox[{
   "Intfuncgen", "[", "integrandD1temp", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"D2temp", "=", 
  RowBox[{
   RowBox[{"-", "dd"}], " ", 
   RowBox[{
   "Intfuncgen", "[", "integrandD2temp", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"D3temp", "=", 
  RowBox[{
   RowBox[{"-", "dd"}], " ", 
   RowBox[{
   "Intfuncgen", "[", "integrandD3temp", 
    "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.575485125934366*^9, 3.575485151418535*^9}, {
  3.575485237243656*^9, 3.575485237698687*^9}, {3.613319922975912*^9, 
  3.6133199321501923`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["m2", 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", "m1", "]"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{"m2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["m2", "3"], "+", 
        RowBox[{"2", " ", "m1", " ", "m3", " ", "m4"}], "-", 
        RowBox[{"m2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m3", " ", "m4"}], "+", 
           RowBox[{"m1", " ", 
            RowBox[{"(", 
             RowBox[{"m3", "+", "m4"}], ")"}]}]}], ")"}]}]}], ")"}], " ", 
      RowBox[{"Log", "[", "m2", "]"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m2"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m3", "2"], " ", 
      RowBox[{"Log", "[", "m3", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"m3", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m4", "2"], " ", 
      RowBox[{"Log", "[", "m4", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m4"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m4"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m3"}], "+", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5754851334620256`*^9, 3.575485151760275*^9}, 
   3.575485237957575*^9, 3.575485414026136*^9, 3.57549122643036*^9, 
   3.575503826674211*^9, 3.5755039913916807`*^9, 3.575741228272266*^9, 
   3.5757429366240873`*^9, 3.575818579841455*^9, 3.5759271892066174`*^9, 
   3.575928318178528*^9, 3.575930609470294*^9, 3.575931355756001*^9, 
   3.575932846326412*^9, 3.5759331323645267`*^9, 3.575933719543247*^9, 
   3.5759350003859043`*^9, 3.575935285404949*^9, 3.575935919241797*^9, 
   3.575938402541052*^9, 3.5759386665746393`*^9, 3.575939457380842*^9, 
   3.575940313757991*^9, 3.575940649796879*^9, 3.575943051479282*^9, 
   3.575944561671377*^9, 3.575945981249181*^9, 3.5760077510567427`*^9, 
   3.576008665128105*^9, 3.5760151091671677`*^9, 3.576097354184353*^9, 
   3.576100016360208*^9, 3.576192480169691*^9, 3.576195044179431*^9, 
   3.576198822688588*^9, 3.57624603709592*^9, 3.576251981481688*^9, 
   3.576253393856606*^9, 3.57635905896844*^9, 3.576363246421652*^9, 
   3.576364142173952*^9, 3.5763642684832907`*^9, 3.57636434977099*^9, 
   3.576364486614077*^9, 3.576372588940572*^9, 3.576429174808331*^9, 
   3.576536750989402*^9, 3.576539519005745*^9, 3.576539853987096*^9, 
   3.576690474695641*^9, 3.5766929402489777`*^9, 3.576712179506568*^9, 
   3.576795773388871*^9, 3.576983766737238*^9, 3.576983846581143*^9, 
   3.577215488038548*^9, 3.577278700231367*^9, 3.577283293994624*^9, 
   3.577296690005501*^9, 3.577383153662348*^9, 3.5774013234455843`*^9, 
   3.5776630874597063`*^9, {3.5776752132862997`*^9, 3.577675230222495*^9}, 
   3.5776754870079813`*^9, 3.577677100256071*^9, 3.577816978903994*^9, 
   3.577834620689064*^9, 3.577915989965722*^9, 3.577928474887126*^9, 
   3.577931077221633*^9, 3.577934537207737*^9, 3.5783303171343727`*^9, 
   3.5783426315979147`*^9, 3.578344542892704*^9, 3.5784942210094624`*^9, 
   3.578756843396254*^9, 3.5788499774245577`*^9, 3.578850494071684*^9, 
   3.578867504434476*^9, 3.5789536333990593`*^9, 3.579005061907187*^9, 
   3.5790059176168413`*^9, 3.579006079945463*^9, 3.579006466009828*^9, 
   3.579007237838382*^9, 3.579016664137938*^9, 3.579016701122136*^9, 
   3.57902395369766*^9, 3.5791951680588083`*^9, 3.5792169296329527`*^9, 
   3.5792825360904493`*^9, 3.579283988125764*^9, 3.579303933071027*^9, 
   3.579452416065567*^9, 3.579456381145067*^9, 3.579820912322165*^9, 
   3.579824041281846*^9, 3.580008844640867*^9, 3.5804391952511253`*^9, 
   3.580596089430189*^9, 3.5806429970184793`*^9, 3.580671618809381*^9, 
   3.580699223314061*^9, 3.5808459659360743`*^9, 3.581172112488388*^9, 
   3.58119039288946*^9, 3.581195645808908*^9, 3.581269002564117*^9, 
   3.58127095647579*^9, 3.5812763333248167`*^9, 3.581291838174244*^9, 
   3.581295541347492*^9, 3.5812976773007727`*^9, 3.5812982765610943`*^9, 
   3.581298876112742*^9, 3.5813466702890263`*^9, 3.581363566406095*^9, 
   3.581369226054878*^9, 3.581369616299371*^9, 3.581442037638076*^9, 
   3.581450840802713*^9, 3.58175796424507*^9, 3.581915440103265*^9, 
   3.58202178551322*^9, 3.582483507074375*^9, 3.5840655510092487`*^9, 
   3.584109881234305*^9, 3.584110111211309*^9, 3.584208735061964*^9, 
   3.584208943113117*^9, 3.58421131939331*^9, 3.584213661793914*^9, 
   3.584214208478002*^9, 3.584221468014371*^9, 3.584224143253491*^9, 
   3.58427981732124*^9, 3.58431712902717*^9, 3.584362291670815*^9, 
   3.584383407473838*^9, 3.584448861635951*^9, 3.584453359760448*^9, 
   3.584453442218988*^9, 3.584453644462735*^9, 3.5844538834895287`*^9, 
   3.5844796211826572`*^9, 3.584485230344225*^9, 3.584708544785718*^9, 
   3.584720431213213*^9, 3.58472556556429*^9, 3.584874803475388*^9, 
   3.584876056283066*^9, 3.5848796003765078`*^9, 3.603755761310216*^9, 
   3.603757505147403*^9, 3.603849924377825*^9, 3.603850251026742*^9, 
   3.6038539976341867`*^9, 3.603925624330783*^9, 3.6041576465800734`*^9, 
   3.604236443510688*^9, 3.605050501225808*^9, 3.6054522846474857`*^9, 
   3.605453061611868*^9, 3.612794862364303*^9, 3.612794987122232*^9, 
   3.612798737575655*^9, 3.6127988193184147`*^9, 3.612800506833547*^9, {
   3.613319933501606*^9, 3.613319940021532*^9}, 3.61332034556533*^9, 
   3.613578479515424*^9, 3.6148130515961847`*^9, 3.661547407345935*^9, 
   3.661612343263875*^9, 3.6616159382872887`*^9, 3.661638147383082*^9, 
   3.661706610739725*^9, 3.6618102627993393`*^9, 3.674840215757444*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["m3", 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m3", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", "m1", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m2", "2"], " ", 
      RowBox[{"Log", "[", "m2", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{"m3", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["m3", "3"]}], "+", 
        RowBox[{"m2", " ", "m3", " ", "m4"}], "+", 
        RowBox[{"m1", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m2", " ", 
            RowBox[{"(", 
             RowBox[{"m3", "-", 
              RowBox[{"2", " ", "m4"}]}], ")"}]}], "+", 
           RowBox[{"m3", " ", "m4"}]}], ")"}]}]}], ")"}], " ", 
      RowBox[{"Log", "[", "m3", "]"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m3"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m3"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m3", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m4", "2"], " ", 
      RowBox[{"Log", "[", "m4", "]"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m3", "-", "m4"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m1"}], "+", "m4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5754851334620256`*^9, 3.575485151760275*^9}, 
   3.575485237957575*^9, 3.575485414026136*^9, 3.57549122643036*^9, 
   3.575503826674211*^9, 3.5755039913916807`*^9, 3.575741228272266*^9, 
   3.5757429366240873`*^9, 3.575818579841455*^9, 3.5759271892066174`*^9, 
   3.575928318178528*^9, 3.575930609470294*^9, 3.575931355756001*^9, 
   3.575932846326412*^9, 3.5759331323645267`*^9, 3.575933719543247*^9, 
   3.5759350003859043`*^9, 3.575935285404949*^9, 3.575935919241797*^9, 
   3.575938402541052*^9, 3.5759386665746393`*^9, 3.575939457380842*^9, 
   3.575940313757991*^9, 3.575940649796879*^9, 3.575943051479282*^9, 
   3.575944561671377*^9, 3.575945981249181*^9, 3.5760077510567427`*^9, 
   3.576008665128105*^9, 3.5760151091671677`*^9, 3.576097354184353*^9, 
   3.576100016360208*^9, 3.576192480169691*^9, 3.576195044179431*^9, 
   3.576198822688588*^9, 3.57624603709592*^9, 3.576251981481688*^9, 
   3.576253393856606*^9, 3.57635905896844*^9, 3.576363246421652*^9, 
   3.576364142173952*^9, 3.5763642684832907`*^9, 3.57636434977099*^9, 
   3.576364486614077*^9, 3.576372588940572*^9, 3.576429174808331*^9, 
   3.576536750989402*^9, 3.576539519005745*^9, 3.576539853987096*^9, 
   3.576690474695641*^9, 3.5766929402489777`*^9, 3.576712179506568*^9, 
   3.576795773388871*^9, 3.576983766737238*^9, 3.576983846581143*^9, 
   3.577215488038548*^9, 3.577278700231367*^9, 3.577283293994624*^9, 
   3.577296690005501*^9, 3.577383153662348*^9, 3.5774013234455843`*^9, 
   3.5776630874597063`*^9, {3.5776752132862997`*^9, 3.577675230222495*^9}, 
   3.5776754870079813`*^9, 3.577677100256071*^9, 3.577816978903994*^9, 
   3.577834620689064*^9, 3.577915989965722*^9, 3.577928474887126*^9, 
   3.577931077221633*^9, 3.577934537207737*^9, 3.5783303171343727`*^9, 
   3.5783426315979147`*^9, 3.578344542892704*^9, 3.5784942210094624`*^9, 
   3.578756843396254*^9, 3.5788499774245577`*^9, 3.578850494071684*^9, 
   3.578867504434476*^9, 3.5789536333990593`*^9, 3.579005061907187*^9, 
   3.5790059176168413`*^9, 3.579006079945463*^9, 3.579006466009828*^9, 
   3.579007237838382*^9, 3.579016664137938*^9, 3.579016701122136*^9, 
   3.57902395369766*^9, 3.5791951680588083`*^9, 3.5792169296329527`*^9, 
   3.5792825360904493`*^9, 3.579283988125764*^9, 3.579303933071027*^9, 
   3.579452416065567*^9, 3.579456381145067*^9, 3.579820912322165*^9, 
   3.579824041281846*^9, 3.580008844640867*^9, 3.5804391952511253`*^9, 
   3.580596089430189*^9, 3.5806429970184793`*^9, 3.580671618809381*^9, 
   3.580699223314061*^9, 3.5808459659360743`*^9, 3.581172112488388*^9, 
   3.58119039288946*^9, 3.581195645808908*^9, 3.581269002564117*^9, 
   3.58127095647579*^9, 3.5812763333248167`*^9, 3.581291838174244*^9, 
   3.581295541347492*^9, 3.5812976773007727`*^9, 3.5812982765610943`*^9, 
   3.581298876112742*^9, 3.5813466702890263`*^9, 3.581363566406095*^9, 
   3.581369226054878*^9, 3.581369616299371*^9, 3.581442037638076*^9, 
   3.581450840802713*^9, 3.58175796424507*^9, 3.581915440103265*^9, 
   3.58202178551322*^9, 3.582483507074375*^9, 3.5840655510092487`*^9, 
   3.584109881234305*^9, 3.584110111211309*^9, 3.584208735061964*^9, 
   3.584208943113117*^9, 3.58421131939331*^9, 3.584213661793914*^9, 
   3.584214208478002*^9, 3.584221468014371*^9, 3.584224143253491*^9, 
   3.58427981732124*^9, 3.58431712902717*^9, 3.584362291670815*^9, 
   3.584383407473838*^9, 3.584448861635951*^9, 3.584453359760448*^9, 
   3.584453442218988*^9, 3.584453644462735*^9, 3.5844538834895287`*^9, 
   3.5844796211826572`*^9, 3.584485230344225*^9, 3.584708544785718*^9, 
   3.584720431213213*^9, 3.58472556556429*^9, 3.584874803475388*^9, 
   3.584876056283066*^9, 3.5848796003765078`*^9, 3.603755761310216*^9, 
   3.603757505147403*^9, 3.603849924377825*^9, 3.603850251026742*^9, 
   3.6038539976341867`*^9, 3.603925624330783*^9, 3.6041576465800734`*^9, 
   3.604236443510688*^9, 3.605050501225808*^9, 3.6054522846474857`*^9, 
   3.605453061611868*^9, 3.612794862364303*^9, 3.612794987122232*^9, 
   3.612798737575655*^9, 3.6127988193184147`*^9, 3.612800506833547*^9, {
   3.613319933501606*^9, 3.613319940021532*^9}, 3.61332034556533*^9, 
   3.613578479515424*^9, 3.6148130515961847`*^9, 3.661547407345935*^9, 
   3.661612343263875*^9, 3.6616159382872887`*^9, 3.661638147383082*^9, 
   3.661706610739725*^9, 3.6618102627993393`*^9, 3.67484021835826*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["m4", 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m3"}], "+", "m4"}], ")"}], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m1", "2"], " ", 
      RowBox[{"Log", "[", "m1", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m2", "2"], " ", 
      RowBox[{"Log", "[", "m2", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m2"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"m2", "-", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["m3", "2"], " ", 
      RowBox[{"Log", "[", "m3", "]"}]}], 
     RowBox[{"32", " ", 
      RowBox[{"(", 
       RowBox[{"m1", "-", "m3"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "m2"}], "+", "m3"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m3", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]], "-", 
    FractionBox[
     RowBox[{"m4", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"m2", " ", "m3", " ", "m4"}], "-", 
        SuperscriptBox["m4", "3"], "+", 
        RowBox[{"m1", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m3", " ", "m4"}], "+", 
           RowBox[{"m2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"-", "2"}], " ", "m3"}], "+", "m4"}], ")"}]}]}], 
          ")"}]}]}], ")"}], " ", 
      RowBox[{"Log", "[", "m4", "]"}]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m1", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m2", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m3", "-", "m4"}], ")"}], "2"], " ", 
      SuperscriptBox["\[Pi]", "2"]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5754851334620256`*^9, 3.575485151760275*^9}, 
   3.575485237957575*^9, 3.575485414026136*^9, 3.57549122643036*^9, 
   3.575503826674211*^9, 3.5755039913916807`*^9, 3.575741228272266*^9, 
   3.5757429366240873`*^9, 3.575818579841455*^9, 3.5759271892066174`*^9, 
   3.575928318178528*^9, 3.575930609470294*^9, 3.575931355756001*^9, 
   3.575932846326412*^9, 3.5759331323645267`*^9, 3.575933719543247*^9, 
   3.5759350003859043`*^9, 3.575935285404949*^9, 3.575935919241797*^9, 
   3.575938402541052*^9, 3.5759386665746393`*^9, 3.575939457380842*^9, 
   3.575940313757991*^9, 3.575940649796879*^9, 3.575943051479282*^9, 
   3.575944561671377*^9, 3.575945981249181*^9, 3.5760077510567427`*^9, 
   3.576008665128105*^9, 3.5760151091671677`*^9, 3.576097354184353*^9, 
   3.576100016360208*^9, 3.576192480169691*^9, 3.576195044179431*^9, 
   3.576198822688588*^9, 3.57624603709592*^9, 3.576251981481688*^9, 
   3.576253393856606*^9, 3.57635905896844*^9, 3.576363246421652*^9, 
   3.576364142173952*^9, 3.5763642684832907`*^9, 3.57636434977099*^9, 
   3.576364486614077*^9, 3.576372588940572*^9, 3.576429174808331*^9, 
   3.576536750989402*^9, 3.576539519005745*^9, 3.576539853987096*^9, 
   3.576690474695641*^9, 3.5766929402489777`*^9, 3.576712179506568*^9, 
   3.576795773388871*^9, 3.576983766737238*^9, 3.576983846581143*^9, 
   3.577215488038548*^9, 3.577278700231367*^9, 3.577283293994624*^9, 
   3.577296690005501*^9, 3.577383153662348*^9, 3.5774013234455843`*^9, 
   3.5776630874597063`*^9, {3.5776752132862997`*^9, 3.577675230222495*^9}, 
   3.5776754870079813`*^9, 3.577677100256071*^9, 3.577816978903994*^9, 
   3.577834620689064*^9, 3.577915989965722*^9, 3.577928474887126*^9, 
   3.577931077221633*^9, 3.577934537207737*^9, 3.5783303171343727`*^9, 
   3.5783426315979147`*^9, 3.578344542892704*^9, 3.5784942210094624`*^9, 
   3.578756843396254*^9, 3.5788499774245577`*^9, 3.578850494071684*^9, 
   3.578867504434476*^9, 3.5789536333990593`*^9, 3.579005061907187*^9, 
   3.5790059176168413`*^9, 3.579006079945463*^9, 3.579006466009828*^9, 
   3.579007237838382*^9, 3.579016664137938*^9, 3.579016701122136*^9, 
   3.57902395369766*^9, 3.5791951680588083`*^9, 3.5792169296329527`*^9, 
   3.5792825360904493`*^9, 3.579283988125764*^9, 3.579303933071027*^9, 
   3.579452416065567*^9, 3.579456381145067*^9, 3.579820912322165*^9, 
   3.579824041281846*^9, 3.580008844640867*^9, 3.5804391952511253`*^9, 
   3.580596089430189*^9, 3.5806429970184793`*^9, 3.580671618809381*^9, 
   3.580699223314061*^9, 3.5808459659360743`*^9, 3.581172112488388*^9, 
   3.58119039288946*^9, 3.581195645808908*^9, 3.581269002564117*^9, 
   3.58127095647579*^9, 3.5812763333248167`*^9, 3.581291838174244*^9, 
   3.581295541347492*^9, 3.5812976773007727`*^9, 3.5812982765610943`*^9, 
   3.581298876112742*^9, 3.5813466702890263`*^9, 3.581363566406095*^9, 
   3.581369226054878*^9, 3.581369616299371*^9, 3.581442037638076*^9, 
   3.581450840802713*^9, 3.58175796424507*^9, 3.581915440103265*^9, 
   3.58202178551322*^9, 3.582483507074375*^9, 3.5840655510092487`*^9, 
   3.584109881234305*^9, 3.584110111211309*^9, 3.584208735061964*^9, 
   3.584208943113117*^9, 3.58421131939331*^9, 3.584213661793914*^9, 
   3.584214208478002*^9, 3.584221468014371*^9, 3.584224143253491*^9, 
   3.58427981732124*^9, 3.58431712902717*^9, 3.584362291670815*^9, 
   3.584383407473838*^9, 3.584448861635951*^9, 3.584453359760448*^9, 
   3.584453442218988*^9, 3.584453644462735*^9, 3.5844538834895287`*^9, 
   3.5844796211826572`*^9, 3.584485230344225*^9, 3.584708544785718*^9, 
   3.584720431213213*^9, 3.58472556556429*^9, 3.584874803475388*^9, 
   3.584876056283066*^9, 3.5848796003765078`*^9, 3.603755761310216*^9, 
   3.603757505147403*^9, 3.603849924377825*^9, 3.603850251026742*^9, 
   3.6038539976341867`*^9, 3.603925624330783*^9, 3.6041576465800734`*^9, 
   3.604236443510688*^9, 3.605050501225808*^9, 3.6054522846474857`*^9, 
   3.605453061611868*^9, 3.612794862364303*^9, 3.612794987122232*^9, 
   3.612798737575655*^9, 3.6127988193184147`*^9, 3.612800506833547*^9, {
   3.613319933501606*^9, 3.613319940021532*^9}, 3.61332034556533*^9, 
   3.613578479515424*^9, 3.6148130515961847`*^9, 3.661547407345935*^9, 
   3.661612343263875*^9, 3.6616159382872887`*^9, 3.661638147383082*^9, 
   3.661706610739725*^9, 3.6618102627993393`*^9, 3.674840220880937*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Dij", " ", "with", " ", 
     SuperscriptBox["p1", "2"]}], "=", 
    RowBox[{
     SuperscriptBox["p2", "2"], "=", 
     RowBox[{
      SuperscriptBox["p3", "2"], "=", 
      RowBox[{
       RowBox[{
       "0", " ", "and", " ", "expanded", " ", "to", " ", "zeroth", " ", 
        "order", " ", "around", " ", "p1dotp2"}], "=", 
       RowBox[{"p1dotp3", "=", 
        RowBox[{"p2dotp3", "=", "0"}]}]}]}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Use", " ", 
     RowBox[{"identity", ":", " ", 
      RowBox[{"\[Integral]", " ", 
       RowBox[{
        SuperscriptBox["d", "4"], "k", " ", 
        SuperscriptBox["k", "\[Mu]"], 
        SuperscriptBox["k", "\[Nu]"], 
        SuperscriptBox["k", "\[Rho]"], 
        SuperscriptBox["k", "\[Sigma]"], 
        SuperscriptBox["k", "\[Alpha]"], 
        SuperscriptBox["k", "\[Beta]"], 
        RowBox[{"f", "[", 
         SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "192"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Eta]", "\[Mu]\[Nu]"], 
        SuperscriptBox["\[Eta]", "\[Rho]\[Sigma]"], 
        SuperscriptBox["\[Eta]", "\[Alpha]\[Beta]"]}], "+", "perms"}], ")"}], 
     
     RowBox[{"\[Integral]", " ", 
      RowBox[{
       SuperscriptBox["d", "4"], "k", " ", 
       SuperscriptBox["k", "6"], 
       RowBox[{"f", "[", 
        SuperscriptBox["k", "2"], "]"}]}]}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"integrandDijtemp", "=", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Normal", "[", 
                RowBox[{"Series", "[", 
                 RowBox[{
                  RowBox[{"k\[Mu]", " ", 
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"k\[Nu]", "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", "m1"}], ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp1"}], "+", "m2"}], 
                    ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp2"}], "+", "m3"}], 
                    ")"}]}], "/", 
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["k", "2"], "+", 
                    RowBox[{"2", "\[Epsilon]", " ", "kdotp3"}], "+", "m4"}], 
                    ")"}]}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"\[Epsilon]", ",", "0", ",", "2"}], "}"}]}], "]"}],
                 "]"}], "/.", 
               RowBox[{
                SuperscriptBox["\[Epsilon]", "k_"], "\[RuleDelayed]", 
                RowBox[{"1", "/;", 
                 RowBox[{"EvenQ", "[", "k", "]"}]}]}]}], "/.", 
              RowBox[{"\[Epsilon]", "\[Rule]", "0"}]}], "]"}], "/.", 
            RowBox[{
             RowBox[{
              SuperscriptBox["kdotp3", "2"], " ", "k\[Mu]", " ", "k\[Nu]"}], 
             "\[Rule]", 
             RowBox[{
              RowBox[{"1", "/", 
               RowBox[{"(", 
                RowBox[{"d", 
                 RowBox[{"(", 
                  RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
              RowBox[{"(", 
               RowBox[{"2", "p3\[Mu]", " ", "p3\[Nu]", " ", 
                SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
           RowBox[{
            RowBox[{
             SuperscriptBox["kdotp2", "2"], " ", "k\[Mu]", " ", "k\[Nu]"}], 
            "\[Rule]", 
            RowBox[{
             RowBox[{"1", "/", 
              RowBox[{"(", 
               RowBox[{"d", 
                RowBox[{"(", 
                 RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
             RowBox[{"(", 
              RowBox[{"2", "p2\[Mu]", " ", "p2\[Nu]", " ", 
               SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
          RowBox[{
           RowBox[{
            SuperscriptBox["kdotp1", "2"], " ", "k\[Mu]", " ", "k\[Nu]"}], 
           "\[Rule]", 
           RowBox[{
            RowBox[{"1", "/", 
             RowBox[{"(", 
              RowBox[{"d", 
               RowBox[{"(", 
                RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
            RowBox[{"(", 
             RowBox[{"2", "p1\[Mu]", " ", "p1\[Nu]", " ", 
              SuperscriptBox["k", "4"]}], ")"}]}]}]}], "/.", 
         RowBox[{
          RowBox[{"kdotp1", " ", "kdotp2", " ", "k\[Mu]", " ", "k\[Nu]"}], 
          "\[Rule]", 
          RowBox[{
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"d", 
              RowBox[{"(", 
               RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"p1\[Mu]", " ", "p2\[Nu]"}], "+", 
             RowBox[{"p2\[Mu]", " ", "p1\[Nu]"}]}], ")"}], 
           SuperscriptBox["k", "4"]}]}]}], "/.", 
        RowBox[{
         RowBox[{"kdotp1", " ", "kdotp3", " ", "k\[Mu]", " ", "k\[Nu]"}], 
         "\[Rule]", 
         RowBox[{
          RowBox[{"1", "/", 
           RowBox[{"(", 
            RowBox[{"d", 
             RowBox[{"(", 
              RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"p1\[Mu]", " ", "p3\[Nu]"}], "+", 
            RowBox[{"p3\[Mu]", " ", "p1\[Nu]"}]}], ")"}], 
          SuperscriptBox["k", "4"]}]}]}], "/.", 
       RowBox[{
        RowBox[{"kdotp2", " ", "kdotp3", " ", "k\[Mu]", " ", "k\[Nu]"}], 
        "\[Rule]", 
        RowBox[{
         RowBox[{"1", "/", 
          RowBox[{"(", 
           RowBox[{"d", 
            RowBox[{"(", 
             RowBox[{"d", "+", "2"}], ")"}]}], ")"}]}], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"p2\[Mu]", " ", "p3\[Nu]"}], "+", 
           RowBox[{"p3\[Mu]", " ", "p2\[Nu]"}]}], ")"}], 
         SuperscriptBox["k", "4"]}]}]}], "/.", 
      RowBox[{
       RowBox[{"k\[Mu]", " ", "k\[Nu]"}], "\[Rule]", 
       RowBox[{
        RowBox[{"1", "/", "d"}], " ", "\[Eta]\[Mu]\[Nu]", " ", 
        SuperscriptBox["k", "2"]}]}]}], "/.", 
     RowBox[{"d", "\[Rule]", 
      RowBox[{"4", "-", "\[Delta]"}]}]}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.574900226988413*^9, 3.574900264108863*^9}, {
   3.574900328087408*^9, 3.574900396959166*^9}, {3.574901224333816*^9, 
   3.574901243102832*^9}, {3.574901702649322*^9, 3.5749017754695377`*^9}, {
   3.574901849582118*^9, 3.5749019113833723`*^9}, {3.5749026103462667`*^9, 
   3.574902628406314*^9}, {3.574903114210644*^9, 3.5749032650500603`*^9}, {
   3.574903473994944*^9, 3.574903484149876*^9}, {3.5776702440754642`*^9, 
   3.577670273015163*^9}, {3.613312950631494*^9, 3.6133129529209347`*^9}, {
   3.6133129939484167`*^9, 3.613312998708416*^9}, {3.613313062400214*^9, 
   3.613313064736075*^9}, {3.613313362498254*^9, 3.6133133933291817`*^9}, 
   3.613313951707707*^9, {3.613314020655747*^9, 3.613314104077064*^9}, 
   3.613319946536448*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Dijtemp", "=", 
   RowBox[{"Intfuncgen", "[", "integrandDijtemp", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5754829785813828`*^9, 3.575482980173191*^9}, {
  3.5754847863059807`*^9, 3.5754847893206453`*^9}, {3.613314168216404*^9, 
  3.613314170560111*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"D00temp", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "dd"}], " ", 
     RowBox[{"D", "[", 
      RowBox[{"Dijtemp", ",", "\[Eta]\[Mu]\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D11temp", "=", 
   RowBox[{
    RowBox[{"dd", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Dijtemp", ",", "p1\[Mu]", ",", "p1\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D12temp", "=", 
   RowBox[{
    RowBox[{"dd", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Dijtemp", ",", "p1\[Mu]", ",", "p2\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D22temp", "=", 
   RowBox[{
    RowBox[{"dd", " ", 
     RowBox[{"D", "[", 
      RowBox[{"Dijtemp", ",", "p2\[Mu]", ",", "p2\[Nu]"}], "]"}]}], "/.", 
    "subtry"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.575482955815819*^9, 3.575482975374505*^9}, {
   3.575484938445628*^9, 3.575484976569477*^9}, {3.575498108963306*^9, 
   3.575498120837516*^9}, {3.57729663952792*^9, 3.577296651078128*^9}, {
   3.577675322044446*^9, 3.5776753315322533`*^9}, {3.6133141726630507`*^9, 
   3.61331417992109*^9}, {3.61331850765359*^9, 3.613318510997018*^9}, {
   3.6133190641145163`*^9, 3.613319067041044*^9}, 3.613319950684722*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculator", "Subsubsection",
 CellChangeTimes->{{3.66154753257652*^9, 3.661547535063819*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}], ",", 
    RowBox[{"m4", "\[Rule]", "w"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.661547541658764*^9, 3.661547560917994*^9}}],

Cell[BoxData[
 RowBox[{"16", " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"w", " ", 
      RowBox[{"Log", "[", "w", "]"}]}], 
     RowBox[{"16", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"w", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"w", "-", "z"}], ")"}]}]], "-", 
    FractionBox[
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", "x", "]"}]}], 
     RowBox[{"16", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{"y", " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"16", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"y", "-", "z"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{"z", " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"16", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.661547543291243*^9, 3.661547561710485*^9}, 
   3.661612432374745*^9, 3.661616025187312*^9, 3.661638234391014*^9, 
   3.66170669730126*^9, 3.6618103495808783`*^9, 3.674840300380377*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"D0func0", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"16", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"w", " ", 
       RowBox[{"Log", "[", "w", "]"}]}], 
      RowBox[{"16", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"w", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"w", "-", "z"}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{"x", " ", 
       RowBox[{"Log", "[", "x", "]"}]}], 
      RowBox[{"16", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"y", " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"16", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"z", " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"16", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.661547580865869*^9, 3.661547594968193*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FFF", "=", "D0func0"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Normal", "[", 
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"\[Epsilon]1", ",", "y", ",", 
           RowBox[{"y", "+", "\[Delta]1"}], ",", 
           RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"30", " ", 
     SuperscriptBox["y", "4"]}], "-", 
    RowBox[{"20", " ", 
     SuperscriptBox["y", "3"], " ", "\[Delta]1"}], "+", 
    RowBox[{"15", " ", 
     SuperscriptBox["y", "2"], " ", 
     SuperscriptBox["\[Delta]1", "2"]}], "-", 
    RowBox[{"12", " ", "y", " ", 
     SuperscriptBox["\[Delta]1", "3"]}], "+", 
    RowBox[{"10", " ", 
     SuperscriptBox["\[Delta]1", "4"]}]}], 
   RowBox[{"60", " ", 
    SuperscriptBox["y", "6"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "140"}], " ", 
       SuperscriptBox["y", "4"]}], "+", 
      RowBox[{"105", " ", 
       SuperscriptBox["y", "3"], " ", "\[Delta]1"}], "-", 
      RowBox[{"84", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["\[Delta]1", "2"]}], "+", 
      RowBox[{"70", " ", "y", " ", 
       SuperscriptBox["\[Delta]1", "3"]}], "-", 
      RowBox[{"60", " ", 
       SuperscriptBox["\[Delta]1", "4"]}]}], ")"}], " ", "\[Delta]2"}], 
   RowBox[{"420", " ", 
    SuperscriptBox["y", "7"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"210", " ", 
       SuperscriptBox["y", "4"]}], "-", 
      RowBox[{"168", " ", 
       SuperscriptBox["y", "3"], " ", "\[Delta]1"}], "+", 
      RowBox[{"140", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["\[Delta]1", "2"]}], "-", 
      RowBox[{"120", " ", "y", " ", 
       SuperscriptBox["\[Delta]1", "3"]}], "+", 
      RowBox[{"105", " ", 
       SuperscriptBox["\[Delta]1", "4"]}]}], ")"}], " ", 
    SuperscriptBox["\[Delta]2", "2"]}], 
   RowBox[{"840", " ", 
    SuperscriptBox["y", "8"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "504"}], " ", 
       SuperscriptBox["y", "4"]}], "+", 
      RowBox[{"420", " ", 
       SuperscriptBox["y", "3"], " ", "\[Delta]1"}], "-", 
      RowBox[{"360", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["\[Delta]1", "2"]}], "+", 
      RowBox[{"315", " ", "y", " ", 
       SuperscriptBox["\[Delta]1", "3"]}], "-", 
      RowBox[{"280", " ", 
       SuperscriptBox["\[Delta]1", "4"]}]}], ")"}], " ", 
    SuperscriptBox["\[Delta]2", "3"]}], 
   RowBox[{"2520", " ", 
    SuperscriptBox["y", "9"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"420", " ", 
       SuperscriptBox["y", "4"]}], "-", 
      RowBox[{"360", " ", 
       SuperscriptBox["y", "3"], " ", "\[Delta]1"}], "+", 
      RowBox[{"315", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["\[Delta]1", "2"]}], "-", 
      RowBox[{"280", " ", "y", " ", 
       SuperscriptBox["\[Delta]1", "3"]}], "+", 
      RowBox[{"252", " ", 
       SuperscriptBox["\[Delta]1", "4"]}]}], ")"}], " ", 
    SuperscriptBox["\[Delta]2", "4"]}], 
   RowBox[{"2520", " ", 
    SuperscriptBox["y", "10"]}]]}]], "Output",
 CellChangeTimes->{3.661548396346196*^9, 3.661612433322735*^9, 
  3.661616026044735*^9, 3.6616382356595297`*^9, 3.661706698259162*^9, 
  3.661810350445833*^9, 3.674840301244033*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "D0func0"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"2", ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"3", " ", "&"}], " ", "4", " ", "zeros"}], " ", "\[Rule]", " ", 
     "0"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "trip", " ", "degeneracy"}],
    " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz1d234", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"\[Epsilon]1", ",", "y", ",", 
             RowBox[{"y", "+", "\[Delta]1"}], ",", 
             RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz2d134", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"x", ",", "\[Epsilon]1", ",", 
             RowBox[{"x", "+", "\[Delta]1"}], ",", 
             RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz3d124", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"x", ",", 
             RowBox[{"x", "+", "\[Delta]1"}], ",", "\[Epsilon]1", ",", 
             RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcz4d123", "[", 
    RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"Limit", "[", 
           RowBox[{
            RowBox[{"FFF", "[", 
             RowBox[{"x", ",", 
              RowBox[{"x", "+", "\[Delta]1"}], ",", 
              RowBox[{"x", "+", "\[Delta]2"}], ",", "\[Epsilon]1"}], "]"}], 
            ",", 
            RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "a", " ", "degeneracy"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz1d34", "[", 
   RowBox[{"y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", 
          RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz1d24", "[", 
   RowBox[{"y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz1d23", "[", 
   RowBox[{"y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz2d34", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", 
          RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz2d14", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz2d13", "[", 
   RowBox[{"x_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz3d14", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz3d24", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz3d12", "[", 
   RowBox[{"x_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1", ",", "w"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz4d23", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
        
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz4d13", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
        
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcz4d12", "[", 
    RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "z", ",", "\[Epsilon]1"}], 
          "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz1", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz2", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcz3", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcz4", "[", 
    RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"quad", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcd1234", "[", 
    RowBox[{"x_", ",", "\[Delta]1_", ",", "\[Delta]2_", ",", "\[Delta]3_"}], 
    "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"Normal", "[", 
           RowBox[{"Series", "[", 
            RowBox[{
             RowBox[{"FFF", "[", 
              RowBox[{"x", ",", 
               RowBox[{"x", "+", "\[Delta]1"}], ",", 
               RowBox[{"x", "+", "\[Delta]2"}], ",", 
               RowBox[{"x", "+", "\[Delta]3"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]3", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd234", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"y", "+", "\[Delta]1"}], ",", 
           RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd134", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", 
           RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd124", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", "z", ",", 
           RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcd123", "[", 
    RowBox[{"x_", ",", "w_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}], ",", "w"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"double", " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd12d34", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", "z", ",", 
           RowBox[{"z", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd13d24", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", 
           RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D0funcd14d23", "[", 
    RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"x", ",", "y", ",", 
            RowBox[{"y", "+", "\[Delta]2"}], ",", 
            RowBox[{"x", "+", "\[Delta]1"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"single", " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd14", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd13", "[", 
   RowBox[{"x_", ",", "y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd12", "[", 
   RowBox[{"x_", ",", "z_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z", ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd24", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd23", "[", 
   RowBox[{"x_", ",", "y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0funcd34", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGCQAmIQzbWISe6G+2vHW8zLVEB0nqq4PYheIW/gCqI/HncM
B9Fhfy/GgOg9qjung2i/Q4fng+jAaOnjINr44umzIPpCkMZ9EB0kx/cURE+7
Y/cXROfKuoHplNJ3XDeBNM/i/Twgep/DBlEQvaTKQwJEl6aH2YFolgiHRBDt
uvprKoie8iZkIYguvlmwFKxu/s21INrg/arNIDpw74w3IHpbROUHEK0rtuIv
iM7s0We6BaTdi+/KgGi+ju8KIPpG+nJtED1v80MjEN04jTsWRG9f9jEZRKf2
XGsA0QIcV1pBdEPoFzCd4OffA6LPGD7aCqKXyEjuBtFSRslg2sJxH/9tIK3F
eU4YRLfolRuxegD9sYbRAkRvy/r2TgBIV0azfgHR5w4f2aAGpF9rmu8E0Q82
9u8F0WlbdoJpADU32As=
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"D0funcAcc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]14", "=", 
       RowBox[{"w", "-", "x"}]}], ",", 
      RowBox[{"S14", "=", 
       RowBox[{"x", "+", "w"}]}], ",", 
      RowBox[{"\[Epsilon]34", "=", 
       RowBox[{"w", "-", "z"}]}], ",", 
      RowBox[{"S34", "=", 
       RowBox[{"w", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]24", "=", 
       RowBox[{"w", "-", "y"}]}], ",", 
      RowBox[{"S24", "=", 
       RowBox[{"y", "+", "w"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d4comp", "=", "0.06"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
        "S12", "*", "S13", "*", "S14", "*", "S24", "*", "S23", "*", "S34"}], 
        "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz1d234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz2d134", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz3d124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz4d123", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz1d34", "[", 
         RowBox[{"y", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz1d24", "[", 
         RowBox[{"y", ",", "z", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz1d23", "[", 
         RowBox[{"y", ",", "w", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz2d34", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz2d14", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz2d13", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz3d12", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz3d14", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz3d24", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz4d12", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz4d13", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz4d23", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz1", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz2", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz3", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"w", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcz4", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d4comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd1234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd134", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd123", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd12d34", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}], ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd23", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd24", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd34", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd14", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd13", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0funcd12", "[", 
         RowBox[{"x", ",", "z", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D0func0", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661809992405385*^9, 3.661809992778615*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"empirically", ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"ony", " ", "d0", " ", "and", " ", "d00", " ", "appear"}], " ", 
      "&"}], " ", "only", " ", "z", " ", "is", " ", "ever", " ", "0"}]}], " ",
    "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "shortening", " ", "to", " ", "the", " ", "version", " ", "below", " ", 
    "for", " ", "speed"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"D0funccomp", "[", 
    RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Epsilon]12", "=", 
        RowBox[{"y", "-", "x"}]}], ",", 
       RowBox[{"S12", "=", 
        RowBox[{"x", "+", "y"}]}], ",", 
       RowBox[{"\[Epsilon]13", "=", 
        RowBox[{"z", "-", "x"}]}], ",", 
       RowBox[{"S13", "=", 
        RowBox[{"x", "+", "z"}]}], ",", 
       RowBox[{"\[Epsilon]23", "=", 
        RowBox[{"z", "-", "y"}]}], ",", 
       RowBox[{"S23", "=", 
        RowBox[{"y", "+", "z"}]}], ",", 
       RowBox[{"\[Epsilon]14", "=", 
        RowBox[{"w", "-", "x"}]}], ",", 
       RowBox[{"S14", "=", 
        RowBox[{"x", "+", "w"}]}], ",", 
       RowBox[{"\[Epsilon]34", "=", 
        RowBox[{"w", "-", "z"}]}], ",", 
       RowBox[{"S34", "=", 
        RowBox[{"w", "+", "z"}]}], ",", 
       RowBox[{"\[Epsilon]24", "=", 
        RowBox[{"w", "-", "y"}]}], ",", 
       RowBox[{"S24", "=", 
        RowBox[{"y", "+", "w"}]}], ",", 
       RowBox[{"dzcomp", "=", "0.001"}], ",", 
       RowBox[{"d4comp", "=", "0.06"}], ",", 
       RowBox[{"d3comp", "=", "0.03"}], ",", 
       RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ReleaseHold", "[", 
      RowBox[{"Which", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
         "S12", "*", "S13", "*", "S14", "*", "S24", "*", "S23", "*", "S34"}], 
         "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"z", "\[Equal]", "0"}], "&&", 
         RowBox[{
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
           RowBox[{"Abs", "[", 
            RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
           RowBox[{"Abs", "[", 
            RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcz3d124", "[", 
          RowBox[{"x", ",", "z", ",", 
           RowBox[{"y", "-", "x"}], ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"z", "\[Equal]", "0"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcz3d12", "[", 
          RowBox[{"x", ",", "w", ",", 
           RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"z", "\[Equal]", "0"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcz3d14", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"z", "\[Equal]", "0"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcz3d24", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"z", "\[Equal]", "0"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcz3", "[", 
          RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d4comp"}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd1234", "[", 
          RowBox[{"x", ",", 
           RowBox[{"y", "-", "x"}], ",", 
           RowBox[{"z", "-", "x"}], ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd234", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"z", "-", "y"}], ",", 
           RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd134", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"z", "-", "x"}], ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd124", "[", 
          RowBox[{"x", ",", "z", ",", 
           RowBox[{"y", "-", "x"}], ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd123", "[", 
          RowBox[{"x", ",", "w", ",", 
           RowBox[{"y", "-", "x"}], ",", 
           RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd12d34", "[", 
          RowBox[{"x", ",", "z", ",", 
           RowBox[{"y", "-", "x"}], ",", 
           RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd12d34", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"z", "-", "x"}], ",", 
           RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}]}], 
        ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd12d34", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"w", "-", "x"}], ",", 
           RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd23", "[", 
          RowBox[{"x", ",", "y", ",", "w", ",", 
           RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd24", "[", 
          RowBox[{"x", ",", "y", ",", "z", ",", 
           RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd34", "[", 
          RowBox[{"x", ",", "y", ",", "z", ",", 
           RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd14", "[", 
          RowBox[{"x", ",", "y", ",", "z", ",", 
           RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd13", "[", 
          RowBox[{"x", ",", "y", ",", "w", ",", 
           RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0funcd12", "[", 
          RowBox[{"x", ",", "z", ",", "w", ",", 
           RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", 
        "\[IndentingNewLine]", "True", ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{"D0func0", "[", 
          RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}]}], "]"}], 
      "\[IndentingNewLine]", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6615513255602503`*^9, 3.661551396181416*^9}, {
  3.6615516242048817`*^9, 3.661551627226475*^9}, {3.66155166237234*^9, 
  3.661551674863545*^9}, {3.661551763483233*^9, 3.661551764247327*^9}, {
  3.6615519803904867`*^9, 3.6615519829346113`*^9}, {3.66163809331047*^9, 
  3.661638095675194*^9}, {3.6616621176075163`*^9, 3.661662169490568*^9}, {
  3.661662485022064*^9, 3.661662493812602*^9}, {3.661706924878146*^9, 
  3.661706925533955*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D00temp", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"m1", "\[Rule]", "x"}], ",", 
    RowBox[{"m2", "\[Rule]", "y"}], ",", 
    RowBox[{"m3", "\[Rule]", "z"}], ",", 
    RowBox[{"m4", "\[Rule]", "w"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.661547541658764*^9, 3.661547560917994*^9}, 
   3.66155200400436*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], " ", 
  SuperscriptBox["\[Pi]", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["w", "2"], " ", 
       RowBox[{"Log", "[", "w", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"w", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"w", "-", "z"}], ")"}]}]]}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["x", "2"], " ", 
      RowBox[{"Log", "[", "x", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["y", "2"], " ", 
      RowBox[{"Log", "[", "y", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "y"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"y", "-", "z"}], ")"}]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["z", "2"], " ", 
      RowBox[{"Log", "[", "z", "]"}]}], 
     RowBox[{"64", " ", 
      SuperscriptBox["\[Pi]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"x", "-", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "w"}], "+", "z"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.661547543291243*^9, 3.661547561710485*^9}, 
   3.661552008986907*^9, 3.66161244444197*^9, 3.661616036718288*^9, 
   3.661638248724729*^9, 3.661706709200403*^9, 3.6618103610971413`*^9, 
   3.674840312010541*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"D00func0", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "16"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["w", "2"], " ", 
        RowBox[{"Log", "[", "w", "]"}]}], 
       RowBox[{"64", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"w", "-", "y"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"w", "-", "z"}], ")"}]}]]}], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "2"], " ", 
       RowBox[{"Log", "[", "x", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "x"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["y", "2"], " ", 
       RowBox[{"Log", "[", "y", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"y", "-", "z"}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], " ", 
       RowBox[{"Log", "[", "z", "]"}]}], 
      RowBox[{"64", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"x", "-", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "w"}], "+", "z"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "y"}], "+", "z"}], ")"}]}]]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.661547580865869*^9, 3.661547594968193*^9}, {
  3.6615520063821793`*^9, 3.6615520157132807`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"FFF", "=", "D00func0"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"2", ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"3", " ", "&"}], " ", "4", " ", "zeros"}], " ", "\[Rule]", " ", 
     "0"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "trip", " ", "degeneracy"}],
    " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz1d234", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"\[Epsilon]1", ",", "y", ",", 
             RowBox[{"y", "+", "\[Delta]1"}], ",", 
             RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz2d134", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"x", ",", "\[Epsilon]1", ",", 
             RowBox[{"x", "+", "\[Delta]1"}], ",", 
             RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz3d124", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"Limit", "[", 
          RowBox[{
           RowBox[{"FFF", "[", 
            RowBox[{"x", ",", 
             RowBox[{"x", "+", "\[Delta]1"}], ",", "\[Epsilon]1", ",", 
             RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
           RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcz4d123", "[", 
    RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"Limit", "[", 
           RowBox[{
            RowBox[{"FFF", "[", 
             RowBox[{"x", ",", 
              RowBox[{"x", "+", "\[Delta]1"}], ",", 
              RowBox[{"x", "+", "\[Delta]2"}], ",", "\[Epsilon]1"}], "]"}], 
            ",", 
            RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "zero"}], " ", "&"}], " ", "a", " ", "degeneracy"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz1d34", "[", 
   RowBox[{"y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", 
          RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz1d24", "[", 
   RowBox[{"y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz1d23", "[", 
   RowBox[{"y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"\[Epsilon]1", ",", "y", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz2d34", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", 
          RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz2d14", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz2d13", "[", 
   RowBox[{"x_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz3d14", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz3d24", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz3d12", "[", 
   RowBox[{"x_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1", ",", "w"}], 
         "]"}], ",", 
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz4d23", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"y", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
        
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz4d13", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Limit", "[", 
       RowBox[{
        RowBox[{"FFF", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"x", "+", "\[Epsilon]"}], ",", "\[Epsilon]1"}], "]"}], ",", 
        
        RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcz4d12", "[", 
    RowBox[{"x_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Limit", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Epsilon]"}], ",", "z", ",", "\[Epsilon]1"}], 
          "]"}], ",", 
         RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"a", " ", "zero"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz1", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"\[Epsilon]1", ",", "y", ",", "z", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz2", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"x", ",", "\[Epsilon]1", ",", "z", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcz3", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Limit", "[", 
    RowBox[{
     RowBox[{"FFF", "[", 
      RowBox[{"x", ",", "y", ",", "\[Epsilon]1", ",", "w"}], "]"}], ",", 
     RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcz4", "[", 
    RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Limit", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", "\[Epsilon]1"}], "]"}], ",", 
      RowBox[{"\[Epsilon]1", "\[Rule]", "0"}]}], "]"}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"quad", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcd1234", "[", 
    RowBox[{"x_", ",", "\[Delta]1_", ",", "\[Delta]2_", ",", "\[Delta]3_"}], 
    "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"Normal", "[", 
           RowBox[{"Series", "[", 
            RowBox[{
             RowBox[{"FFF", "[", 
              RowBox[{"x", ",", 
               RowBox[{"x", "+", "\[Delta]1"}], ",", 
               RowBox[{"x", "+", "\[Delta]2"}], ",", 
               RowBox[{"x", "+", "\[Delta]3"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]3", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"trip", " ", "degeneracy"}], " ", "-", " ", 
    RowBox[{
    "may", " ", "want", " ", "to", " ", "minize", " ", "this", " ", "error", 
     " ", "by", " ", "choosing", " ", "judiciously"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd234", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"y", "+", "\[Delta]1"}], ",", 
           RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd134", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", 
           RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd124", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", "z", ",", 
           RowBox[{"x", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcd123", "[", 
    RowBox[{"x_", ",", "w_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"x", ",", 
            RowBox[{"x", "+", "\[Delta]1"}], ",", 
            RowBox[{"x", "+", "\[Delta]2"}], ",", "w"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"double", " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd12d34", "[", 
   RowBox[{"x_", ",", "z_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", "z", ",", 
           RowBox[{"z", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd13d24", "[", 
   RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"Series", "[", 
        RowBox[{
         RowBox[{"FFF", "[", 
          RowBox[{"x", ",", "y", ",", 
           RowBox[{"x", "+", "\[Delta]1"}], ",", 
           RowBox[{"y", "+", "\[Delta]2"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"D00funcd14d23", "[", 
    RowBox[{"x_", ",", "y_", ",", "\[Delta]1_", ",", "\[Delta]2_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{
       RowBox[{"Normal", "[", 
        RowBox[{"Series", "[", 
         RowBox[{
          RowBox[{"FFF", "[", 
           RowBox[{"x", ",", "y", ",", 
            RowBox[{"y", "+", "\[Delta]2"}], ",", 
            RowBox[{"x", "+", "\[Delta]1"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[Delta]1", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Delta]2", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
    "]"}]}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"single", " ", "degeneracy"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd14", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd13", "[", 
   RowBox[{"x_", ",", "y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd12", "[", 
   RowBox[{"x_", ",", "z_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", 
        RowBox[{"x", "+", "\[Epsilon]"}], ",", "z", ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd24", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd23", "[", 
   RowBox[{"x_", ",", "y_", ",", "w_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", 
        RowBox[{"y", "+", "\[Epsilon]"}], ",", "w"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00funcd34", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "\[Epsilon]_"}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"FFF", "[", 
       RowBox[{"x", ",", "y", ",", "z", ",", 
        RowBox[{"z", "+", "\[Epsilon]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Epsilon]", ",", "0", ",", "4"}], "}"}]}], "]"}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJwdyk0ogwEAh/EZilmjxJgZymQ+sg/JambvYbYcpi1mhcKs5eCj1g6cRi0O
O2pzsjCzSKmFtJkirYxlJW1Dopy2fEVKveX9v4en3+WpG58xWJgMBoNHBVkb
TEFSkyHSuf56OC3kdsNAjVgNP6PEIDSSiWEYFh57oO7s3Av1Q1VRKEvEruGN
ofEJGgScV+h+UJJwqrqHdsL+xkpRsjdP2TCi2i+DvnltBbRbjUqYZ1KNQfXu
jwWuZPvXoS01u0V/3tQeFL/vBKH+ZDULD01zH7C1PEDCSVcbM02psT3yIWf5
txYmrdvNcC34LIUL7qIReOT/NEOL684BSwpundAx8E07qutzwSvJywH08StD
kCc103YSkeJ7yqbCeCn8IzztsEUkl8MOgegCLuYsxWC2V6HI12aIuMTcBS/J
r1ADpVPGDcN/YOHXaw==
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"D00funcAcc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]14", "=", 
       RowBox[{"w", "-", "x"}]}], ",", 
      RowBox[{"S14", "=", 
       RowBox[{"x", "+", "w"}]}], ",", 
      RowBox[{"\[Epsilon]34", "=", 
       RowBox[{"w", "-", "z"}]}], ",", 
      RowBox[{"S34", "=", 
       RowBox[{"w", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]24", "=", 
       RowBox[{"w", "-", "y"}]}], ",", 
      RowBox[{"S24", "=", 
       RowBox[{"y", "+", "w"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.01"}], ",", 
      RowBox[{"d4comp", "=", "0.06"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
        "S12", "*", "S13", "*", "S14", "*", "S24", "*", "S23", "*", "S34"}], 
        "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz1d234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz2d134", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz4d123", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz1d34", "[", 
         RowBox[{"y", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz1d24", "[", 
         RowBox[{"y", ",", "z", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"x", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz1d23", "[", 
         RowBox[{"y", ",", "w", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz2d34", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz2d14", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz2d13", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d12", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d14", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d24", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz4d12", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz4d13", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"w", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz4d23", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"x", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz1", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"y", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz2", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"w", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz4", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d4comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd1234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd134", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd123", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}], ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd23", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd24", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd34", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd14", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd13", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12", "[", 
         RowBox[{"x", ",", "z", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00func0", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661706752136304*^9, 3.661706887787922*^9}, 
   3.661810003589879*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"D00funccomp", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Epsilon]12", "=", 
       RowBox[{"y", "-", "x"}]}], ",", 
      RowBox[{"S12", "=", 
       RowBox[{"x", "+", "y"}]}], ",", 
      RowBox[{"\[Epsilon]13", "=", 
       RowBox[{"z", "-", "x"}]}], ",", 
      RowBox[{"S13", "=", 
       RowBox[{"x", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]23", "=", 
       RowBox[{"z", "-", "y"}]}], ",", 
      RowBox[{"S23", "=", 
       RowBox[{"y", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]14", "=", 
       RowBox[{"w", "-", "x"}]}], ",", 
      RowBox[{"S14", "=", 
       RowBox[{"x", "+", "w"}]}], ",", 
      RowBox[{"\[Epsilon]34", "=", 
       RowBox[{"w", "-", "z"}]}], ",", 
      RowBox[{"S34", "=", 
       RowBox[{"w", "+", "z"}]}], ",", 
      RowBox[{"\[Epsilon]24", "=", 
       RowBox[{"w", "-", "y"}]}], ",", 
      RowBox[{"S24", "=", 
       RowBox[{"y", "+", "w"}]}], ",", 
      RowBox[{"dzcomp", "=", "0.001"}], ",", 
      RowBox[{"d4comp", "=", "0.06"}], ",", 
      RowBox[{"d3comp", "=", "0.03"}], ",", 
      RowBox[{"d2comp", "=", "0.01"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ReleaseHold", "[", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
        "S12", "*", "S13", "*", "S14", "*", "S24", "*", "S23", "*", "S34"}], 
        "\[Equal]", "0"}], ",", "0", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}]}], 
       ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d12", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d14", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"z", "\[Equal]", "0"}], "&&", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "dzcomp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3d24", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"z", "\[Equal]", "0"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcz3", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}]}], "<", "d4comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd1234", "[", 
         RowBox[{"x", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd234", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "y"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd134", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd124", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}]}], "<", "d3comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd123", "[", 
         RowBox[{"x", ",", "w", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "z", ",", 
          RowBox[{"y", "-", "x"}], ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"z", "-", "x"}], ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], "&&", 
        
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}]}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12d34", "[", 
         RowBox[{"x", ",", "y", ",", 
          RowBox[{"w", "-", "x"}], ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]23", "/", "S23"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd23", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]24", "/", "S24"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd24", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "y"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]34", "/", "S34"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd34", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "z"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]14", "/", "S14"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd14", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", 
          RowBox[{"w", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]13", "/", "S13"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd13", "[", 
         RowBox[{"x", ",", "y", ",", "w", ",", 
          RowBox[{"z", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"\[Epsilon]12", "/", "S12"}], "]"}], "<", "d2comp"}], ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00funcd12", "[", 
         RowBox[{"x", ",", "z", ",", "w", ",", 
          RowBox[{"y", "-", "x"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]",
        "True", ",", 
       RowBox[{"Evaluate", "[", 
        RowBox[{"D00func0", "[", 
         RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "]"}], "]"}]}], "]"}], 
     "\[IndentingNewLine]", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661553265666627*^9, 3.661553276361918*^9}, {
  3.661638131014235*^9, 3.661638132458495*^9}, {3.661694358975754*^9, 
  3.661694406678295*^9}, {3.6617069401041203`*^9, 3.661706940551198*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D00func", "[", 
  RowBox[{"x", ",", "y", ",", "z", ",", "x"}], "]"}]], "Input",
 CellChangeTimes->{{3.661694428701563*^9, 3.661694463452059*^9}}],

Cell[BoxData[
 RowBox[{"D00func", "[", 
  RowBox[{"x", ",", "y", ",", "z", ",", "x"}], "]"}]], "Output",
 CellChangeTimes->{{3.661694434700678*^9, 3.661694464994609*^9}, 
   3.661706719878572*^9, 3.661810371211955*^9, 3.674840322040689*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparison", "Subsubsection",
 CellChangeTimes->{{3.661551046798184*^9, 3.661551048373403*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"D0func", "[", 
  RowBox[{
   SuperscriptBox["1000.0", "2"], ",", 
   SuperscriptBox["1000.0", "2"], ",", 
   SuperscriptBox["1000.0", "2"], ",", 
   SuperscriptBox["1000.0", "2"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"D0func", "[", 
   RowBox[{
    SuperscriptBox["1000.0", "2"], ",", 
    SuperscriptBox["1005.0", "2"], ",", 
    SuperscriptBox["1010.0", "2"], ",", 
    SuperscriptBox["1030.0", "2"]}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"AbsoluteTiming", "[", 
  RowBox[{"D0func0", "[", 
   RowBox[{
    SuperscriptBox["1000.0", "2"], ",", 
    SuperscriptBox["1005.0", "2"], ",", 
    SuperscriptBox["1010.0", "2"], ",", 
    SuperscriptBox["1030.0", "2"]}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.661551079956415*^9, 3.661551141075513*^9}}],

Cell[BoxData["1.6666666666666665`*^-13"], "Output",
 CellChangeTimes->{{3.661551095154724*^9, 3.661551141545817*^9}, {
  3.661551401817882*^9, 3.661551406053371*^9}, {3.661551713567713*^9, 
  3.661551715872292*^9}, {3.661551754381364*^9, 3.661551782370623*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00046500000000000002702699175571865453`2.688052866169578", ",", 
   "1.5842344266945697`*^-13"}], "}"}]], "Output",
 CellChangeTimes->{{3.661551095154724*^9, 3.661551141545817*^9}, {
  3.661551401817882*^9, 3.661551406053371*^9}, {3.661551713567713*^9, 
  3.661551715872292*^9}, {3.661551754381364*^9, 3.66155178237442*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00003699999999999999811392190074421649`1.588801637346619", ",", 
   "1.5938108453300194`*^-13"}], "}"}]], "Output",
 CellChangeTimes->{{3.661551095154724*^9, 3.661551141545817*^9}, {
  3.661551401817882*^9, 3.661551406053371*^9}, {3.661551713567713*^9, 
  3.661551715872292*^9}, {3.661551754381364*^9, 3.6615517823797283`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0func", "[", 
  RowBox[{
   SuperscriptBox["1002.00", "2"], ",", 
   SuperscriptBox["1000.", "2"], ",", 
   SuperscriptBox["1000.01", "2"], ",", 
   SuperscriptBox["1001.001", "2"]}], "]"}]], "Input",
 CellChangeTimes->{{3.661551643674717*^9, 3.661551648808689*^9}}],

Cell[BoxData["1.6616582170083928`*^-13"], "Output",
 CellChangeTimes->{3.661551649645916*^9, 3.6615516938026037`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"D0func0", "[", 
      RowBox[{
       SuperscriptBox["1202.00", "2"], ",", 
       SuperscriptBox["1000.000", "2"], ",", 
       SuperscriptBox["1201.99", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}], ",", 
     RowBox[{"D0func", "[", 
      RowBox[{
       SuperscriptBox["1202.00", "2"], ",", 
       SuperscriptBox["1000.000", "2"], ",", 
       SuperscriptBox["1201.99", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.999", ",", "1000.001"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.661551463584682*^9, 3.661551540224175*^9}, {
   3.6615515721091948`*^9, 3.661551574381608*^9}, 3.6615516883836813`*^9, {
   3.6615518399325247`*^9, 3.6615519664709253`*^9}, {3.661553402126563*^9, 
   3.661553432000222*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"D00func0", "[", 
      RowBox[{
       SuperscriptBox["1202.00", "2"], ",", 
       SuperscriptBox["1000.000", "2"], ",", 
       SuperscriptBox["1201.99", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}], ",", 
     RowBox[{"D00func", "[", 
      RowBox[{
       SuperscriptBox["1202.00", "2"], ",", 
       SuperscriptBox["1000.000", "2"], ",", 
       SuperscriptBox["1201.99", "2"], ",", 
       SuperscriptBox["x", "2"]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "999.999", ",", "1000.001"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6615532892803497`*^9, 3.661553326792281*^9}, {
  3.661553437894711*^9, 3.661553440568079*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Compilation", "Subsubsection",
 CellChangeTimes->{{3.661611318949438*^9, 3.661611320451943*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"D0comp1", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D0funccomp", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0comp2", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D0funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.661706949727455*^9, 3.661706956066483*^9}, 
   3.661707019138748*^9, {3.661707236616116*^9, 3.6617072519910793`*^9}, 
   3.661810298077881*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"AbsoluteTiming", "[", 
    RowBox[{"D0comp2", "[", 
     RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
    "]"}], "\[IndentingNewLine]", 
   RowBox[{"AbsoluteTiming", "[", 
    RowBox[{"D0comp1", "[", 
     RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6617069666536837`*^9, 3.661706998880266*^9}, {
  3.661707136389679*^9, 3.661707138051536*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00258299999999999979269360572686764499`3.4327243194529413", ",", 
   "1.22756486922655`*^-7"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706999107873*^9, 3.661707037413053*^9}, {
   3.661707133815052*^9, 3.661707141810066*^9}, {3.6617072539745617`*^9, 
   3.6617072647107897`*^9}, {3.6618104080376177`*^9, 3.661810409336117*^9}, 
   3.67484033992069*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00180299999999999991523447206986929814`3.2765956400020264", ",", 
   "1.22756486922655`*^-7"}], "}"}]], "Output",
 CellChangeTimes->{{3.661706999107873*^9, 3.661707037413053*^9}, {
   3.661707133815052*^9, 3.661707141810066*^9}, {3.6617072539745617`*^9, 
   3.6617072647107897`*^9}, {3.6618104080376177`*^9, 3.661810409336117*^9}, 
   3.674840339922278*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"D0comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D0funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00comp", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D00funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.661611340858354*^9, 3.661611351796791*^9}, {
  3.661707272049865*^9, 3.661707280063137*^9}, {3.661810291568474*^9, 
  3.6618102948006763`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"AbsoluteTiming", "[", 
    RowBox[{"D0comp", "[", 
     RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
    "]"}], "\[IndentingNewLine]", 
   RowBox[{"AbsoluteTiming", "[", 
    RowBox[{"D00comp", "[", 
     RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6617072881509933`*^9, 3.661707291983307*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00282799999999999978478326667641340464`3.4720793184044862", ",", 
   "1.22756486922655`*^-7"}], "}"}]], "Output",
 CellChangeTimes->{{3.661707292301564*^9, 3.661707299816423*^9}, {
   3.661707356521441*^9, 3.661707363317051*^9}, {3.661810411936532*^9, 
   3.661810413471779*^9}, 3.674840340128039*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00228800000000000010313971898767704261`3.3800559334006106", ",", 
   RowBox[{"-", "0.00007146500436219238`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661707292301564*^9, 3.661707299816423*^9}, {
   3.661707356521441*^9, 3.661707363317051*^9}, {3.661810411936532*^9, 
   3.661810413471779*^9}, 3.6748403401294394`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"D0compA", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D0funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D0compB", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"q", ",", "_Complex"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "_Complex"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Complex"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"z", ",", "_Complex"}], "}"}]}], "}"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"D0funcAcc", "[", 
       RowBox[{"q", ",", "x", ",", "y", ",", "z"}], "]"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.661707383212967*^9, 3.661707425486349*^9}, {
  3.6618103357126293`*^9, 3.661810337184182*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Timing", "[", 
  RowBox[{"D0compA", "[", 
   RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "[", 
  RowBox[{"D0compB", "[", 
   RowBox[{"1000.", ",", "1230.", ",", "1233.", ",", "1208."}], "]"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.6617074103074083`*^9, 3.661707440739223*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00244800000000000008912315330178444128`3.4094113267531476", ",", 
   "1.22756486922655`*^-7"}], "}"}]], "Output",
 CellChangeTimes->{{3.661707414979459*^9, 3.661707448321159*^9}, {
   3.661810416554364*^9, 3.6618104180861883`*^9}, 3.6748403403191757`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00409699999999999994737542863276757998`3.6330658772327666", ",", 
   RowBox[{"1.22756486922655`*^-7", "+", 
    RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.661707414979459*^9, 3.661707448321159*^9}, {
   3.661810416554364*^9, 3.6618104180861883`*^9}, 3.674840340320876*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"D0FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "D0func0", ",", "D0funcz1d234", ",", "D0funcz2d134", ",", "D0funcz3d124", 
     ",", "D0funcz4d123", ",", "D0funcz1d34", ",", "D0funcz1d24", ",", 
     "D0funcz1d23", ",", "D0funcz2d34", ",", "D0funcz2d14", ",", 
     "D0funcz2d13", ",", "D0funcz3d14", ",", "D0funcz3d24", ",", 
     "D0funcz3d12", ",", "D0funcz4d23", ",", "D0funcz4d13", ",", 
     "D0funcz4d12", ",", "D0funcz1", ",", "D0funcz2", ",", "D0funcz3", ",", 
     "D0funcz4", ",", "D0funcd1234", ",", "D0funcd234", ",", "D0funcd134", 
     ",", "D0funcd124", ",", "D0funcd123", ",", "D0funcd12d34", ",", 
     "D0funcd13d24", ",", "D0funcd14d23", ",", "D0funcd14", ",", "D0funcd13", 
     ",", "D0funcd12", ",", "D0funcd24", ",", "D0funcd23", ",", "D0funcd34", 
     ",", "D0funcAcc"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"D00FunctionList", "=", 
   RowBox[{"{", 
    RowBox[{
    "D00func0", ",", "D00funcz1d234", ",", "D00funcz2d134", ",", 
     "D00funcz3d124", ",", "D00funcz4d123", ",", "D00funcz1d34", ",", 
     "D00funcz1d24", ",", "D00funcz1d23", ",", "D00funcz2d34", ",", 
     "D00funcz2d14", ",", "D00funcz2d13", ",", "D00funcz3d14", ",", 
     "D00funcz3d24", ",", "D00funcz3d12", ",", "D00funcz4d23", ",", 
     "D00funcz4d13", ",", "D00funcz4d12", ",", "D00funcz1", ",", "D00funcz2", 
     ",", "D00funcz3", ",", "D00funcz4", ",", "D00funcd1234", ",", 
     "D00funcd234", ",", "D00funcd134", ",", "D00funcd124", ",", 
     "D00funcd123", ",", "D00funcd12d34", ",", "D00funcd13d24", ",", 
     "D00funcd14d23", ",", "D00funcd14", ",", "D00funcd13", ",", "D00funcd12",
      ",", "D00funcd24", ",", "D00funcd23", ",", "D00funcd34", ",", 
     "D00funcAcc"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DFunctionList", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"D0FunctionList", ",", "D00FunctionList"}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.661612392966733*^9, 3.661612525572176*^9}, {
  3.6616125692359047`*^9, 3.661612588402534*^9}, {3.661612622662608*^9, 
  3.661612698142272*^9}, {3.6616151903796577`*^9, 3.66161519423361*^9}, {
  3.661810023876699*^9, 3.66181002682237*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.66161256417045*^9, 3.661612567854602*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write everything out to file for future use", "Subsection",
 CellChangeTimes->{{3.612800564097028*^9, 3.612800572416815*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"CompilationList", "=", 
    RowBox[{"{", 
     RowBox[{
     "B00comp", ",", "B01comp", ",", "B10comp", ",", "B11comp", ",", "C0comp",
       ",", "C1comp", ",", "C2comp", ",", "C00comp", ",", "C11comp", ",", 
      "C12comp", ",", "C22comp", ",", "D0comp", ",", "D00comp"}], "}"}]}], 
   ";"}], "*)"}]], "Input",
 CellChangeTimes->{{3.661615175341279*^9, 3.6616152038938932`*^9}, {
  3.6616152589583397`*^9, 3.661615273177166*^9}, {3.661617001763184*^9, 
  3.661617003618496*^9}, {3.6616172433617773`*^9, 3.6616172451224537`*^9}, {
  3.674840321325963*^9, 3.674840331802285*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"CurDir", "=", 
    RowBox[{"NotebookDirectory", "[", "]"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"OutPathName", "=", 
   RowBox[{"CurDir", "<>", "\"\<LoopIntegralsAcc.m\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DefinitionList", "[", 
   RowBox[{"listname_", ",", "outfilename_"}], "]"}], ":=", 
  RowBox[{"Append", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Definition", "[", 
       RowBox[{"Evaluate", "[", "X", "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"X", ",", "listname"}], "}"}]}], "]"}], ",", "outfilename"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DefinitionList", "[", "listname_", "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Definition", "[", 
     RowBox[{"Evaluate", "[", "X", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"X", ",", "listname"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{
   RowBox[{"CompileWrite", "[", "listname_", "]"}], ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Definition", "[", "X", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"X", ",", "listname"}], "}"}]}], "]"}]}], 
  "\n"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.661612779630525*^9, 3.661612810577746*^9}, {
   3.661614090270157*^9, 3.661614093604024*^9}, {3.661614286291396*^9, 
   3.661614290734255*^9}, {3.661614617457458*^9, 3.6616146188898983`*^9}, {
   3.6616146932611628`*^9, 3.66161480789075*^9}, {3.6616148541695747`*^9, 
   3.661614869632012*^9}, {3.6616149087531157`*^9, 3.6616149121802893`*^9}, {
   3.6616150209306192`*^9, 3.661615035589333*^9}, {3.661615090796633*^9, 
   3.661615092057474*^9}, {3.661615131063925*^9, 3.661615132478417*^9}, {
   3.661615292583774*^9, 3.661615293862192*^9}, {3.661615715617824*^9, 
   3.661615721473666*^9}, 3.661615921466167*^9, {3.661616700007165*^9, 
   3.661616707199842*^9}, {3.661617286468577*^9, 3.661617315611866*^9}, 
   3.661810473956184*^9, {3.6748403182457438`*^9, 3.674840357772937*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\"\<(* Loop Integral Library *)\>\"", " ", ">>", 
   RowBox[{"CurDir", "<>", "\"\<LoopIntegralsAcc.m\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"X", ">>>", 
     RowBox[{"CurDir", "<>", "\"\<LoopIntegralsAcc.m\>\""}]}], ",", 
    RowBox[{"{", 
     RowBox[{"X", ",", 
      RowBox[{"DefinitionList", "[", "BFunctionList", "]"}]}], "}"}]}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"X", ">>>", 
     RowBox[{"CurDir", "<>", "\"\<LoopIntegralsAcc.m\>\""}]}], ",", 
    RowBox[{"{", 
     RowBox[{"X", ",", 
      RowBox[{"DefinitionList", "[", "CFunctionList", "]"}]}], "}"}]}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"X", ">>>", 
     RowBox[{"CurDir", "<>", "\"\<LoopIntegralsAcc.m\>\""}]}], ",", 
    RowBox[{"{", 
     RowBox[{"X", ",", 
      RowBox[{"DefinitionList", "[", "DFunctionList", "]"}]}], "}"}]}], "]"}],
   ";"}]}], "Input",
 CellChangeTimes->{{3.661616730788931*^9, 3.661616817713344*^9}, {
   3.66161687514694*^9, 3.661616954679162*^9}, {3.661617049930297*^9, 
   3.661617104735991*^9}, {3.66161717989392*^9, 3.661617184429297*^9}, {
   3.6616172345220823`*^9, 3.6616172517904167`*^9}, {3.6616173276276503`*^9, 
   3.661617329664473*^9}, {3.6616175597604523`*^9, 3.66161756144911*^9}, {
   3.661617592657712*^9, 3.661617597966814*^9}, {3.6617076454435368`*^9, 
   3.661707646756947*^9}, {3.661707835460449*^9, 3.661707837531529*^9}, 
   3.661707900126522*^9, {3.661810458922778*^9, 3.661810467042523*^9}}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1081, 619},
WindowMargins->{{54, Automatic}, {46, Automatic}},
ShowSelection->True,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 27, \
2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 98, 1, 80, "Section"],
Cell[CellGroupData[{
Cell[702, 27, 102, 1, 44, "Subsection"],
Cell[807, 30, 233, 5, 28, "Input"],
Cell[CellGroupData[{
Cell[1065, 39, 285, 5, 28, "Input"],
Cell[1353, 46, 397, 9, 41, "Message"]
}, Open  ]],
Cell[1765, 58, 771, 16, 46, "Input"],
Cell[2539, 76, 817, 22, 28, "Input"],
Cell[3359, 100, 2436, 56, 165, "Input"],
Cell[5798, 158, 3102, 79, 233, "Input"],
Cell[CellGroupData[{
Cell[8925, 241, 1623, 38, 101, "Input"],
Cell[10551, 281, 6169, 107, 35, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16757, 393, 958, 29, 36, "Input"],
Cell[17718, 424, 3913, 70, 70, "Output"]
}, Open  ]],
Cell[21646, 497, 1804, 52, 99, "Input"],
Cell[23453, 551, 3155, 73, 204, "Input"],
Cell[26611, 626, 217, 5, 28, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26865, 636, 312, 6, 44, "Subsection"],
Cell[CellGroupData[{
Cell[27202, 646, 104, 1, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[27331, 651, 2238, 54, 91, "Input"],
Cell[29572, 707, 6271, 107, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35880, 819, 680, 15, 28, "Input"],
Cell[36563, 836, 2385, 70, 97, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38985, 911, 2946, 71, 91, "Input"],
Cell[41934, 984, 6418, 114, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48389, 1103, 656, 16, 28, "Input"],
Cell[49048, 1121, 7706, 156, 97, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56791, 1282, 789, 20, 51, "Input"],
Cell[57583, 1304, 1772, 47, 75, "Output"],
Cell[59358, 1353, 831, 19, 53, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60226, 1377, 1184, 32, 110, "Input"],
Cell[61413, 1411, 557, 10, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62007, 1426, 372, 9, 51, "Input"],
Cell[62382, 1437, 2086, 60, 95, "Output"],
Cell[64471, 1499, 867, 23, 53, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[65375, 1527, 158, 3, 28, "Input"],
Cell[65536, 1532, 328, 6, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[65901, 1543, 1997, 50, 97, "Input"],
Cell[67901, 1595, 517, 8, 28, "Output"],
Cell[68421, 1605, 1735, 53, 48, "Output"],
Cell[70159, 1660, 567, 10, 28, "Output"],
Cell[70729, 1672, 567, 10, 28, "Output"],
Cell[71299, 1684, 685, 13, 28, "Output"]
}, Open  ]],
Cell[71999, 1700, 8576, 277, 614, "Input"],
Cell[80578, 1979, 5986, 147, 488, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86601, 2131, 101, 1, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[86727, 2136, 559, 13, 57, "Input"],
Cell[87289, 2151, 364, 7, 35, "Output"],
Cell[87656, 2160, 345, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88038, 2172, 931, 22, 101, "Input"],
Cell[88972, 2196, 367, 7, 28, "Output"],
Cell[89342, 2205, 369, 7, 28, "Output"],
Cell[89714, 2214, 370, 7, 28, "Output"],
Cell[90087, 2223, 371, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90495, 2235, 375, 10, 52, "Input"],
Cell[90873, 2247, 161, 2, 28, "Output"]
}, Open  ]],
Cell[91049, 2252, 1665, 49, 79, "Input"],
Cell[92717, 2303, 1340, 43, 79, "Input"],
Cell[CellGroupData[{
Cell[94082, 2350, 896, 24, 140, "Input"],
Cell[94981, 2376, 207, 3, 32, "Output"],
Cell[95191, 2381, 209, 3, 32, "Output"],
Cell[95403, 2386, 209, 3, 32, "Output"],
Cell[95615, 2391, 207, 3, 32, "Output"],
Cell[95825, 2396, 207, 3, 32, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[96081, 2405, 104, 1, 29, "Subsubsection"],
Cell[96188, 2408, 1933, 59, 80, "Input"],
Cell[CellGroupData[{
Cell[98146, 2471, 749, 15, 80, "Input"],
Cell[98898, 2488, 483, 9, 28, "Output"],
Cell[99384, 2499, 471, 9, 35, "Output"],
Cell[99858, 2510, 467, 9, 35, "Output"],
Cell[100328, 2521, 485, 8, 35, "Output"]
}, Open  ]],
Cell[100828, 2532, 653, 13, 46, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[101530, 2551, 294, 5, 69, "Subsection"],
Cell[CellGroupData[{
Cell[101849, 2560, 104, 1, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[101978, 2565, 2513, 66, 114, "Input"],
Cell[104494, 2633, 5660, 99, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110191, 2737, 395, 7, 28, "Input"],
Cell[110589, 2746, 8002, 168, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118628, 2919, 5167, 133, 178, "Input"],
Cell[123798, 3054, 6580, 126, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[130415, 3185, 728, 18, 46, "Input"],
Cell[131146, 3205, 9702, 219, 70, "Output"],
Cell[140851, 3426, 9677, 217, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150565, 3648, 7890, 202, 240, "Input"],
Cell[158458, 3852, 9494, 216, 70, "Output"]
}, Open  ]],
Cell[167967, 4071, 247, 5, 28, "Input"],
Cell[168217, 4078, 1142, 32, 80, "Input"],
Cell[CellGroupData[{
Cell[169384, 4114, 495, 15, 51, "Input"],
Cell[169882, 4131, 1504, 43, 70, "Output"],
Cell[171389, 4176, 563, 15, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[171989, 4196, 374, 9, 28, "Input"],
Cell[172366, 4207, 3518, 106, 70, "Output"]
}, Open  ]],
Cell[175899, 4316, 3663, 108, 123, "Input"],
Cell[179565, 4426, 8057, 216, 335, "Input"],
Cell[187625, 4644, 14753, 360, 777, "Input"],
Cell[202381, 5006, 298, 5, 28, "Input"],
Cell[CellGroupData[{
Cell[202704, 5015, 397, 9, 28, "Input"],
Cell[203104, 5026, 5270, 157, 70, "Output"]
}, Open  ]],
Cell[208389, 5186, 5493, 158, 218, "Input"],
Cell[213885, 5346, 7747, 212, 335, "Input"],
Cell[221635, 5560, 15032, 370, 794, "Input"],
Cell[CellGroupData[{
Cell[236692, 5934, 385, 7, 63, "Input"],
Cell[237080, 5943, 456, 12, 70, "Output"],
Cell[237539, 5957, 379, 10, 70, "Output"],
Cell[237921, 5969, 390, 9, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[238348, 5983, 425, 10, 28, "Input"],
Cell[238776, 5995, 5280, 156, 70, "Output"]
}, Open  ]],
Cell[244071, 6154, 5538, 159, 176, "Input"],
Cell[249612, 6315, 7774, 212, 335, "Input"],
Cell[257389, 6529, 15004, 369, 794, "Input"],
Cell[CellGroupData[{
Cell[272418, 6902, 385, 7, 63, "Input"],
Cell[272806, 6911, 281, 8, 70, "Output"],
Cell[273090, 6921, 358, 10, 70, "Output"],
Cell[273451, 6933, 294, 7, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[273782, 6945, 587, 13, 28, "Input"],
Cell[274372, 6960, 6198, 190, 70, "Output"]
}, Open  ]],
Cell[280585, 7153, 6456, 191, 210, "Input"],
Cell[287044, 7346, 7811, 213, 335, "Input"],
Cell[294858, 7561, 15256, 374, 811, "Input"],
Cell[CellGroupData[{
Cell[310139, 7939, 445, 9, 80, "Input"],
Cell[310587, 7950, 540, 16, 70, "Output"],
Cell[311130, 7968, 267, 7, 70, "Output"],
Cell[311400, 7977, 542, 16, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311979, 7998, 524, 11, 28, "Input"],
Cell[312506, 8011, 7670, 218, 70, "Output"]
}, Open  ]],
Cell[320191, 8232, 8070, 222, 333, "Input"],
Cell[328264, 8456, 7858, 213, 335, "Input"],
Cell[336125, 8671, 15069, 370, 794, "Input"],
Cell[CellGroupData[{
Cell[351219, 9045, 676, 13, 114, "Input"],
Cell[351898, 9060, 1284, 46, 70, "Output"],
Cell[353185, 9108, 1218, 44, 70, "Output"],
Cell[354406, 9154, 1275, 46, 70, "Output"],
Cell[355684, 9202, 1691, 55, 70, "Output"],
Cell[357378, 9259, 829, 26, 70, "Output"],
Cell[358210, 9287, 1403, 43, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[359650, 9335, 389, 7, 63, "Input"],
Cell[360042, 9344, 356, 9, 70, "Output"],
Cell[360401, 9355, 292, 6, 70, "Output"],
Cell[360696, 9363, 279, 7, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361012, 9375, 574, 12, 28, "Input"],
Cell[361589, 9389, 7216, 210, 70, "Output"]
}, Open  ]],
Cell[368820, 9602, 7630, 214, 264, "Input"],
Cell[376453, 9818, 7910, 214, 335, "Input"],
Cell[384366, 10034, 13596, 333, 709, "Input"],
Cell[CellGroupData[{
Cell[397987, 10371, 794, 17, 114, "Input"],
Cell[398784, 10390, 1710, 62, 70, "Output"],
Cell[400497, 10454, 402, 10, 70, "Output"],
Cell[400902, 10466, 402, 10, 70, "Output"],
Cell[401307, 10478, 1689, 53, 70, "Output"],
Cell[402999, 10533, 401, 10, 70, "Output"],
Cell[403403, 10545, 404, 10, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403844, 10560, 320, 8, 28, "Input"],
Cell[404167, 10570, 7643, 215, 70, "Output"]
}, Open  ]],
Cell[411825, 10788, 8063, 219, 290, "Input"],
Cell[419891, 11009, 7959, 215, 335, "Input"],
Cell[427853, 11226, 15069, 370, 794, "Input"],
Cell[CellGroupData[{
Cell[442947, 11600, 390, 7, 63, "Input"],
Cell[443340, 11609, 237, 7, 70, "Output"],
Cell[443580, 11618, 246, 6, 70, "Output"],
Cell[443829, 11626, 312, 9, 70, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[444190, 11641, 100, 1, 29, "Subsubsection"],
Cell[CellGroupData[{
Cell[444315, 11646, 581, 14, 57, "Input"],
Cell[444899, 11662, 283, 5, 35, "Output"],
Cell[445185, 11669, 289, 6, 35, "Output"]
}, Open  ]],
Cell[445489, 11678, 344, 11, 28, "Input"],
Cell[CellGroupData[{
Cell[445858, 11693, 619, 17, 28, "Input"],
Cell[446480, 11712, 363, 11, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446880, 11728, 279, 6, 35, "Input"],
Cell[447162, 11736, 239, 5, 35, "Output"]
}, Open  ]],
Cell[447416, 11744, 669, 19, 35, "Input"],
Cell[CellGroupData[{
Cell[448110, 11767, 1018, 27, 78, "Input"],
Cell[449131, 11796, 1031, 28, 84, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[450199, 11829, 392, 9, 46, "Input"],
Cell[450594, 11840, 236, 4, 28, "Output"],
Cell[450833, 11846, 236, 4, 28, "Output"]
}, Open  ]],
Cell[451084, 11853, 663, 18, 35, "Input"],
Cell[451750, 11873, 669, 19, 35, "Input"],
Cell[452422, 11894, 661, 18, 35, "Input"],
Cell[CellGroupData[{
Cell[453108, 11916, 317, 9, 52, "Input"],
Cell[453428, 11927, 114, 1, 28, "Output"]
}, Open  ]],
Cell[453557, 11931, 763, 19, 35, "Input"],
Cell[454323, 11952, 72, 1, 30, "Input"],
Cell[454398, 11955, 663, 18, 35, "Input"],
Cell[455064, 11975, 661, 18, 35, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[455762, 11998, 104, 1, 29, "Subsubsection"],
Cell[455869, 12001, 11471, 356, 403, "Input"],
Cell[CellGroupData[{
Cell[467365, 12361, 3648, 89, 403, "Input"],
Cell[471016, 12452, 360, 6, 28, "Output"],
Cell[471379, 12460, 349, 7, 28, "Output"],
Cell[471731, 12469, 348, 7, 28, "Output"],
Cell[472082, 12478, 358, 7, 28, "Output"],
Cell[472443, 12487, 363, 7, 28, "Output"],
Cell[472809, 12496, 361, 7, 35, "Output"],
Cell[473173, 12505, 356, 6, 35, "Output"],
Cell[473532, 12513, 360, 6, 28, "Output"],
Cell[473895, 12521, 348, 7, 28, "Output"],
Cell[474246, 12530, 348, 7, 28, "Output"],
Cell[474597, 12539, 357, 7, 28, "Output"],
Cell[474957, 12548, 363, 7, 28, "Output"],
Cell[475323, 12557, 363, 7, 35, "Output"],
Cell[475689, 12566, 363, 7, 35, "Output"],
Cell[476055, 12575, 349, 7, 35, "Output"],
Cell[476407, 12584, 362, 7, 35, "Output"],
Cell[476772, 12593, 362, 7, 35, "Output"],
Cell[477137, 12602, 362, 7, 35, "Output"],
Cell[477502, 12611, 347, 7, 35, "Output"],
Cell[477852, 12620, 347, 7, 35, "Output"],
Cell[478202, 12629, 347, 7, 35, "Output"]
}, Open  ]],
Cell[478564, 12639, 3752, 77, 284, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[482365, 12722, 227, 4, 44, "Subsection"],
Cell[CellGroupData[{
Cell[482617, 12730, 103, 1, 35, "Subsubsection"],
Cell[482723, 12733, 588, 19, 35, "Input"],
Cell[CellGroupData[{
Cell[483336, 12756, 233, 6, 28, "Input"],
Cell[483572, 12764, 5930, 113, 92, "Output"]
}, Open  ]],
Cell[489517, 12880, 2386, 77, 84, "Input"],
Cell[CellGroupData[{
Cell[491928, 12961, 657, 19, 80, "Input"],
Cell[492588, 12982, 6861, 140, 101, "Output"],
Cell[499452, 13124, 6958, 144, 101, "Output"],
Cell[506413, 13270, 6976, 145, 101, "Output"]
}, Open  ]],
Cell[513404, 13418, 7409, 193, 192, "Input"],
Cell[520816, 13613, 296, 6, 28, "Input"],
Cell[521115, 13621, 1332, 35, 80, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[522484, 13661, 100, 1, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[522609, 13666, 316, 8, 28, "Input"],
Cell[522928, 13676, 1831, 56, 52, "Output"]
}, Open  ]],
Cell[524774, 13735, 1847, 57, 70, "Input"],
Cell[526624, 13794, 75, 2, 28, "Input"],
Cell[CellGroupData[{
Cell[526724, 13800, 651, 18, 28, "Input"],
Cell[527378, 13820, 3134, 92, 147, "Output"]
}, Open  ]],
Cell[530527, 13915, 22997, 632, 828, "Input"],
Cell[553527, 14549, 16216, 392, 709, "Input"],
Cell[569746, 14943, 11617, 292, 488, "Input"],
Cell[CellGroupData[{
Cell[581388, 15239, 342, 9, 28, "Input"],
Cell[581733, 15250, 2020, 63, 54, "Output"]
}, Open  ]],
Cell[583768, 15316, 2070, 64, 72, "Input"],
Cell[585841, 15382, 23032, 632, 828, "Input"],
Cell[608876, 16016, 16278, 393, 709, "Input"],
Cell[625157, 16411, 10454, 251, 454, "Input"],
Cell[CellGroupData[{
Cell[635636, 16666, 169, 3, 28, "Input"],
Cell[635808, 16671, 240, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[636097, 16681, 101, 1, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[636223, 16686, 829, 21, 79, "Input"],
Cell[637055, 16709, 261, 3, 32, "Output"],
Cell[637319, 16714, 369, 7, 35, "Output"],
Cell[637691, 16723, 372, 7, 35, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[638100, 16735, 291, 7, 35, "Input"],
Cell[638394, 16744, 117, 1, 32, "Output"]
}, Open  ]],
Cell[638526, 16748, 884, 22, 35, "Input"],
Cell[639413, 16772, 757, 20, 56, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[640207, 16797, 102, 1, 29, "Subsubsection"],
Cell[640312, 16800, 1328, 39, 46, "Input"],
Cell[CellGroupData[{
Cell[641665, 16843, 510, 12, 63, "Input"],
Cell[642178, 16857, 401, 8, 35, "Output"],
Cell[642582, 16867, 402, 8, 35, "Output"]
}, Open  ]],
Cell[642999, 16878, 1327, 39, 46, "Input"],
Cell[CellGroupData[{
Cell[644351, 16921, 460, 11, 63, "Input"],
Cell[644814, 16934, 346, 7, 35, "Output"],
Cell[645163, 16943, 365, 7, 28, "Output"]
}, Open  ]],
Cell[645543, 16953, 1290, 38, 46, "Input"],
Cell[CellGroupData[{
Cell[646858, 16995, 383, 9, 46, "Input"],
Cell[647244, 17006, 300, 6, 35, "Output"],
Cell[647547, 17014, 357, 7, 35, "Output"]
}, Open  ]],
Cell[647919, 17024, 2208, 40, 199, "Input"],
Cell[650130, 17066, 91, 1, 28, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[650270, 17073, 131, 1, 44, "Subsection"],
Cell[650404, 17076, 633, 13, 46, "Input"],
Cell[651040, 17091, 2104, 50, 131, "Input"],
Cell[653147, 17143, 1638, 39, 80, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
